
bgvar<-function(Data,W,Wex.restr=NULL,saves=5000,burns=5000,plag=2,hyperparm=NULL,Cpu=4,prior="SSVS",save_thin=.1,eigen=FALSE,logfile=FALSE,stats=FALSE,trend=FALSE, trim=NULL,SV="SV.M",ea.weights=NULL,OC.weights=NULL,variable.list=NULL){
  #------------------------------------ Load packages needed -------------------------------------------------------------------#
  #require(compiler)
  require(snowfall)
  require(Matrix)
  require(abind)
  require(stochvol)
  require(mvtnorm)
  require(rARPACK)
  require(GIGrvg)
  #enableJIT(2)
  # user checks ------------------------------------------------------------------------------------------------------------------------------
  if(any(is.na(unlist(Data))) || any(is.na(W)) ){
    stop("The data you have submitted contains NAs. Please check the country data including the weight matrix.")
  }
  s.idx<-which(names(Data)%in% c("EB","OC"))

  if(is.matrix(W)){
    W.aux<-list();W.aux$W<-W;W<-W.aux;rm(W.aux) # convert W into a list
  }

  for(i in 1:length(W)){
    if(length(s.idx)>0){
      if (!nrow(W[[i]])==(length(Data)-length(s.idx))){
        stop("Data and W matrix not of the same dimension.")
       }
      if(!all(names(Data)[-s.idx]%in%rownames(W[[i]]))){
          stop("Please provide the same country names for the Data and W objects.")
      }
      #make sure that W and Data are in the same order
      W[[i]]<-W[[i]][names(Data)[-s.idx],names(Data)[-s.idx]]

      if(!all(names(Data)[-s.idx]%in%rownames(W[[i]]))){
        stop("Please provide the same country names for the Data and W objects.")
      }
    }else{
      if (!nrow(W[[i]])==length(Data)){
        stop("Data and W matrix not of the same dimension.")
      }
      if(!all(names(Data)%in%rownames(W[[i]]))){
        stop("Please provide the same country names for the Data and W objects.")
      }
      #make sure that W and Data are in the same order
      W[[i]]<-W[[i]][names(Data),names(Data)]

      if(!all(names(Data)%in%rownames(W[[i]]))){
        stop("Please provide the same country names for the Data and W objects.")
      }
    }
  }

  if(is.null(names(Data))){
    names(Data)<-paste0(c,1:length(Data))
  }



  # end user checks -------------------------------------------------------------------------------------------------------------------------------------------------------
  if(trend&&prior!="SSVS"){
    trend=FALSE #trend=TRUE only possible with SSVS prior, hence currently defunct
    print("Trend has been set to FALSE (currently only implemented with the SSVS prior)")
  }







  #----------------------------------- get weights------------------------------------#

  gW <- list();Wstorage <- list()
  names <- names(Data)



  xglobal <- getweights(Data=Data,W=W,EA.weights=ea.weights,wex.restr=Wex.restr,OC.weights=OC.weights, variable.list=variable.list)
  exo<-xglobal$exo;exo.countries<-xglobal$exo.countries;endo<-xglobal$endo
  #W1 <- xglobal$W
  gW<-xglobal$gW
  bigx<-xglobal <- xglobal$bigx
  Names <-cN <-  substr(names(Data),1,2)


  #-----------------------------------this part does the stacking---------------------------#
  x <- t(xglobal)
  names(gW)<-Names
  #linear <- cmpfun(BVAR_linear) #Compile BVAR function to speed up MCMC

  # generate logfile if desired
  if(logfile!=FALSE){
    if (is.character(logfile)) {
      sfilename=logfile}
    else {
      sfilename="test.log"
    }
    file.create(sfilename)
    logfile=TRUE
    cat(as.character(Sys.time()),": start estimation of country models... \n",append=TRUE, file=sfilename)  #write one line
  }






  #the prior inclusion probs of a given variable and covariance.

  if(!prior%in% c("SSVS","SIMS","NG")){
    stop("Please selecte one of the following prior options: SSVS, SIMS or NG")
  }

  if(is.null(hyperparm)&& prior=="SIMS"&& is.null(SV)){
    hyperparm<-list(a_i=0.01,b_i=0.01)
    print("No hyperparameters for diagonal elements of SIGMA (Gamma density) chosen, default setting applied: a_i=b_i=.01.")
  }
  if(is.null(hyperparm)&& prior=="SSVS"){
    hyperparm <- list(tau0=0.1,tau1=3,kappa0=0.1,kappa1=7,a_i=0.01,b_i=0.01,p_i=0.5,q_ij=0.5)
    print("You have not selected any hyperparameters for the SSVS prior. Standard setting used; for more details, see bgvar html help files.")
  }
  if(prior=="SSVS"&&!is.null(hyperparm)&&!all(c("tau0","tau1","kappa0","kappa1","a_i","b_i","p_i","q_ij")%in% names(hyperparm))){
    hyperparm <- list(tau0=0.1,tau1=3,kappa0=0.1,kappa1=7,a_i=0.01,b_i=0.01,p_i=0.5,q_ij=0.5)
    print("You have not submitted all hyperparameters needed for the SSVS prior. Standard setting used; for more details, see bgvar html help files.")
  }
  if(is.null(hyperparm)&& prior=="NG"){
    hyperparm <- list(c_tau = 0.01, d_tau = 0.01,e_lambda=1.5,a_i=0.01,b_i=0.01,d_lambda=1, prmean=1,a_start=.6,sample_A=FALSE)
    print("You have not selected hyperparameters for the NG prior. Standard setting used; for more details, see bgvar html help files.")
  }
  if(prior=="NG"&&!is.null(hyperparm)&&!all(c("c_tau","d_tau","e_lambda","a_i","b_i","d_lambda","prmean","a_start","sample_A")%in% names(hyperparm))){
    hyperparm <- list(c_tau = 0.01, d_tau = 0.01,e_lambda=1.5,a_i=0.01,b_i=0.01,d_lambda=1, prmean=1,a_start=.6,sample_A=FALSE)
    print("You have not submitted all hyperparameters needed for the NG prior. Standard setting used; for more details, see bgvar html help files.")
  }

  #----------------------------------- Start parallel apply to estimate the model faster----------------------------------------------#
  if (Cpu>1){
    globalpost <- list()
    stacked.results<-NULL
    sfInit(parallel=TRUE,cpus=Cpu)
    #sfExport(list=list("mlag","bernoulli","BVAR_linear","datahandling","xglobal","gW","cN","saves","burns","plag","hyperparm","prior","save_thin"))
    sfExportAll()
    globalpost <- suppressWarnings(sfLapply(1:length(Names),function(i) BVAR_linear(i,p=plag,gW=gW,xglobal=xglobal,cN,nsave=saves,nburn=burns,hyper=hyperparm,prior=prior,thin=save_thin,trend=trend,sv=SV)))
    sfStop()
  }else{
    globalpost <- list()

    for (jj in 1:length(Names)) globalpost[[jj]] <- BVAR_linear(jj,p=plag,gW=gW,xglobal=xglobal,cN,nsave=saves,nburn=burns,hyper=hyperparm,prior=prior,thin=save_thin,trend=trend,sv=SV)
  }

  #creates the global x matrix of dimension T x K
  VAR0 <- globalpost[[1]]
  x <- t(VAR0$End)
  for (i in 2:length(Names)){
    VAR1 <- globalpost[[i]]
    x <- rbind(x,t(VAR1$End))
  }

  if(!is.null(trim)&!eigen){
    eigen=TRUE
  }
  if(logfile){
    cat(as.character(Sys.time()),"Start stacking \n",append=TRUE, file=sfilename)
  } #write one line
  else{
    cat("Start stacking: \n")
  }



  # insert stacking function here
  stacked.results<-BGVAR:::.gvar.stacking(x=x,saves=save_thin*saves,globalpost=globalpost,plag=plag,xglobal=xglobal,eigen=eigen,stats=stats,Trim=trim)


  ggW=lapply(gW,Matrix) # to save memory
  # country model residuals
  country.res<-unlist(lapply(globalpost,function(x)x$country.res));dim(country.res)<-c(length(country.res)/ncol(xglobal),ncol(xglobal))
  colnames(country.res)<-colnames(xglobal)

  # re-label
  country.coeffs<-lapply(globalpost,function(x) x$A_post)
  country.coeffs2<-try(BGVAR:::.relabel(gW1=gW,Names1=Names,country.coeffs=country.coeffs,Plag=plag,exoVars=exo,exo.countries1=exo.countries,endo1=endo))
  if(!class(country.coeffs2)=="try-error"){
    country.coeffs<-country.coeffs2;rm(country.coeffs2)
  }

   if(prior=="SSVS"){

    PIP<-lapply(globalpost,function(x) x$PIP);names(PIP)<-Names
    for(i in 1:length(PIP)){rownames(PIP[[i]])<-rownames(country.coeffs[[i]])}
    PIP<-BGVAR:::.avg.pip(Names1=Names,PIP1=PIP,exoVars=exo,exo.countries1=exo.countries,endo1=endo)

  }else{
    PIP<-NA
  }
  if(!is.null(SV)){
    vola_store<-lapply(globalpost,function(x) x$vola.store);names(vola_store)<-names(globalpost)
  }
  else{
    vola_store<-NA
  }

  if(prior=="NG"){
    kappa<-lapply(globalpost, function (x) x$kappa)
    a_tau<-lapply(globalpost, function(x) x$a_tau)
  }
  else{
    kappa<-a_tau<-NA
  }
  browser()
  cc.results<-list(coeffs=country.coeffs,res=country.res, PIP=PIP,vola=vola_store,kappa=kappa,a_tau=a_tau)
  # main results list
  results<-list(args = BGVAR:::.construct.arglist(bgvar),xglobal=xglobal,x=x,gW=ggW,ccode=Names,stacked.results=stacked.results,cc.results=cc.results)
  class(results)<-"bgvar"

  return(results)
}






remove_outliers <- function(x, na.rm = TRUE, ...) {
  qnt <- quantile(x, probs=c(.25, .75), na.rm = na.rm, ...)
  H <- 1.5 * IQR(x, na.rm = na.rm)
  y <- x
  y[x < (qnt[1] - H)] <- NA
  y[x > (qnt[2] + H)] <- NA
  y
}


conv.diag<-function(gvar.obj){
  require(coda)
  ALPHA<-gvar.obj$stacked.results$ALPHA_large
  d1<-dim(ALPHA)[1]
  d2<-dim(ALPHA)[2]
  d3<-dim(ALPHA)[3]
  K<-d2*d1

  mcmc.obj<-NULL
  for(i in 1:d1){
    aux<-NULL
    for(j in 1:d2){
      aux<-cbind(aux,ALPHA[i,j,])
    }
    mcmc.obj<-cbind(mcmc.obj,aux)
  }
  mcmc.obj<-mcmc(mcmc.obj)
  geweke<-geweke.diag(mcmc.obj)
  geweke.z<-as.numeric(unlist(geweke$z)) # if z is smaller or greater than 1.96 there is evidence that the means of both distributions are different
  idx<-which(abs(geweke.z)>1.96)
  xx<-paste0(length(idx), " out of ",K, " variables' z-values exceed the 1.96 threshold", " (", round(length(idx)/K*100,2),"%)")
  print(xx)
  return(list(geweke.z=geweke.z,xx=xx))
}






IRF<-function(gvar.obj,shockc=NULL,nhor=24,sign.constr=NA,rotSpec=NULL,cpu=1,full.post=FALSE){
  require(abind)
  require(doParallel)
  xx<-rot.nr<-NA
  x<-gvar.obj$x
  bigT<-ncol(x)
  F_large<-gvar.obj$stacked.results$F_large
  S_large<-gvar.obj$stacked.results$S_large
  R<-gvar.obj$stacked.results$R
  F.eigen<-gvar.obj$stacked.results$F.eigen
  Ginv_large<-gvar.obj$stacked.results$Ginv_large
  ALPHA_large<-gvar.obj$stacked.results$ALPHA_large
  plag<-gvar.obj$args$plag
  xglobal<-gvar.obj$xglobal
  kkk<-ncol(xglobal)
  Kbig<-plag*kkk
  ccode<-shock<-Rmed<-NA
  Global<-FALSE


  # user checks
  if(is.null(shockc)&&is.na(sign.constr)){
    stop("Please provide either the list objects shock OR sign.constr to run GIRF / Cholesky or sign restriction based impulse response analysis.")
  }
  if(!is.null(shockc)&& !is.na(sign.constr)){
    stop("Please provide either the list objects shock OR sign.constr to run GIRF / Cholesky or sign restriction based impulse response analysis.")
  }
  if(!is.null(shockc)){
    if(!all(c("shock","ident","ccode")%in%names(shockc))){
      stop("Please provide list entries shock, ccode and ident for the object shock. See the help files for more details.")
    }
    ident=shockc$ident
    if(!ident%in% c("GIRF","chol")){
      ident="GIRF"
      cat("Warning: Impulse response analysis based on generalized impulse responses (GIRF).")
    }

    shock<-shockc$shock;ccode<-shockc$ccode;Scal=shockc$scal
    # check whether specified shock makes sense
    ss<-paste(ccode,shock,sep=".")
    if(!all(ss%in%colnames(xglobal))){
      stop("Not all variables you have selected to shock are contained in the model. Please re-specify.")
    }
    if(length(ss)>1){
      Global=TRUE
    }
    ss<-list(ss)
  }

  saves<-dim(Ginv_large)[3]
  xglobal<-gvar.obj$xglobal
  R.store<-array(NA,dim=c(kkk,kkk,saves))

  if(!any(is.na(sign.constr))){  # if you have submitted a list that contains sign restrictions, identification is via sign restrictions
    ident<-"sign"
    shock.nr<-length(sign.constr)
    # check whether sign restriction list is correctly specified, for each shock have to specify
    tt<-all(sapply(lapply(sign.constr,names),function(x) all(c("shock","sign","constr")%in% x)))
    if(!tt){
      stop("For each shock (i.e., first layer in the list), please specify lists named shock, sign and constr and restrictions. See the details and examples in the manual.")
    }
    Scal<-unlist(sapply(sign.constr,function(x) x$scal))
    ss<-lapply(sign.constr,function(x) x$shock)
    if(length(sign.constr[[1]]$shock)>1){ # in case you have specified a global shock
      Global=TRUE
    }else{
      Global=FALSE
    }
    if(!is.null(rotSpec)){
      Srots<-rotSpec$Srots;MaxTries<-rotSpec$MaxTries
    }else{
      Srots<-1;MaxTries<-7500
    }
  }else{
    shock.nr<-1
  }




  ######################################################################################################################################
  #assign the irf function

  if(ident=="sign"){
    irf<-BGVAR:::.irf.sign
    # first check whether all elements of sign restriction list have been specified
    for(m in 1:length(sign.constr)){
      aux<-sign.constr[[m]]
      if(!all(c("shock","restrictions","sign","rest.horz","constr")%in%names(aux))){
        stop("Please specify for the object 'sign.contr' slots labeled 'shock', 'restrictions', 'sign', 'rest.horz' and 'constr'. A slot 'scal' is optional. See the help files for more information.")
      }
      mm<-1+length(aux$restrictions) # one for the shock, rest for the restrictions
      if(length(aux$sign)!=mm || length(aux$rest.horz)!=mm || length(aux$constr)!=mm){
        stop("Please specify M +1 signs, rest.horz and constr with M denoting the nr. of restrictions.")
      }
    }

    strg.list<-list()

    if(Global){ # Global shock and restr horizon makes invoking apply necessary if irf horz >0,
      for(z in 1:shock.nr){

        cc.res<-c(length(sign.constr[[z]]$shock),sapply(sign.constr[[z]]$restrictions,length)) # nr. of cc restrictions, need it to calculate percentages
        # build up the restriction matrix  and check whether valid

        strg<-paste0("((sum(",paste0("apply(irf_fac[sign.constr[[",z,"]]$shock,",z,","),paste0("1:",sign.constr[[z]]$rest.horz[1],",drop=FALSE]") ,paste0(sign.constr[[z]]$sign[[1]],"0,1,all))/",cc.res[1],")>=",sign.constr[[z]]$constr[[1]],")*"));na.idx<-c()
        for(i in 1:length(sign.constr[[z]]$restr)){
          strg<-c(strg,paste0("((sum(",paste0("apply(irf_fac[sign.constr[[",z,"]]$restr[[",i,"]],",z,","),paste0("1:",sign.constr[[z]]$rest.horz[(i+1)],",drop=FALSE]"),paste0(sign.constr[[z]]$sign[[(i+1)]],"0,1,all))/",cc.res[(i+1)],")>=",sign.constr[[z]]$constr[[(i+1)]],")*")))
          na.idx<-c(na.idx,all(sign.constr[[z]]$restr[[i]]%in% colnames(xglobal)))
        }
        strg[length(strg)]<-gsub("*","",strg[length(strg)],fixed=TRUE);strg<-paste(strg,collapse="") # make one string

        if(any(!(na.idx))){
          stop(paste0("Restriction(s) ",which(!na.idx), " is (are) not valid. Please re-specify."))
        }
        strg.list[[z]]<-strg
      }
    }else{ # this is the same as with the global shock, but without the apply around it, all is needed for the IRF horizon (all horizons should fulfill the restr's)

      for(z in 1:shock.nr){
        # build up the restriction matrix
        cc.res<-c(length(sign.constr[[z]]$shock),sapply(sign.constr[[z]]$restrictions,length)) # nr. of cc restrictions, need it to calculate percentages

        strg<-paste0("((sum(all(",paste0("irf_fac[sign.constr[[",z,"]]$shock,",z,","),paste0("1:",sign.constr[[z]]$rest.horz[1],",drop=FALSE]"),paste0(sign.constr[[z]]$sign[[1]],"0))/",cc.res[1],")>=",sign.constr[[z]]$constr[[1]],")*"));na.idx<-c()
        for(i in 1:length(sign.constr[[z]]$restr)){
          if(cc.res[(i+1)]>1){ # in case cc-sign restrictions have been applied for a non-global shock
            strg<-c(strg,paste0("((sum(",paste0("apply(irf_fac[sign.constr[[",z,"]]$restr[[",i,"]],",z,","),paste0("1:",sign.constr[[z]]$rest.horz[(i+1)],",drop=FALSE]"),paste0(sign.constr[[z]]$sign[[(i+1)]],"0,1,all))/",cc.res[(i+1)],")>=",sign.constr[[z]]$constr[[(i+1)]],")*")))
          }else{   # without cc-sign restrictions can spare on apply, makes evaluation faster
            strg<-c(strg,paste0("((sum(all(",paste0("irf_fac[sign.constr[[",z,"]]$restr[[",i,"]],",z,","),paste0("1:",sign.constr[[z]]$rest.horz[(i+1)],",drop=FALSE]"),paste0(sign.constr[[z]]$sign[[(i+1)]],"0))/",cc.res[(i+1)],")>=",sign.constr[[z]]$constr[[(i+1)]],")*")))
          }
          na.idx<-c(na.idx,all(sign.constr[[z]]$restr[[i]]%in% colnames(xglobal)))
        }
        strg[length(strg)]<-gsub("*","",strg[length(strg)],fixed=TRUE);strg<-paste(strg,collapse="") # make one string

        if(any(!(na.idx))){
          stop(paste0("Restriction(s) ",which(!na.idx), " is (are) not valid. Please re-specify."))
        }
        strg.list[[z]]<-strg
      }
    }

    #strg.list.neg<-lapply(strg.list,function(x) gsub("#","<",gsub("<",">",gsub(">","#",x))))

  }
  else if(ident=="chol"){
    irf<-BGVAR:::.irf.chol;MaxTries<-str<-sign.constr<-NA;print("chol")
  }
  else{
    irf<-BGVAR:::.irf.girf;MaxTries<-str<-sign.constr<-NA;print("girf")
  }

  # initialize objects to save IRFs, HDs, etc.
  IRF_post<-array(NA,dim=c(nrow(x),nhor+1,shock.nr,saves)) # IRF Post object
  imp_posterior<-array(NA,dim=c(ncol(xglobal),(nhor+1),shock.nr,7)) # 7 for low25,median,high75,low16,high84
  dimnames(imp_posterior)[[1]]<-colnames(xglobal)
  dimnames(imp_posterior)[[2]]<-1:(nhor+1)
  dimnames(imp_posterior)[[3]]<-paste("shock",1:shock.nr,sep="_")
  dimnames(imp_posterior)[[4]]<-c("low25","low16","low05","median","high75","high84","high95")


  # END initializing --------------------------------------------------------------------------------------------------------------



  # start loop here########
  if(cpu>1){# use parallel computing for IRF analysis
    registerDoParallel(cores=cpu)
    cat(paste0(as.character(Sys.time()),": Start impulse response analysis on ", cpu, " cores", " (",saves," stable draws in total)."),"\n")
    sfilename<-"log.txt";file.create(sfilename)
    cat(paste0(as.character(Sys.time()),": Start impulse response analysis on ", cpu, " cores"," (",saves," stable draws in total)."),"\n",append=TRUE,file=sfilename)

    imp.obj <-foreach(irep1=1:saves) %dopar%{
      imp.obj <- irf(maxlag=plag,ccode=ccode,shockvar=shockc,invG=Ginv_large[,,irep1],lF=F_large[,,,irep1,drop=FALSE],gcov=S_large[,,irep1],x=x,horizon=nhor,xglobal=xglobal,M=kkk,srots=Srots,rest=strg.list,MaxTrys=MaxTries,sign.constr=sign.constr,Global=Global,ss=ss)
      if(!any(is.na(imp.obj$rot))){
        cat("\n",as.character(Sys.time()), "MCMC draw", irep1, ": rotation found after ",imp.obj$icounter," tries", "\n",append=TRUE,file=sfilename)
      }else{
        cat("\n",as.character(Sys.time()), "MCMC draw", irep1, ": no rotation found", "\n",append=TRUE,file=sfilename)
      }
      return(list(impl=imp.obj$impl,rot=imp.obj$rot,xglobal=imp.obj$xglobal))
    }

    for(irep1 in 1:saves){
      R.store[,,irep1]<-imp.obj[[irep1]]$rot
      IRF_post[,,,irep1] <- imp.obj[[irep1]]$impl
    }
  }else{ #carry out impulse response function analysis in a loop
    cat("\n",paste0(as.character(Sys.time()),":")," Start impulse response analysis.","\n")
    pb <- txtProgressBar(min = 0, max = saves, style = 3)
    for(irep1 in 1:saves){
      #t1<-Sys.time()
       #F_array <- array(F_large[,,,irep1],c(kkk,kkk,plag))
      imp.obj  <- irf(maxlag=plag,ccode=ccode,shockvar=shockc,invG=Ginv_large[,,irep1],lF=F_large[,,,irep1,drop=FALSE],gcov=S_large[,,irep1],x=x,horizon=nhor,xglobal=xglobal,M=kkk,srots=Srots,rest=strg.list,MaxTrys=MaxTries,sign.constr=sign.constr,Global=Global,ss=ss)
      imp<-imp.obj$impl
      R.store[,,irep1]<-imp.obj$rot


      IRF_post[,,,irep1] <- imp
      setTxtProgressBar(pb, irep1)
      # print sign restriction info
      if(ident=="sign"){
        if(!any(is.na(imp.obj$rot))){
          cat("\n","MCMC draw", irep1, ": rotation found after ",imp.obj$icounter," tries", "\n")
        }
      }
    }
  }


  dimnames(IRF_post)[[1]]<-colnames(xglobal)

  #----------------------------------------------------------------------------------------------------------------------------------------------------------------
  # END LOOP HERE ----------------------------------------------------------------------------------------------------------------
  #-----------------------------------------------------------------------------------------------------------------------------------------------------------------


  # re-set IRF object in case we have found only a few rotation matrices
  if(ident=="sign"){
    idx<-which(!is.na(apply(IRF_post,4,sum)))
    rot.nr<-paste("For ", length(idx), " draws out of ", saves, " draws, a rotation matrix has been found.")
    print(rot.nr)
    # subset posterior draws
    IRF_post<-IRF_post[,,,idx,drop=FALSE];Ginv_large<-Ginv_large[,,idx]
    ALPHA_large<-ALPHA_large[,,idx];S_large<-S_large[,,idx]
    R.store<-R.store[,,idx]
    saves<-length(idx)
  }



  # In case we have found at least one rotation matrix
  if(saves>0){
    # do normalization here
    if(!is.null(Scal)){
      # do this for each identified shock (if sign restrictions, there might be more than one shock identified)
      if(Global){
         for(z in 1:shock.nr){
           Mean<-colMeans(IRF_post[ss[[z]],1,z,,drop=FALSE]) #in case we have multiple country shocks, scale as average on impact
           for(i in 1:saves){
            IRF_post[,,z,i]<-(IRF_post[,,z,i]/Mean[i])*Scal[z]
           }
         }
        }else{ #if no global shock has been carried out, but a local shock
        for(z in 1:shock.nr){
          Mean<-IRF_post[ss[[z]],1,z,]
          for(i in 1:saves){
            IRF_post[,,z,i]<-(IRF_post[,,z,i]/Mean[i])*Scal[z]
          }
        }
      }
    }


    for(i in 1:shock.nr){
      imp_posterior[,,i,"low25"] <- apply(IRF_post[,,i,],c(1,2),quantile,0.25,na.rm=TRUE)#;rownames(imp_low25)<-colnames(xglobal)
      imp_posterior[,,i,"low16"] <- apply(IRF_post[,,i,],c(1,2),quantile,0.16,na.rm=TRUE)#;rownames(imp_low25)<-colnames(xglobal)
      imp_posterior[,,i,"low05"] <- apply(IRF_post[,,i,],c(1,2),quantile,0.05,na.rm=TRUE)#;rownames(imp_low25)<-colnames(xglobal)
      imp_posterior[,,i,"median"] <- apply(IRF_post[,,i,],c(1,2),median,na.rm=TRUE)#;rownames(imp_low25)<-colnames(xglobal)
      imp_posterior[,,i,"high75"] <- apply(IRF_post[,,i,],c(1,2),quantile,0.75,na.rm=TRUE)#;rownames(imp_low25)<-colnames(xglobal)
      imp_posterior[,,i,"high84"] <- apply(IRF_post[,,i,],c(1,2),quantile,0.84,na.rm=TRUE)#;rownames(imp_low25)<-colnames(xglobal)
      imp_posterior[,,i,"high95"] <- apply(IRF_post[,,i,],c(1,2),quantile,0.95,na.rm=TRUE)#;rownames(imp_low25)<-colnames(xglobal)
    }
  }

  # calculate objects needed for HD and struc shock functions later---------------------------------------------
  # median quantitities
  Ginv<-apply(Ginv_large,c(1:2),median);S<-apply(S_large,c(1:2),median)
  Sigma_u<-Ginv%*%S%*%t(Ginv);
  ALPHA<-apply(ALPHA_large,c(1:2),median)


  Fmat2<-apply(F_large[,,,,drop=FALSE],c(1:3),median)[,drop=FALSE]
  Fmat<-abind(Fmat2,Fmat2,rev.along=0)
  if(ident=="sign"){
    # look for median rotation matrix based on the posterior median
    irf.obj<-try(irf(maxlag=plag,ccode=ccode,shockvar=shockc,invG=Ginv,lF=Fmat[,,,1,drop=FALSE],gcov=S,x=x,horizon=nhor,xglobal=xglobal,M=kkk,srots=10,rest=strg.list,MaxTrys=MaxTries,sign.constr=sign.constr,Global=Global,ss=ss),silent=TRUE)
    if(!class(irf.obj)=="try-error"){
      Rmed<-irf.obj$rot
    }
  }

  struc.obj<-list(Sigma_u=Sigma_u,xglobal=xglobal,plag=plag, ALPHA=ALPHA,Rmed=Rmed,x=x,Ginv=Ginv,S=S,Fmat=Fmat,ccode=ccode,Global=Global,ss=ss)
  #---------------------------------------------------------------------------------------------------------------

  if(!full.post){
    IRF_post<-NA
  }
  results<-list(posterior = imp_posterior, rot.nr=rot.nr,sign.constr=sign.constr,nr.draws=dim(IRF_post)[4],ident=ident,struc.obj=struc.obj,full.post=IRF_post)
  cat(paste("Size of IRF object:", format(object.size(results),unit="MB")))
  return(results)
}


# END IRF FUNCTION ##################################################################################################################


hd.decomp<-function(irf.obj, R=NULL){


  # get objects from irf.obj
  Sigma_u=irf.obj$struc.obj$Sigma_u
  if(is.null(R)){ #we use the median rotation matrix at the posterior median
    R<-irf.obj$struc.obj$Rmed
  }
  xglobal<-irf.obj$struc.obj$xglobal;x<-irf.obj$struc.obj$x
  plag<-irf.obj$struc.obj$plag;ALPHA=irf.obj$struc.obj$ALPHA
  bigT<-ncol(x)
  kkk<-ncol(xglobal)
  if(length(R)==1 && is.na(R)){
    R<-diag(kkk) #in case we have not used sign restrictions R is identity matrix
  }
  rownames(R) <- colnames(R) <- rownames(x)



  #initialize objects here
  struc_post <- array(NA,dim=c(bigT,nrow(x))) #Posterior of structural shocks
  hd_array<-array(0,c(kkk,bigT,(kkk+3)))#shock_post <- array(0,c(saves,kkk,bigT,kkk)) +3 because we concatenate contributions of initial conditions and constant term
  dimnames(hd_array)<-list(rownames(x), NULL, c(paste("contribution of shock to", c(rownames(x))),"constant","initial cond.","residual"))



  #This computes the structural error   of the model AY(t)-BY(t-1)=v(t), v(t)~N(0,I) and its historical decomposition

  Rinv <-solve(R)
  #Sigma_u <- Ginv_large[,,irep1]%*%S_large[,,irep1]%*%t(Ginv_large[,,irep1])
  Sigchol_u <- t(chol(Sigma_u))
  Sigcholinv <- solve(Sigchol_u)


  vv <- matrix(0,bigT,ncol(xglobal),dimnames=list(NULL,colnames(xglobal)))
  YY <- xglobal[(plag+1):nrow(xglobal),]
  XX <- cbind(BGVAR:::.mlag(xglobal,plag),1); XX <- XX[(plag+1):nrow(XX),]
  strMat <- Rinv%*%Sigcholinv


  for (t in 1:bigT){
    Yhat <- strMat%*%YY[t,]
    Xhat <- strMat%*%ALPHA%*%XX[t,]

    PHI <- Rinv%*%Sigcholinv%*%ALPHA
    vv[t,] <- Yhat-Xhat
  }

  #Start historical decompositions --------------------------------------------------------------------------------------
  if(irf.obj$ident=="GIRF"){
    print("Historical decomposition of the time series not implemented for GIRFs since cross-correlation is unequal to zero (and hence decompositions do not sum up to original time series).")
    return(list(hd_array=NA,struc.shock=vv,xglobal=xglobal) )
  }


  HDshock_big <- array(0,c(plag*kkk,bigT,kkk))
  HDshock <- array(0,c(kkk,bigT,kkk))
  HDconst_big <- matrix(0,plag*kkk,bigT)
  HDinit_big <- matrix(0,plag*kkk,bigT)
  HDinit <- matrix(0,kkk,bigT)
  HDconst_big <- matrix(0,plag*kkk,bigT)
  HDconst <- matrix(0,kkk,bigT)


  #NOTE FOR MARTIN: IF SIGN RESTRICTIONS: R = ROTATION ELSE R = diag(M)
  solveA <- (Sigchol_u%*%R) #Depends on identification, if Cholesky then solveA = t(chol(SIGMA)), where SIGMA is the VC of the global model
  eps <- (YY-XX%*%t(ALPHA))%*%t(solve(solveA)) #Atilda is the matrix of autoregressive coefficients of the global model
  Fcomp <- BGVAR:::.get_companion(ALPHA[,1:(kkk*plag)],varndxv = c(kkk,0,plag))$MM#Fcomp is the companion matrix (used in the eigenvalue stuff without the constant)

  invA_big <- matrix(0,kkk*plag,kkk)  #M is the number of endogenous variables ; p is the number of lags
  invA_big[1:kkk,] <- solveA
  Icomp <- cbind(diag(kkk),matrix(0,kkk,(plag-1)*kkk))
  for (nn in 2:bigT){
    for (jj in 1:kkk){
      eps_big <- matrix(0,kkk,1)
      eps_big[jj,] <- eps[nn,jj]
      HDshock_big[,nn,jj] <- (invA_big)%*%eps_big+Fcomp%*%HDshock_big[,nn-1,jj]
      HDshock[,nn,jj] <- Icomp%*%HDshock_big[,nn,jj]
    }
    #Initial value
    HDinit_big[,1] <- XX[1,1:(plag*kkk)]
    HDinit[,1] <- Icomp%*%HDinit_big[,1]
    HDinit_big[,nn] <- Fcomp%*%HDinit_big[,nn-1]
    HDinit[,nn] <- Icomp%*%HDinit_big[,nn]

    #Constant
    CC <- matrix(0,kkk*plag,1)
    CC[1:kkk] <- t(ALPHA)[(kkk*plag)+1,]
    HDconst_big[,nn] <- CC+Fcomp%*%HDconst_big[,nn-1]
    HDconst[,nn] <- Icomp%*%HDconst_big[,nn]
  }

  #org.ts <- sum.HD.shock+HDconst+HDinit
  # put them in an array

  hd_array[,,1:kkk]<-HDshock_big
  hd_array[,,(kkk+1)]<-HDconst_big
  hd_array[,,(kkk+2)]<-HDinit_big
  hd_array[,,(kkk+3)]<-(x-apply(hd_array,c(1,2),sum)) # residual part



  results<-list(hd_array=hd_array,struc.shock=vv,x=x)
  return(results)
}







# irfplot --------------------------------------------------------------------------------------------------------------------

irf.plot<-function(irf.obj,resp,shock.nr=1,B=NULL,N=8,Main=NULL,cex.la=1.2,cum=FALSE){
  posterior<-irf.obj$posterior

  idx<-which(resp==dimnames(posterior)[[1]])
  if(!length(idx)>0){
    idx<-which(grepl(resp,dimnames(posterior)[[1]]))
    if(!length(idx)>0){
      stop("The variable name you have specified does not match any names in the data. Please re-specify.")
    }
  }


  x<-posterior[idx,,shock.nr,c("low25","median","high75"),drop=FALSE] # first dimension is nr. of variables to be plotted
  y<-posterior[idx,,shock.nr,c("low16","median","high84"),drop=FALSE] # first dimension is nr. of variables to be plotted
  nr<-dim(x)[[1]]

  if(nr>1){
    par(mfrow=c(3,ceiling(nr/3)))
  }

  for(i in 1:nr){
    x1<-x[i,,,]
    y1<-y[i,,,]
    if(cum){
      x1<-apply(x1,2,cumsum)
      y1<-apply(y1,2,cumsum)
    }
    if(is.null(B)){
      b<-range(cbind(x1,y1))
    }else{
      b<-B
    }
    if(is.null(Main)){
      Main2<-dimnames(x)[[1]][i]
    }else{
      Main2<-Main
    }

    b1<-b[1];b2<-rev(b)[1]
    matplot(x1,type="l",col=c("blue","black","blue"),yaxt="n",xaxt="n",lwd=4,ylab="",main=Main2,
            cex.main=2,cex.axis=2.5,cex.lab=2.5,lty=c(0,1,0),ylim=c(b1,b2))
    polygon(c(1:nrow(y1),rev(1:nrow(y1))),c(y1[,1],rev(y1[,3])),col="lightblue",border=NA)
    polygon(c(1:nrow(x1),rev(1:nrow(x1))),c(x1[,1],rev(x1[,3])),col="#0099B4",border=NA)

    abline(h=0,lty=1,lwd=4,col="red")
    lines(x1[,2],col="darkblue",lwd=4)

    axis(2, at=seq(b1,b2,length.out=N), labels=format(seq(b1,b2,length.out=N),digits=1,nsmall=1),cex.axis=cex.la,las=1)

    axisindex<-c(1:nrow(x1))[c(TRUE,FALSE)]
    axis(side=1, las=1,at=axisindex, labels=c(0:nrow(x1))[axisindex], cex.axis=1.6,tick=FALSE)
    abline(v=axisindex,col="lightgrey",lty=3)
  }
  if(nr>1){
    par(mfrow=c(1,1))
  }
}


#Helper functions needed for the Sims & Zha (1998) prior
#Creates Dummies
.get_dum <- function(lambda,lambda1,lambda2,M,M_,p,S_prior,sigma_sq,nabla,X,Y_=Ypre,W_=Wexpre,s1=sig_wex,lambda3){
  #nabla <- 1/10000
  #Prior mean
  pr_mean <- diag(M)*1
  #pr_mean[grepl("Dp",colnames(Y_)),grepl("Dp",colnames(Y_))] <- 1
  y_ <- as.numeric(apply(Y_,2,mean))
  w_ <- as.numeric(apply(W_,2,mean))
  #create dummy matrices
  K <- ncol(X)
  d_end <- matrix(0,M*p,K)
  d_det <- matrix(0,1,K)
  d_wex <- matrix(0,(M_*p)+M_,K)
  colnames(d_end) <- colnames(d_det) <- colnames(d_wex) <- colnames(X)

  ydum <- rbind(matrix(0,1,M),matrix(0,(M_*p+M_),M),S_prior,(diag(as.numeric(sigma_sq))%*%pr_mean)/lambda,matrix(0,(M*p)-M,M))
  #same for sum of coefficients
  y_init <- t(as.matrix(y_))/lambda2
  #same for sum of coefficients
  y_sum <- (diag(y_)*pr_mean)/lambda3

  ydum <- rbind(ydum,y_init,y_sum) #,y_sum,
  #------------------------------------------------------------------------------------------------
  J <- diag(1:p)
  J1 <- diag(1:(p+1))
  #dummy part for endogenous variables
  d_end[1:(M*p),grepl("Ylag",colnames(d_wex))] <- kronecker(J,diag(as.numeric(sigma_sq))/lambda)
  d_end_sum <- kronecker(matrix(1,1,p),diag(y_)*pr_mean)/lambda3
  #dummy for the initial obs prior
  d_end_init <- kronecker(matrix(1,1,p),t(as.matrix(y_/lambda2)))
  #dummy for determinisics
  d_det[1:1,1:1] <- diag(1)*nabla
  d_det_init <- matrix(0,1,1)
  d_det_sum <- matrix(0,M,1)
  #dummy for SIGMA
  d_sig <- matrix(0,M,K)
  #dummy for weakly exogenous
  d_wex[1:((M_*p)+M_),grepl("Wex",colnames(d_wex))] <- kronecker(J1,diag(as.numeric(s1))/lambda1)
  d_wex_init <- kronecker(matrix(1,1,p+1),t(as.matrix(w_/lambda2)))
  #d_wex_sum <- kronecker(t(as.matrix(seq(1,(p+1)))),diag(w_))*1/lambda3
  d_wex_sum <- matrix(0,M,(M_*p)+M_)
  #combine for sum of coefficient prior
  d_init <- cbind(d_det_init,d_wex_init,d_end_init)
  d_sum <- cbind(d_det_sum,d_wex_sum,d_end_sum)

  xdum <- rbind(d_det,d_wex,d_sig,d_end,d_init,d_sum)#,),,d_init
  return(list(X=xdum,Y=ydum))
}
.get_ML <- function(lambdas_,Y_,X_,K,A_prior,S_prior,sigma_sq,v_prior,bigT,M,M_,p,sig_wex){
  lambda <- lambdas_[[1]];lambda1 <- lambdas_[[2]];lambda2 <- lambdas_[[3]];lambda3 <- lambdas_[[4]]

  dum <- BGVAR:::.get_dum(lambda=lambdas_[[1]],lambda1=lambdas_[[2]],lambda2=1,M=M,M_=M_,S_prior=diag(as.numeric(sigma_sq)),p=p,sigma_sq=sigma_sq,nabla=lambda2,X=X_,s1=sig_wex,lambda3=1)
  X_ <- rbind(X_,dum$X);Y_ <- rbind(Y_,dum$Y)

  #check whether V_prior exist
  cc <- try(solve(crossprod(dum$X)), silent=TRUE)
  if(is(cc,"try-error")) {
    ml <- -Inf
  }else{

    V_prior <- solve(crossprod(dum$X))
    V_post <- solve(crossprod(X_))
    A_post <- V_post%*%t(X_)%*%Y_

    S_post <- crossprod(Y_-X_%*%A_post)

    ml <- (-M/2 * (log(det(V_prior))+log(det(crossprod(X_))))+(-(bigT+M+v_prior-1)/2)*log(det(S_post)))
  }
  return(ml)
}
#Helper functions for the NG prior
.atau_post <- function(atau=a_tau,lambda2=lambda2_tau,thetas=theta,k=length(theta),rat=1){
  logpost <- sum(dgamma(thetas,atau,(atau*lambda2/2),log=TRUE))+dexp(atau,rate=rat,log=TRUE)
  return(logpost)
}
# Main function to compute the country models
BVAR_linear <- function(nr,gW=gW,xglobal=xglobal,cN,p=4,nsave=500,nburn=500,hyper=list(c_tau = 0.01, d_tau = 0.01,e_lambda=1.5,d_lambda=1, prmean=0,a_start=.6,sample_A=TRUE),prior="NG",thin=0.1,sv=TRUE,trend=FALSE){
  #Parts of this code is based on the code created by Dimitris Korobilis and Gary Koop for their BVAR monograph.
  require(MASS)
  require(compiler)
  require(stochvol)
  require(GIGrvg)

  ntot <- nsave+nburn # total draws
  #---------------------------------------Data stuff------------------------------------------------
  End <- xglobal[,substr(colnames(xglobal),1,2)==cN[[nr]],drop=FALSE]
  class(End) <- "numeric"
  class(xglobal) <- "numeric"
  W <- gW[[nr]]

  #Construct weakly exogenous variables
  all <- W%*%t(xglobal)
  Wex <- all[(ncol(End)+1):nrow(all),,drop=FALSE]
  Wex <- t(Wex)
  class(Wex) <- "numeric"

  X <- End
  Yraw <- X

  Yraw <- as.matrix(Yraw)
  class(Yraw) <- "numeric"
  #-----------------------------------------Data Preparation-----------------------------------------------
  dimYraw <- dim(Yraw) #get dimensions of the dependent variable
  Traw <- dimYraw[1]
  N <- dimYraw[2]

  #----------------------------------------Model Specification-----------------------------------------
  #uses the function above
  Ylag <- BGVAR:::.mlag(Yraw,p)
  Wexlag <- BGVAR:::.mlag(Wex,p)

  M_ <- ncol(Wex)
  colnames(Wex) <- rep("Wex",ncol(Wex))

  #creates nametags for the lags, makes it easier to select the corresponding coefficient matrices later on
  nameslags <- NULL
  wexnameslags <- NULL
  for (ii in 1:p){
    nameslags <- c(nameslags,rep(paste("Ylag",ii,sep=""),ncol(Yraw)))
    wexnameslags <- c(wexnameslags,rep(paste("Wexlag",ii,sep=""),ncol(Wex)))
  }
  colnames(Ylag) <- nameslags
  colnames(Wexlag) <- wexnameslags
  X1 <- cbind(Wex,Wexlag,Ylag)

  Wexpre <<- Wex[1:p,,drop=FALSE]
  Ypre <<- Yraw[1:p,,drop=FALSE]

  X1 <- X1[(p+1):nrow(X1),,drop=FALSE]
  #---------we need to include this for the trend component and constant------------#
  X1 <- cbind(1,X1)
  colnames(X1)[[1]] <- "constant"

  # do this only for ssvs prior
  if(prior=="SSVS"){
    if (trend){
      X1 <- cbind(1:nrow(X1),X1)
      colnames(X1)[[1]] <- "trend"
    } else {
      X1 <- X1
    }
  }

  #-----------------get size of the final matrix X---------------------#
  dimX <- dim(X1)
  Traw3 <- dimX[1]
  K <- dimX[2]
  bigT <- Traw-p
  Y1 <- Yraw[(p+1):Traw,,drop=FALSE]
  Yfull <- Y1 #saves the full data
  Xfull <- X1
  Y <- Y1
  X <- X1
  #----------------------PRIOR FUN------------------------------
  #----------------------getting OLS estimates------------------
  A_OLS  <-  ginv(crossprod(X))%*%crossprod(X,Y)
  a_OLS <- as.vector(A_OLS) #vectorizes A_OLS, i.e. a_OLS=vec(A_OLS)
  SSE  <-  t((Y - X%*%A_OLS))%*%(Y - X%*%A_OLS)
  Em <- Em.str <- Y - X%*%A_OLS
  SIGMA_OLS  <-  SSE/(bigT-K+1)

  #-------------Initialize Bayesian Posteriors using OLS values-
  alpha <- a_OLS # single draw from the posterior of alpha
  ALPHA <- A_OLS # -,,- ALPHA
  SSE_Gibbs <- SSE # of SSE
  SIGMA <- SIGMA_OLS #of SIGMA

  M <- N
  n <- K*N
  count <- 0

  #SV
  svdraw <- list(para=c(mu=-10,phi=.9,sigma=.2),latent=rep(-3,bigT))
  svl <- list()
  for (jj in 1:M) svl[[jj]] <- svdraw

  hv <- svdraw$latent
  para <- list(mu=-3,phi=.9,sigma=.2)
  H <- matrix(-3,bigT,M)
  eta <- list()

  if(is.null(sv)){
    sv<-FALSE
  }else{
    aux<- unlist(strsplit(sv,".",fix=TRUE))[2];sv<-TRUE
    if(aux=="T"){  # this is the case where we take last data point for SV
      Sigma1<-function(H,bigT){
        SIGMA <- matrix(exp(H[bigT,]))
        return(SIGMA)
      }
      Sigma2<-function(Upsilon_draw,H,bigT){
        SIGMA <- Upsilon_draw%*%diag(exp(H[bigT,]))%*%t(Upsilon_draw)
        return(SIGMA)
      }
    }else{
      Sigma1<-function(H,bigT){
        SIGMA<-apply(exp(H),2,mean)
        return(SIGMA)
      }
      Sigma2<-function(Upsilon_draw,H,bigT){
        SIGMA<-Upsilon_draw%*%diag(apply(exp(H),2,mean))%*%t(Upsilon_draw)
        return(SIGMA)
      }
    }
  }


  #store the draws:
  thinsaves <- round(thin*nsave)
  ALPHA_draws <- array(0,dim=c(thinsaves,K,N))
  SIGMA_draws <- array(0,dim=c(thinsaves,N,N))
  lambda_draws <- matrix(0,thinsaves,3)
  gamma_draws <- matrix(0,thinsaves,n)
  res_store<-matrix(0,nrow(X),M)
  thin.draws <- round(seq(nburn+1,ntot,length.out = thinsaves))
  theta_store <- array(0,dim=c(thinsaves,3))
  hv_store <- array(0,c(thinsaves,bigT,M))
  pars_store <- array(0,c(thinsaves,3,M))
  kappa_store <- array(NA,c(thinsaves,p+1,2))
  a_store <- array(NA,c(thinsaves,p+1,2))
  PL <- matrix(0,nsave,1)


  #---------------------------------------------------START SSVS--------------------------------------
  if (prior=="SSVS"){
    #hyper <- list(tau0=0.1,tau1=3,kappa0=0.1,kappa1=7,a_i=0.01,b_i=0.01,p_i=0.5,q_ij=0.5)#

    gamma_draws <- matrix(0,nsave,n)
    M <- N
    #-PRIOR SETUP FOR SSVS PRIOR-----------------------------------#
    # Priors on SIGMA
    # SSVS variances for non-diagonal elements of SIGMA
    kappa_0  <-  hyper$kappa0 #Set kappa_[0ij], kappa_[1ij]
    kappa_1  <-  hyper$kappa1

    # Hyperparameters for diagonal elements of SIGMA (Gamma density)
    a_i  <-  hyper$a_i
    b_i  <-  hyper$b_i

    # Hyperparameters for Gamma ~ BERNOULLI(m,p_i), see eq. (14)
    p_i  <-  hyper$p_i

    # Hyperparameters for Omega_[j] ~ BERNOULLI(j,q_ij), see eq. (17)
    q_ij  <- hyper$q_i

    # Initialize Gamma and Omega vectors
    gammas  <-  matrix(1,n,1)       # vector of Gamma
    if (M>1){
    omega <- list() #!!!!!!!
    for (kk_1 in 1:(M-1)){
      omega[[kk_1]]  <-  matrix(1,kk_1,1)  # Omega_j
    }
    }
    #--------------------------------------------PRIOR Calibration ----------------------------------------------
    #------------------------------------------getting OLS estimates---------------------------------------------
    XtXinv <- ginv(t(X)%*%X)
    A_OLS <- A_draw  <-  XtXinv%*%(t(X)%*%Y)
    a_OLS <- as.vector(A_OLS) #vectorizes A_OLS, i.e. a_OLS=vec(A_OLS)
    SSE  <-  t((Y - X%*%A_OLS))%*%(Y - X%*%A_OLS)
    SIGMA_OLS  <-  SSE/(bigT-K+1)
    IXY  <-   kronecker(diag(M),(t(X)%*%Y))
    #-------------Initialize Bayesian Posteriors using OLS values-
    alpha <- a_OLS # single draw from the posterior of alpha
    ALPHA <- A_OLS # -,,- ALPHA
    SSE_Gibbs <- SSE # of SSE
    SIGMA <- SIGMA_OLS #of SIGMA
    if (M>1){
    b_draw <- lower.tri(SIGMA_OLS)*10
    Upsilon_draw <- b_draw
    diag(Upsilon_draw) <- 1
    }
    A_prior <<- matrix(0,K,N)

    # ind <- matrix(0,N,p)
    # for (i in 1:N){
    #   ind[i,] <- seq(1+ncol(Wex)+ncol(Wexlag)+i,K, by=N)
    # }
    #
    # for (j in 1:N){
    #     if (j==2){
    #         A_prior[ind[j,1],j] <- 1
    #           A_prior[,j] <- 1
    #     }else{
    #         A_prior[,j] <- 1 #prior view that the variables follow a random walk
    #     }
    # }


    a_prior  <-  as.vector(A_prior)

    theta <- matrix(10,K,M)
    # This is the std of the OLS estimate of alpha. You can use this to
    # scale tau_0 and tau_1 (see below) if you want.
    # SSVS variances for alpha
    sigma_alpha  <-  sqrt(diag(kronecker(SIGMA_OLS,XtXinv)))

    tau_0  <-  hyper$tau0*sigma_alpha   # Set tau_[0i], tau_[1i] #MARTIN CHGHGHCHGHHGHGH!!!
    tau_1  <-  hyper$tau1*sigma_alpha
    for (irep in 1:ntot){
      #Step I: Sample autoregressive parameters
      for (mm in 1:M){
        #Slow step (if T>>K)
        #Sample autoregressive coefficients equation by equation
        if (mm==1){
          Y.i <- Y[,mm]*exp(-0.5*H[,mm])
          X.i <- X*exp(-0.5*H[,mm])

          V_post <- try(chol2inv(chol(crossprod(X.i)+diag(1/theta[,mm]))),silent=TRUE)
          if (is(V_post,"try-error")) V_post <- ginv(crossprod(X.i)+diag(1/theta[,mm]))
          A_post <- V_post%*%(crossprod(X.i,Y.i)+diag(1/theta[,mm])%*%A_prior[,mm])

          A.draw.i <- try(A_post+t(chol(V_post))%*%rnorm(ncol(X.i)),silent=TRUE)
          if (is(A.draw.i,"try-error")) A.draw.i <- mvrnorm(1,A_post,V_post)
          A_draw[,mm] <- A.draw.i
          Em[,mm] <-  Em.str[,mm] <- Y[,mm]-X%*%A.draw.i
        }else{
          Y.i <- Y[,mm]*exp(-0.5*H[,mm])
          X.i <- cbind(X,Em[,1:(mm-1)])*exp(-0.5*H[,mm])

          V_post <- try(chol2inv(chol((crossprod(X.i)+diag(1/c(theta[,mm],b_draw[mm,1:(mm-1)]))))),silent=TRUE)    #solve(crossprod(X.i)+diag(1/c(theta[,mm],b_draw[mm,1:(mm-1)])))
          if (is(V_post,"try-error")) V_post <- ginv((crossprod(X.i)+diag(1/c(theta[,mm],b_draw[mm,1:(mm-1)]))))
          A_post <- V_post%*%(crossprod(X.i,Y.i)+diag(1/c(theta[,mm],b_draw[mm,1:(mm-1)]))%*%c(A_prior[,mm],rep(0,mm-1)))

          A.draw.i <- try(A_post+t(chol(V_post))%*%rnorm(ncol(X.i)),silent=TRUE)
          if (is(A.draw.i,"try-error")) A.draw.i <- mvrnorm(1,A_post,V_post)

          A_draw[,mm] <- A.draw.i[1:ncol(X)]
          Em[,mm] <- Y[,mm]-X%*%A.draw.i[1:ncol(X)]
          Em.str[,mm] <- Y[,mm]-X%*%A.draw.i[1:ncol(X)]-Em[,1:(mm-1),drop=FALSE]%*%A.draw.i[(ncol(X)+1):ncol(X.i),drop=FALSE]
          Upsilon_draw[mm,1:(mm-1)] <- eta[[mm]]<- A.draw.i[(ncol(X)+1):ncol(X.i)]
        }
      }
      rownames(A_draw) <- colnames(X)

      alpha <- as.vector(A_draw)
      # Draw gamma|phi,psi,eta,omega,DATA from BERNOULLI dist.
      for (nn_6 in 1:n){
        u_i1  <-  (1/tau_0[nn_6])*exp(-0.5*(alpha[nn_6]/(tau_0[nn_6]))^2)*p_i
        u_i2  <-  (1/tau_1[nn_6])*exp(-0.5*(alpha[nn_6]/(tau_1[nn_6]))^2)*(1-p_i)
        gst  <-  u_i1/(u_i1 + u_i2)
        if (gst=="NaN") gst <- 0
        gammas[nn_6,1]  <-  BGVAR:::.bernoulli(gst)
        gammas[is.na(gammas)] <- 1
      }
      theta <-  matrix(0,n,1)   # h_i is tau_0 if gamma=0 and tau_1 if gamma=1
      for (nn_3 in 1:n){
        if (gammas[nn_3,1] == 0){
          theta[nn_3,1]  <-  tau_0[nn_3]^2
        }else if (gammas[nn_3,1] == 1){
          theta[nn_3,1]  <-  tau_1[nn_3]^2
        }
      }

      theta <- matrix(theta,K,M)


      if (sv){
        pars_var <- matrix(0,3,M)
         for (jj in 1:M){
          svdraw <- svsample2(Em.str[,jj],startpara=para(svl[[jj]]),startlatent=H[,jj],priorphi = c(25, 1.5))
          svl[[jj]] <- svdraw
          h_ <- exp(svdraw$latent)
          pars_var[,jj] <- para(svl[[jj]])
          H[,jj] <- log(h_)

          if (M==1){
             SIGMA <-Sigma1(H,bigT)
          }else{
             SIGMA <- Sigma2(Upsilon_draw,H,bigT)
          }
        }
      }else{
        for (jj in 1:M){
          S_1 <- a_i+bigT/2
          S_2 <- b_i+crossprod(Em.str[,jj])/2

          sig_eta <- 1/rgamma(1,S_1,S_2)
          H[,jj] <- log(sig_eta)
          if (M==1){
            SIGMA <- matrix(exp(H[bigT,]))
          }else{
            SIGMA <- Upsilon_draw%*%diag(exp(H[1,]))%*%t(Upsilon_draw)
          }
        }
      }
  if (M>1){
      # Create SIGMA
      eta_0 <- unlist(eta)
      #Step VI: Sample the prior scaling factors for the VC matrix
      M1 <- 0
      for (nn in 2:M){
        M1 <-M1+1
        et_0 <- eta[[nn]]
        for (jj in  1:M1){
          u_ij1  <-  (1/kappa_0)*exp(-0.5*((et_0[[jj]])^2)/((kappa_0)^2))*q_ij
          u_ij2  <-  (1/kappa_1)*exp(-0.5*((et_0[[jj]])^2)/((kappa_1)^2))*(1-q_ij)
          ost  <-  u_ij1/(u_ij1 + u_ij2)
          if (is.na(ost)) ost <- 1
          sample_omega <-  BGVAR:::.bernoulli(ost)
          if (is.na(sample_omega)) sample_omega <- 1
          if (sample_omega==1){
            b_draw[nn,jj] <- kappa_1^2
          }else{
            b_draw[nn,jj] <- kappa_0^2
          }
        }
      }
  }

      if (irep %in% thin.draws){
        gamma_draws[count,] <- gammas

        count <- count+1
        ALPHA_draws[count,,] <- A_draw
        SIGMA_draws[count,,] <- SIGMA
        res_store<-res_store+(Y-X%*%A_draw)
        if(sv){
          pars_store[count,,] <- pars_var
          hv_store[count,,] <- H
        }

      }
    }

    dimnames(ALPHA_draws)=list(NULL,colnames(X),colnames(A_OLS))
    #-----------------------------------------------END SSVS --------------------------------------------------------------
  }else if (prior=="SIMS"){ #-----------------------START SIMS ZHA---------------------------------------------------------
    #-------------------PRIOR Hyperparameters--------------------
    n <- K*N
    v_prior <- N
    v <- (M*(M-1))/2

    # Hyperparameters for diagonal elements of SIGMA (Gamma density)
    a_i  <-  hyper$a_i
    b_i  <-  hyper$b_i

    #Hyperparameters on a distributed as N(a_prior, SIGMA x V_prior)
    ind <- matrix(0,N,p)
    for (i in 1:N){
      ind[i,] <- seq(1+ncol(Wex)+ncol(Wexlag)+i,K, by=N)
    }

    A_prior <<- matrix(0,K,N)
    for (j in 1:N){
      if (j==2){
        A_prior[ind[j,1],j] <- 1
      }else{
        A_prior[ind[j,1],j] <- 1 #prior view that the variables follow a random walk
      }
    }
    S_prior <<- diag(N)*10 #0.1
    #Now get residual variances of univariate p-lag AR models.
    #AR(p) is run on each equation, ignoring the constant and exogenous variables
    #-------------------------------------------------------------------------
    sigma_sq  <- matrix(0,N,1) #vector which stores the residual variance
    for (i in 1:N){
      Ylag_i <- BGVAR:::.mlag(Yraw[,i],p)
      Ylag_i <- Ylag_i[(p+1):nrow(Ylag_i),]
      Y_i <- Yraw[(p+1):nrow(Yraw),i]
      alpha_i <- solve(t(Ylag_i)%*%Ylag_i)%*%t(Ylag_i)%*%Y_i
      sigma_sq[i,1] <- (1/(nrow(Yraw)-p))*t(Y_i-Ylag_i%*%alpha_i)%*%(Y_i-Ylag_i%*%alpha_i)
    }
    #------------------------------------------------------------------------
    sig_wex <<- matrix(0,M_,1)
    for (j in 1:M_){
      Ywex_i <- BGVAR:::.mlag(Wex[,j],p)
      Ywex_i <- Ywex_i[(p+1):nrow(Ywex_i),]
      Yw_i <- Wex[(p+1):nrow(Wex),j]
      alpha_w <- solve(crossprod(Ywex_i))%*%t(Ywex_i)%*%Yw_i
      sig_wex[j,1] <- (1/(nrow(Wex)-p))*t(Yw_i-Ywex_i%*%alpha_w)%*%(Yw_i-Ywex_i%*%alpha_w)
    }
    D <- diag(diag(SIGMA))

    a_prior <- as.vector(A_prior)

    beta_prior <- matrix(0,v,1)
    omega <- matrix(10^2,v,1)
    b_draw <- lower.tri(SIGMA_OLS)*10
    Upsilon_draw <- b_draw

    diag(Upsilon_draw) <- 1

    #SV
    svdraw <- list(para=c(mu=-10,phi=.9,sigma=.2),latent=rep(-3,T))
    svl <- list()
    for (jj in 1:M) svl[[jj]] <- svdraw

    hv <- svdraw$latent
    para <- list(mu=-3,phi=.9,sigma=.2)
    H <- matrix(-3,bigT,M)
    eta <- list()
    #MH preliminaries
    accept <- 0
    accept2 <- 0
    accept4 <- 0
    scale1 <- .43
    scale2 <- .45
    scale4 <- .45

    shrink.1 <- 0.1
    shrink.2 <- 0.2
    shrink.3 <- 10^2
    shrink.4 <- 0.1




    theta <- BGVAR:::.get_V(K=K,N=N,X=X,Wex=Wex,Wexlag=Wexlag,trend=trend,ind=ind,a_bar_1=shrink.1,a_bar_2=shrink.2,a_bar_3=shrink.3,a_bar_4=shrink.4,sigma_sq=sigma_sq,sig_wex=sig_wex,p=p)
    A_draw <- A_OLS
    for (irep in 1:ntot){
      #Step I: Sample autoregressive parameters
      for (mm in 1:M){
        #Slow step (if T>>K)
        #Sample autoregressive coefficients equation by equation
        if (mm==1){
          Y.i <- Y[,mm]*exp(-0.5*H[,mm])
          X.i <- X*exp(-0.5*H[,mm])

          V_post <- try(chol2inv(chol(crossprod(X.i)+diag(1/theta[,mm]))),silent=TRUE)
          if (is(V_post,"try-error")) V_post <- ginv(crossprod(X.i)+diag(1/theta[,mm]))
          A_post <- V_post%*%(crossprod(X.i,Y.i)+diag(1/theta[,mm])%*%A_prior[,mm])

          A.draw.i <- try(A_post+t(chol(V_post))%*%rnorm(ncol(X.i)),silent=TRUE)
          if (is(A.draw.i,"try-error")) A.draw.i <- mvrnorm(1,A_post,V_post)
          A_draw[,mm] <- A.draw.i
          Em[,mm] <-  Em.str[,mm] <- Y[,mm]-X%*%A.draw.i
        }else{
          Y.i <- Y[,mm]*exp(-0.5*H[,mm])
          X.i <- cbind(X,Em[,1:(mm-1)])*exp(-0.5*H[,mm])

          V_post <- try(chol2inv(chol((crossprod(X.i)+diag(1/c(theta[,mm],b_draw[mm,1:(mm-1)]))))),silent=TRUE)    #solve(crossprod(X.i)+diag(1/c(theta[,mm],b_draw[mm,1:(mm-1)])))
          if (is(V_post,"try-error")) V_post <- ginv((crossprod(X.i)+diag(1/c(theta[,mm],b_draw[mm,1:(mm-1)]))))
          A_post <- V_post%*%(crossprod(X.i,Y.i)+diag(1/c(theta[,mm],b_draw[mm,1:(mm-1)]))%*%c(A_prior[,mm],rep(0,mm-1)))

          A.draw.i <- try(A_post+t(chol(V_post))%*%rnorm(ncol(X.i)),silent=TRUE)
          if (is(A.draw.i,"try-error")) A.draw.i <- mvrnorm(1,A_post,V_post)

          A_draw[,mm] <- A.draw.i[1:ncol(X)]
          Em[,mm] <- Y[,mm]-X%*%A.draw.i[1:ncol(X)]
          Em.str[,mm] <- Y[,mm]-X%*%A.draw.i[1:ncol(X)]-Em[,1:(mm-1),drop=FALSE]%*%A.draw.i[(ncol(X)+1):ncol(X.i),drop=FALSE]
          Upsilon_draw[mm,1:(mm-1)] <- eta[[mm]]<- A.draw.i[(ncol(X)+1):ncol(X.i)]
        }
      }
      rownames(A_draw) <- colnames(X)

      #Step for the first shrinkage parameter (own lags)
      shrink.1.prop <- exp(rnorm(1,0,scale1))*shrink.1
      theta.prop <- BGVAR:::.get_V(K=K,N=N,X=X,Wex=Wex,Wexlag=Wexlag,trend=trend,ind=ind,a_bar_1=shrink.1.prop,a_bar_2=shrink.2,a_bar_3=shrink.3,a_bar_4=shrink.4,sigma_sq=sigma_sq,sig_wex=sig_wex,p=p)#get_V(shrink.1.prop,shrink.2,ind1=ind,sigma_sq1=sigma_sq,p1=p,set.cons=cons,M1=M,K1=K)

      post1.prop <- sum(dnorm(as.vector(A_draw),as.vector(A_prior),sqrt(as.vector(theta.prop)),log=TRUE))+dgamma(shrink.1.prop,0.01,0.01,log=TRUE)
      post1 <-   sum(dnorm(as.vector(A_draw),as.vector(A_prior),sqrt(as.vector(theta)),log=TRUE))+dgamma(shrink.1,.01,0.01,log=TRUE)
      if ((post1.prop-post1)>log(runif(1,0,1))){
        shrink.1 <- shrink.1.prop
        theta <- theta.prop
        accept <- accept+1
      }
      #Step for the second shrinkage parameter (cross equation)
      shrink.2.prop <- exp(rnorm(1,0,scale2))*shrink.2
      theta.prop <- BGVAR:::.get_V(K=K,N=N,X=X,Wex=Wex,Wexlag=Wexlag,trend=trend,ind=ind,a_bar_1=shrink.1,a_bar_2=shrink.2.prop,a_bar_3=shrink.3,a_bar_4=shrink.4,sigma_sq=sigma_sq,sig_wex=sig_wex,p=p)#get_V(shrink.1.prop,shrink.2,ind1=ind,sigma_sq1=sigma_sq,p1=p,set.cons=cons,M1=M,K1=K)

      post2.prop <- sum(dnorm(as.vector(A_draw),as.vector(A_prior),sqrt(as.vector(theta.prop)),log=TRUE))+dgamma(shrink.2.prop,0.01,0.01,log=TRUE)
      post2 <-   sum(dnorm(as.vector(A_draw),as.vector(A_prior),sqrt(as.vector(theta)),log=TRUE))+dgamma(shrink.2,0.01,0.01,log=TRUE)
      if ((post2.prop-post2)>log(runif(1,0,1))){
        shrink.2 <- shrink.2.prop
        theta <- theta.prop
        accept2 <- accept2+1
      }

      #Step for the final shrinkage parameter (weakly exogenous)
      shrink.4.prop <- exp(rnorm(1,0,scale4))*shrink.4
      theta.prop <- BGVAR:::.get_V(K=K,N=N,X=X,Wex=Wex,Wexlag=Wexlag,trend=trend,ind=ind,a_bar_1=shrink.1,a_bar_2=shrink.2,a_bar_3=shrink.3,a_bar_4=shrink.4.prop,sigma_sq=sigma_sq,sig_wex=sig_wex,p=p)#get_V(shrink.1.prop,shrink.2,ind1=ind,sigma_sq1=sigma_sq,p1=p,set.cons=cons,M1=M,K1=K)

      post4.prop <- sum(dnorm(as.vector(A_draw),as.vector(A_prior),sqrt(as.vector(theta.prop)),log=TRUE))+dgamma(shrink.2.prop,0.01,0.01,log=TRUE)
      post4 <-   sum(dnorm(as.vector(A_draw),as.vector(A_prior),sqrt(as.vector(theta)),log=TRUE))+dgamma(shrink.2,0.01,0.01,log=TRUE)
      if ((post4.prop-post4)>log(runif(1,0,1))){
        shrink.4 <- shrink.4.prop
        theta <- theta.prop
        accept4 <- accept4+1
      }

      #Step II: Sample hyperparameter lambda from G(a,b)

      #Step IV: Sample covariance parameters
      #Get orthogonalized errors
      #Sample stochastic volatilities
      if (sv){
        pars_var <- matrix(0,3,M)
        for (jj in 1:M){
          svdraw <- svsample2(Em.str[,jj],startpara=para(svl[[jj]]),startlatent=H[,jj],priorphi = c(25, 1.5))
          svl[[jj]] <- svdraw
          h_ <- exp(svdraw$latent)
          pars_var[,jj] <- para(svl[[jj]])
          H[,jj] <- log(h_)
          if (M==1){
            SIGMA <-Sigma1(H,bigT)
          }else{
            SIGMA <- Sigma2(Upsilon_draw,H,bigT)
          }
        }
      }else{
        for (jj in 1:M){
          S_1 <- a_i+bigT/2
          S_2 <- b_i+crossprod(Em.str[,jj])/2

          sig_eta <- 1/rgamma(1,S_1,S_2)
          H[,jj] <- log(sig_eta)
          if (M==1){
            SIGMA <- matrix(exp(H[bigT,]))
          }else{
            SIGMA <- Upsilon_draw%*%diag(exp(H[1,]))%*%t(Upsilon_draw)
          }
        }
      }
      # Create SIGMA
      eta_0 <- unlist(eta)
      #Tune the RW proposal during the first 50% of the burn-in stage
      if (irep<(0.5*nburn)){
        if ((accept/irep)>0.3) scale1 <- 1.01*scale1
        if ((accept/irep)<0.15) scale1 <- 0.99*scale1
        if ((accept2/irep)>0.3) scale2 <- 1.01*scale2
        if ((accept2/irep)<0.15) scale2 <- 0.99*scale2
        if ((accept4/irep)<0.15) scale4 <- 0.99*scale4
        if ((accept4/irep)>0.3) scale4 <- 1.01*scale4
      }



      if (irep %in% thin.draws){
        count <- count+1
        ALPHA_draws[count,,] <- A_draw
        SIGMA_draws[count,,] <- SIGMA
        theta_store[count,] <- c(shrink.1,shrink.2,shrink.4)
        res_store<-res_store+(Y-X%*%A_draw)
        if(sv){
          pars_store[count,,] <- pars_var
          hv_store[count,,] <- H
        }

      }

    } #--------------------------------------------END SIMS ZHA----------------------
  }else{ #-----------------------------------------START NORMAL GAMMA PRIOR WITH SV---------------------------------------------------------#

    c_tau <- hyper$c_tau
    d_tau <- hyper$d_tau

    # Hyperparameters for diagonal elements of SIGMA (Gamma density)
    a_i  <-  hyper$a_i
    b_i  <-  hyper$b_i

    prmean <- hyper$prmean

    k <- K*M
    v <- (M*(M-1))/2

    A_OLS <- A_draw <- ginv(crossprod(X))%*%crossprod(X,Y)
    Em <-Em.str <-  Y-X%*%A_OLS
    SIGMA <- SIGMA_OLS <- crossprod(Y-X%*%A_OLS)/(bigT-K)
    #chol_SIG <- t(chol(SIGMA_OLS))
    D <- diag(diag(SIGMA))
    V_prior <- diag(k)*10^2
    V_priorinv <- diag(1/diag(V_prior))

    A_prior <- matrix(0,K,M)
    A_prior[1:M,1:M] <- diag(M)*prmean
    a_prior <- as.vector(A_prior)
    a_tau <- hyper$a_start
    sample_A<-hyper$sample_A


    b_tau <- hyper$a_start
    beta_prior <- matrix(0,v,1)
    theta <- matrix(10,K,M)
    rownames(theta) <- colnames(X)
    omega <- matrix(10^2,v,1)
    b_draw <- lower.tri(SIGMA_OLS)*1
    Upsilon_draw <- b_draw


    diag(Upsilon_draw) <- 1

    #SV
    svdraw <- list(para=c(mu=-10,phi=.9,sigma=.2),latent=rep(-3,bigT))
    svl <- list()
    for (jj in 1:M) svl[[jj]] <- svdraw

    hv <- svdraw$latent
    para <- list(mu=-3,phi=.9,sigma=.2)
    H <- matrix(-3,bigT,M)
    eta <- list()
    #MH preliminaries
    accept <- 0
    accept2 <- 0
    scale1 <- .43
    scale2 <- .45
    #Create storage matrices
    #thinsaves <- round(thin*nsave)
    ALPHA_draws <- array(0,c(thinsaves,K,M))
    SIGMA_draws <- array(0,c(thinsaves,M,M))
    theta_store <- array(0,c(thinsaves,K,M))
    omega_store <- array(0,c(thinsaves,M,M))
    Upsilon_store <- array(0,c(thinsaves,M,M))
    hv_store <- array(0,c(thinsaves,bigT,M))


    pars_store <- array(0,c(thinsaves,3,M))
    count <- 0

    kappa_j <- matrix(0.01,p,1)
    a_tau <- matrix(hyper$a_start,p,1)
    #TO INCLUDE
    var.c1 <- hyper$e_lambda
    var.c2 <- hyper$d_lambda
    c0 <- hyper$c_tau
    c1 <- hyper$d_tau

    kappa_j_star <- matrix(0.01,p+1,1)
    a_tau_star <- matrix(hyper$a_start,p+1,1)
    a_store_star <- matrix(0,nsave,p+1)
    kappa_store_star <- matrix(0,nsave,p+1)
    #-Start big Gibbs loop-------
    # pb <- txtProgressBar(min = 0, max = ntot, style = 3) #start progress bar
    start <- Sys.time()
    for (irep in 1:ntot){
      #Step I: Sample autoregressive parameters
      for (mm in 1:M){
        #Slow step (if T>>K)
        #Sample autoregressive coefficients equation by equation
        if (mm==1){
          Y.i <- Y[,mm]*exp(-0.5*H[,mm])
          X.i <- X*exp(-0.5*H[,mm])

          V_post <- try(chol2inv(chol(crossprod(X.i)+diag(1/theta[,mm]))),silent=TRUE)
          if (is(V_post,"try-error")) V_post <- ginv(crossprod(X.i)+diag(1/theta[,mm]))
          A_post <- V_post%*%(crossprod(X.i,Y.i)+diag(1/theta[,mm])%*%A_prior[,mm])

          A.draw.i <- try(A_post+t(chol(V_post))%*%rnorm(ncol(X.i)),silent=TRUE)
          if (is(A.draw.i,"try-error")) A.draw.i <- mvrnorm(1,A_post,V_post)
          A_draw[,mm] <- A.draw.i
          Em[,mm] <-  Em.str[,mm] <- Y[,mm]-X%*%A.draw.i
        }else{
          Y.i <- Y[,mm]*exp(-0.5*H[,mm])
          X.i <- cbind(X,Em[,1:(mm-1)])*exp(-0.5*H[,mm])

          V_post <- try(chol2inv(chol((crossprod(X.i)+diag(1/c(theta[,mm],b_draw[mm,1:(mm-1)]))))),silent=TRUE)    #solve(crossprod(X.i)+diag(1/c(theta[,mm],b_draw[mm,1:(mm-1)])))
          if (is(V_post,"try-error")) V_post <- ginv((crossprod(X.i)+diag(1/c(theta[,mm],b_draw[mm,1:(mm-1)]))))
          A_post <- V_post%*%(crossprod(X.i,Y.i)+diag(1/c(theta[,mm],b_draw[mm,1:(mm-1)]))%*%c(A_prior[,mm],rep(0,mm-1)))

          A.draw.i <- try(A_post+t(chol(V_post))%*%rnorm(ncol(X.i)),silent=TRUE)
          if (is(A.draw.i,"try-error")) A.draw.i <- mvrnorm(1,A_post,V_post)

          A_draw[,mm] <- A.draw.i[1:ncol(X)]
          Em[,mm] <- Y[,mm]-X%*%A.draw.i[1:ncol(X)]
          Em.str[,mm] <- Y[,mm]-X%*%A.draw.i[1:ncol(X)]-Em[,1:(mm-1),drop=FALSE]%*%A.draw.i[(ncol(X)+1):ncol(X.i),drop=FALSE]
          Upsilon_draw[mm,1:(mm-1)] <- eta[[mm]]<- A.draw.i[(ncol(X)+1):ncol(X.i)]
        }
      }
      rownames(A_draw) <- colnames(X)
      #Step II: Scalings for the weakly exogenous variables
      for (ss in 0:p){
        if (ss==0) slct.i <- which(rownames(A_draw)=="Wex") else slct.i <- which(rownames(A_draw)==paste("Wexlag",ss,sep=""))
        A.lag.star <- A_draw[slct.i,,drop=FALSE]
        theta.lag <- theta[slct.i,,drop=FALSE]
        if (is.null(dim(theta.lag))) theta.lag <- matrix(theta.lag)

        Mstar <- ncol(A.lag.star)*nrow(A.lag.star)
        if (ss==0){
          kappa_j_star[ss+1,1] <- rgamma(1,var.c1+a_tau_star[[ss+1]]*Mstar,var.c2+a_tau_star[[ss+1]]/2*sum(theta.lag))
        }else{
          kappa_j_star[ss+1,1] <- rgamma(1,var.c1+a_tau_star[[ss+1]]*Mstar,var.c2+a_tau_star[[ss+1]]/2*prod(kappa_j_star[ss,1])*sum(theta.lag))
        }
        for (jj in 1:nrow(theta.lag)){
          for (ii in 1:ncol(theta.lag)){
            theta.lag[jj,ii] <- rgig(n=1,lambda=a_tau_star[ss+1]-0.5,(A.lag.star[jj,ii])^2,a_tau_star[ss+1]*prod(kappa_j_star[1:(ss+1),1]))
          }
        }
        theta[slct.i,] <- theta.lag
        theta[theta<1e-7] <- 1e-7

        #TO BE MODIFIED
        if (sample_A){
          #Sample a_tau through a simple RWMH step (on-line tuning of the MH scaling within the first 50% of the burn-in phase)
          a_prop <- exp(rnorm(1,0,scale1))*a_tau_star[ss+1]
          post_a_prop <- BGVAR:::.atau_post(atau=a_prop,thetas = as.vector(theta.lag),lambda2 = prod(kappa_j[1:(ss),1]))
          post_a_old <- BGVAR:::.atau_post(atau=a_tau_star[ss+1],thetas =as.vector(theta.lag),lambda2 = prod(kappa_j[1:(ss),1]))
          post.diff <- post_a_prop-post_a_old
          post.diff <- ifelse(is.nan(post.diff),-Inf,post.diff)

          if (post.diff > log(runif(1,0,1))){
            a_tau_star[ss+1] <- a_prop
            #accept <- accept+1
          }
        }
      }


      #Step III: Scalings for the endogenous variables
      for (ss in 1:p){
        slct.i <- which(rownames(A_draw)==paste("Ylag",ss,sep=""))
        A.lag <- A_draw[slct.i,,drop=FALSE]
        theta.lag <- theta[slct.i,,drop=FALSE]
        if (is.null(dim(theta.lag))) theta.lag <- matrix(theta.lag)
        M.end <- ncol(A.lag.star)*nrow(A.lag.star)
        if (ss==1){
          kappa_j[ss,1] <- rgamma(1,var.c1+a_tau[[ss]]*M.end,var.c2+a_tau[[ss]]/2*sum(theta.lag))
        }else{
          kappa_j[ss,1] <- rgamma(1,var.c1+a_tau_star[[ss+1]]*M.end,var.c2+a_tau_star[[ss+1]]/2*prod(kappa_j_star[ss,1])*sum(theta.lag))
        }
        for (jj in 1:nrow(theta.lag)){
          for (ii in 1:ncol(theta.lag)){
            theta.lag[jj,ii] <- rgig(n=1,lambda=a_tau[ss]-0.5,(A.lag[jj,ii])^2,a_tau[ss]*prod(kappa_j[1:(ss),1]))
          }
        }
        theta[slct.i,] <- theta.lag
        theta[theta<1e-8] <- 1e-8
        #TO BE MODIFIED
        if (sample_A){
          #Sample a_tau through a simple RWMH step (on-line tuning of the MH scaling within the first 50% of the burn-in phase)
          a_prop <- exp(rnorm(1,0,scale1))*a_tau[ss]
          post_a_prop <- BGVAR:::.atau_post(atau=a_prop,thetas = as.vector(theta.lag),lambda2 = prod(kappa_j[1:(ss),1]))
          post_a_old <- BGVAR:::.atau_post(atau=a_tau[ss],thetas =as.vector(theta.lag),lambda2 = prod(kappa_j[1:(ss),1]))
          post.diff <- post_a_prop-post_a_old
          post.diff <- ifelse(is.nan(post.diff),-Inf,post.diff)

          if (post.diff > log(runif(1,0,1))){
            a_tau[ss] <- a_prop
            #accept <- accept+1
          }
        }
      }


      #Step II: Sample hyperparameter lambda from G(a,b)

      #Step IV: Sample covariance parameters
      #Get orthogonalized errors
      #Sample stochastic volatilities
      if (sv){
        pars_var <- matrix(0,3,M)
        for (jj in 1:M){
          svdraw <- svsample2(Em.str[,jj],startpara=para(svl[[jj]]),startlatent=H[,jj],priorphi = c(25, 1.5))
          svl[[jj]] <- svdraw
          h_ <- exp(svdraw$latent)
          pars_var[,jj] <- para(svl[[jj]])
          H[,jj] <- log(h_)
          if (M==1){
            SIGMA <-Sigma1(H,bigT)
          }else{
            SIGMA <- Sigma2(Upsilon_draw,H,bigT)
          }
        }
      }else{
        for (jj in 1:M){
          S_1 <- a_i+bigT/2
          S_2 <- b_i+crossprod(Em.str[,jj])/2

          sig_eta <- 1/rgamma(1,S_1,S_2)
          H[,jj] <- log(sig_eta)
          if (M==1){
            SIGMA <- matrix(exp(H[bigT,]))
          }else{
            SIGMA <- Upsilon_draw%*%diag(exp(H[1,]))%*%t(Upsilon_draw)
          }
        }
      }
      if (M>1){
      # Create SIGMA
      eta_0 <- unlist(eta)

      #Step V: Draw prior scaling factors from GIG for covariances
      lambda_omega <- rgamma(1,c_tau+b_tau*v,d_tau+b_tau/2*sum(b_draw[lower.tri(b_draw)]))
      #Step VI: Sample the prior scaling factors for covariances from GIG
      M1 <- 0
      for (nn in 2:M){
        M1 <-M1+1
        et_0 <- eta[[nn]]
        for (jj in  1:M1){
          b_draw[nn,jj] <- rgig(n=1,lambda=b_tau-0.5,et_0[[jj]]^2,b_tau*lambda_omega)
        }
      }
      if (sample_A){
        #Sample b_tau through a simple RWMH step
        b_prop <- exp(rnorm(1,0,scale2))*b_tau
        post_b_prop <- BGVAR:::.atau_post(atau=b_prop,thetas = b_draw[lower.tri(b_draw)],k = length(b_draw[lower.tri(b_draw)]),lambda2=lambda_omega)
        post_b_old <- BGVAR:::.atau_post(atau=b_tau,thetas = b_draw[lower.tri(b_draw)],k = length(b_draw[lower.tri(b_draw)]),lambda2=lambda_omega)
        post.diff <- post_b_prop-post_b_old
        post.diff <- ifelse(is.nan(post.diff),-Inf,post.diff)
        if (post.diff > log(runif(1,0,1))){

          b_tau <- b_prop
          accept2 <- accept2+1
        }
        if (irep<(0.5*nburn)){
          if ((accept2/irep)>0.3)  scale2 <- 1.01*scale2
          if ((accept2/irep)<0.15) scale2 <- 0.99*scale2
        }
      }
      }
      #Step VIII: Store draws after burn-in/ Compute forecasts/ Impulse responses etc.
      if (irep %in% thin.draws){
        count <- count+1
        ALPHA_draws[count,,] <- A_draw
        SIGMA_draws[count,,] <- SIGMA
        theta_store[count,,] <- matrix(theta,K,M)
        omega_store[count,,] <- b_draw
        Upsilon_store[count,,] <- solve(Upsilon_draw)
        kappa_store[count,,2] <- kappa_j_star
        kappa_store[count,1:p,1] <- kappa_j
        a_store[count,,2] <- a_tau_star
        a_store[count,1:p,1] <- a_tau
        res_store<-res_store+(Y-X%*%A_draw)
        if(sv){
          pars_store[count,,] <- pars_var
          hv_store[count,,] <- H
        }
        # a_store[count,] <- c(a_tau,b_tau)
      }
    }
    #-----------------------------------------END NORMAL GAMMA PRIOR WITH SV---------#
  }
  #-----------------------------------------END ESTIMATION---------------------------------------------------------#

  A_post <- apply(ALPHA_draws, c(2,3), mean);rownames(A_post) <- colnames(X);colnames(A_post) <- colnames(A_OLS)
  S_post <- apply(SIGMA_draws,c(2,3),mean)
  PIP <- matrix(apply(gamma_draws,2,mean),K,N)
  res_store<-res_store/thinsaves
  colnames(PIP) <- colnames(A_post);rownames(PIP) <- rownames(A_post)
  #Res <- Y-X%*%A_post # these are country residuals based on the posterior mean

  SIGMA <- S_post/(bigT-K)
  dimnames(ALPHA_draws)=list(NULL,colnames(X),colnames(A_OLS))
  dims <- dimnames(ALPHA_draws)[[2]]

  #splitting the output of the gibbs sampler for predictive density later on
  if (M==1){
    a0post <- matrix(ALPHA_draws[,which(dims=="constant"),])
  }else{
  a0post <- ALPHA_draws[,which(dims=="constant"),]
  }
  a1post <- ALPHA_draws[,which(dims=="trend"),] #coefficients on the trend
  a2post <- NULL
  alpha_var <- NULL
  dummiespost <- ALPHA_draws[,which(dims=="Dummies"),,drop=FALSE] #coefficients for the dummies
  postExpost <- ALPHA_draws[,which(dims=="Exogenous"),,drop=FALSE] #coefficients for the cont.exogenous
  postExlpost <- ALPHA_draws[,which(dims=="ExogenousLag"),,drop=FALSE] #coefficients for the lagged. ex
  Lambda0post <- ALPHA_draws[,which(dims=="Wex"),,drop=FALSE]#coefficients on WEX
  SIGMApost <- SIGMA_draws

  Lambdapost <- NULL
  Thetapost <- NULL

  for (jj in 1:p) {
    Lambdapost[[jj]] <- ALPHA_draws[,which(dims==paste("Wexlag",jj,sep="")),,drop=FALSE]
    Thetapost[[jj]]<- ALPHA_draws[,which(dims==paste("Ylag",jj,sep="")),,drop=FALSE]
  }
  #--------------Splits the posterior matrix & creates residuals---------------------
  dims <- dimnames(A_post)[[1]]
  fit <- X%*%A_post
  Lik <- 0
  ML <- 0
  rownames(A_post) <- colnames(X);colnames(A_post) <- colnames(A_OLS)

  a0 <- A_post[which(dims=="constant"),] #coefficients on the constant
  a1 <- A_post[which(dims=="trend"),] #coefficients on the trend
  a2 <- NULL
  dummies <- A_post[which(dims=="Dummies"),] #coefficients for the dummies
  postEx <- A_post[which(dims=="Exogenous"),] #coefficients for the cont.exogenous
  postExl <- A_post[which(dims=="ExogenousLag"),] #coefficients for the lagged. ex
  Lambda0 <- A_post[which(dims=="Wex"),]#coefficients on WEX
  Lambda <- A_post[which(dims=="Wexlag"),] #coefficients on Wexlag
  Theta <- A_post[which(dims=="Ylag"),]#coefficients on endogenous lagged

  Lambda <- NULL
  Theta <- NULL

  for (jj in 1:p) {
    Lambda[[jj]] <- A_post[which(dims==paste("Wexlag",jj,sep="")),]
    Theta[[jj]]<- A_post[which(dims==paste("Ylag",jj,sep="")),]
  }
  #Data Splitting cbind(Dummies,Exogenous,Exlag,Wex,Wexlag,Ylag)
  DDummies <- Xfull[,which(dims=="Dummies")]
  DExogenous <- Xfull[,which(dims=="Exogenous")]
  DExogenousL <- Xfull[,which(dims=="ExogenousLag")]
  DWex <- Xfull[,which(dims=="Wex")]
  DWexlag <- Xfull[,which(dims=="Wexlag")]
  DEndlag <- Ylag[(p+1):nrow(Ylag),]

  if (sv){
    vola.store <- apply(hv_store,c(2:3),median);colnames(vola.store)<-colnames(A_OLS)
  }else{
    vola.store <- NULL
  }
  if(prior=="NG"){
    kappa<-apply(kappa_store,(2:3),median)
    a_tau<-apply(a_store,(2:3),median)
    colnames(kappa)<-colnames(a_tau)<-c("endo","weakly exo")
    rownames(kappa)<-rownames(a_tau)<-paste("lag",0:p,sep="_")
  }else{
    kappa<-a_tau<-NA
  }


  return(list(Dummies=DDummies,End=Yfull,Exogenous=DExogenous,Exlag=DExogenousL,Wex=DWex,Wexlag=DWexlag,Ylag=DEndlag,A_post=A_post,country.res=res_store,
              a0=a0,a1=a1,a2=a2,dummies=dummies,postEx=postEx,postExl=postExl,Lambda0=Lambda0,Lambda=Lambda,Theta=Theta,p=p,bigX=X,X=X,W=W,a0post=a0post,a1post=a1post,a2post=a2post,
              dummiespost=dummiespost,postExpost=postExpost,postExlpost=postExlpost,Lambda0post=Lambda0post,Lambdapost=Lambdapost,Thetapost=Thetapost,SIGMApost=SIGMApost,PL=PL,
              ML=ML,SIGMA=SIGMA,PIP=PIP,V_post=V_post,vola.store=vola.store,kappa=kappa,theta=theta_store,a_tau=a_tau))
}




.dmean <- function(x){

  xnew <- (x-mean(x))/sd(x)
  return(xnew)
}



.wish <- function(h,n){
  A = t(chol(h))%*%matrix(rnorm(size(h,1)*n,0,1),size(h,1),n)
  A = A%*%t(A);
}

.mlag <- function(X,lag)
{
  p <- lag
  X <- as.matrix(X)
  Traw <- nrow(X)
  N <- ncol(X)
  Xlag <- matrix(0,Traw,p*N)
  for (ii in 1:p){
    Xlag[(p+1):Traw,(N*(ii-1)+1):(N*ii)]=X[(p+1-ii):(Traw-ii),(1:N)]
  }
  return(Xlag)
}

.bernoulli <- function(p){
  u <- runif(1)
  if (u<p){
    x=0
  }else{
    x=1
  }
  return(x)
}





#-----------------------RMSE function-----------------------------------#
.rmse <- function(x,y) sqrt(mean((x-y)^2))
#-------------------------------------------------------------------------------------------------------------------#




.datahandling <- function(Data,nr){
  #---------------------------------------------------------------------------------------------------------------#
  #Input:Data and the number of the country in the dataset which is the current home country
  #Output: Data_Country , a list, where Data_Country[[2]] refers to the true exogenous variables, Data_Country[[3]]
  #are the weak exogenous variables and Data_Country[[4]] are the endogenous variables and Country[[1]] are the mn
  #This function feeds in DATA and creates a 1 x K vector which indicates which variables are exogenous, own lags
  #lags of other variables
  #indicator 0 means home country, 1 denotes the rest of the world and 2 are truly exogenous variables
  #WARNING: original data needs labels for the countries in order to create the indicator vector
  #---------------------------------------------------------------------------------------------------------------#
  Daten <- as.data.frame(Data) #reads in  data and converts it to a data.frame
  Names  <-  colnames(Daten)
  cols <- ncol(Daten)
  indicator  <-  matrix("NA",1,ncol(Daten))
  ind  <- substr(Names,1,2)

  number  <-nr #indicates which country is selected

  IND <- ind[!duplicated(ind)]

  for (i in 1:cols){

    if (grepl(IND[number],Names[i]))  {

      indicator[i]=0

    }else indicator[i]=1
    if(grepl(IND[length(IND)],Names[i])) indicator[i]=2

  }

  sortingM <- rbind(indicator,as.matrix(Daten))

  #optional: would put the choosen country's columns at the start of the matrix, using indicator would be more efficient i guess
  sort1.data <- sortingM[,order(sortingM[1,],decreasing="TRUE")]

  #splitting the data into (true) exogenous, weak exogenous and system variables
  #could be more elegant, but it works
  ex <- as.matrix(which(sort1.data[1,]==2, arr.ind=TRUE))
  wex <- as.matrix(which(sort1.data[1,]==1, arr.ind=TRUE))
  end <- as.matrix(which(sort1.data[1,]==0, arr.ind=TRUE))
  #Exogenous variables need extra treatment to extract dummys
  Exogenous <- sort1.data[2:nrow(sort1.data),ex]
  Exnames <- colnames(Exogenous)

  #inddummy <- substr(Exnames,7,10) # this is defunct
  dummies <- matrix("NA",1,ncol(Exogenous))

  for (j in 1:ncol(Exogenous)){

    if (grepl("Dumm",Exnames[j])) dummies[j]=1 else dummies[j]=0
  }
  class(dummies) <- "numeric"

  Exogenous <- rbind(dummies,Exogenous)
  sort.Ex <- Exogenous[,order(Exogenous[1,],decreasing="TRUE")]

  tex <- as.matrix(which(sort.Ex[1,]==0),arr.ind=TRUE)
  dum <- as.matrix(which(sort.Ex[1,]==1),arr.ind=TRUE)

  Dummies <- sort.Ex[2:nrow(sort.Ex),dum]
  Exogenous <- sort.Ex[2:nrow(sort.Ex),tex]


  WExogenous <- sort1.data[2:nrow(sort1.data),wex]
  Endogenous <- sort1.data[2:nrow(sort1.data),end]

  assign(paste("WEx_",IND[number],sep=""),WExogenous) #creates a matrix WEx_Countryname
  assign(paste("END_",IND[number],sep=""),Endogenous) # same, we dont need this for Exogenous because it stays the same for all countries
  assign(paste("DataFULL_",IND[number],sep=""),rbind(indicator,as.matrix(Daten))) #returns the full matrix with a indicator row

  assign(paste("",IND[number],sep=""),list(IND,Dummies,Exogenous,assign(paste("WEx_",IND[number],sep=""),WExogenous),assign(paste("END_",IND[number],sep=""),Endogenous)))

  return(get(IND[nr]))
}

.construct.arglist = function (funobj, envir = NULL)
{
  namedlist = formals(funobj)
  argnames = names(namedlist)
  if (!is.environment(envir))
    envir = sys.frame(-1)
  for (argn in 1:length(namedlist)) {
    testval = as.logical(try(exists(argnames[argn], envir = envir),
                             silent = TRUE))
    if (is.na(testval))
      testval = FALSE
    if (testval) {
      testout = try(get(argnames[argn], envir = envir),silent = TRUE)
      if (is.null(testout)) {
        namedlist[[argn]] = "list(NULL)blabla"
      } else {
        namedlist[[argn]] = testout
      }
    }
  }
  namedlist = lapply(namedlist,function(x) if (any(x=="list(NULL)blabla")) NULL else x)
  return(namedlist)
}




getweights <- function(W,Data, EA.weights=NULL,OC.weights=NULL,wex.restr=NULL, variable.list=NULL){
  ord<-names(Data)
  nn<-lapply(Data,colnames)
  exo=which(table(unlist(nn))==1) #exo variables are those that are only in one country model
  endo<-which(!table(unlist(nn))==1)
  exo.countries<-names(which(sapply(lapply(nn,function(y) names(exo)%in%y),any)))#gives you a vector of exo countries
  EA.flag<-OC.flag<-FALSE

  if(is.list(W)&&length(W)>1){
    if(is.null(variable.list)){
      stop("You have submitted more than 1 weight matrix but not specified the according variable sets.")
    }
    if(length(W)!=length(variable.list)){
      stop("You have submitted more than 1 weight matrix but not the same number of variable sets.")
    }
    if(!all(names(endo)%in%unlist(variable.list))){
      stop("You have submitted more than 1 weight matrix but some of the variables are not assigned to a weight matrix by the variable list")
    }
    W.sets<-length(W)
  }
  if(!is.list(W)){ # make sure W is a list
    W.aux<-list();W.aux$W=W;W<-W.aux;rm(W.aux)
  }
  if(is.null(variable.list)){
    variable.list<-list();variable.list$vars<-names(endo)
  }


  # in case we use ECB model, cut ECB data and proceed as before
  if("EB" %in% exo.countries){
    EA.flag<-TRUE
    if(is.null(EA.weights)){
      stop("Please provide a list that contains weights and the names of variables to aggregate for the ECB country model 'EB' as explained in the helpfiles.")
    }
    EB<-Data$EB;class(EB)<-"numeric"
    Data<-Data[-charmatch("EB",names(Data))]
    EB.vars<-EA.weights$variables
    if(!all(EB.vars%in%c(names(exo),names(endo)))){
      stop("The variables you have selected to specify the Taylor rule for the central bank country model are not contained in the data. Please re-specify.")
    }
    EB.exo<-EA.weights$exo
    if(is.null(EB.exo)){
      EB.exo<-colnames(EB)
    }
    EA.weights<-EA.weights$weights
    if(is.null(names(EA.weights)) || !all(names(EA.weights)%in%names(Data))){
      stop("Either you have not provided names attached to the weights for the EB model or the one of the names is not contained in the country Data..")
    }

  }
  # in case we have a country model for oil prices labeled OC
  if("OC" %in% exo.countries){
    if(is.null(OC.weights)){
      stop("Please provide a list that contains weights and the names of variables to aggregate for the oil country model 'OC' as explained in the helpfiles.")
    }
    OC<-Data$OC;class(OC)<-"numeric";OC.flag<-TRUE
    Data<-Data[-charmatch("OC",names(Data))]
    OC.vars<-OC.weights$variables
    if(!all(OC.vars%in%c(names(exo),names(endo)))){
      stop("The variables you have selected to specify the Taylor rule for the central bank country model are not contained in the data. Please re-specify.")
    }
    OC.exo<-OC.weights$exo
    if(is.null(OC.exo)){
      OC.exo<-colnames(OC)
    }
    OC.weights<-OC.weights$weights
    if(is.null(names(OC.weights)) || !all(names(OC.weights)%in%names(Data))){
      stop("Either you have not provided names attached to the weights for the oil country model or the one of the names is not contained in the country Data.")
    }
  }




  gW<-list()
  # exchange rate specification, if er is a domestic variable, no er* is included, if not, er* is included
  if(!is.null(wex.restr)){
    er.idx<-sapply(nn,function(x) wex.restr %in% x)
    er.idx<-which(!er.idx)
  }else{
    er.idx<-c()
  }


  data1 <- BGVAR:::.datahandling(Data,nr=1)

  cnames <- data1[[1]]
  #make sure that W and Data names are in the same order
  W<-lapply(W,function(x) x[cnames,cnames])


  xglobal <- Data[[1]]
  class(xglobal) <- "numeric"
  colnames(xglobal) <- paste(cnames[1],colnames(xglobal),sep=".")
  xglobal <- xglobal[,colSums(is.na(xglobal))<nrow(xglobal)]

  for (jj in 2:length(Data)){
    temp <- Data[[jj]];class(temp) <- "numeric"
    temp <- temp[,colSums(is.na(temp))<nrow(temp)]
    colnames(temp) <- paste(cnames[jj],colnames(temp),sep=".")
    xglobal <- cbind(xglobal,temp)
  }

  #varnames<-c(names(endo),names(exo))
  Max.char<-max(nchar(colnames(xglobal)))
  # for each country
  for(nr in 1:length(cnames)){
    colnames(xglobal) <- colnames(xglobal); xglobal <- xglobal[,!duplicated(colnames(xglobal))]#xglobal <- xglobal[,-which(colnames(xglobal)=="EG.stir")]
    varnames <- substr(colnames(xglobal),4,Max.char); varnames <- varnames[!duplicated(varnames)] #creates a dynamic list of variables

    if(length(er.idx)>0){
      if(length(er.idx)>0&&nr%in%er.idx){
        varnames <- varnames
      }else{
        varnames <- varnames[-charmatch(wex.restr,varnames)]
      }
    }
    # names of endo variables
    endnames <- substr(colnames(xglobal[,substr(colnames(xglobal),1,2)==cnames[[nr]]]),4,Max.char);endnames <- endnames[!duplicated(endnames)]

    Wnew <- matrix(0,length(varnames),ncol(xglobal));colnames(Wnew) <- colnames(xglobal);rownames(Wnew) <- varnames

    #wghts <- W[nr,] # this was the case for just one weight matrix
    for (k in 1:(nrow(Wnew))){
      replace<-substr(names(Wnew[k,substr(colnames(Wnew),4,Max.char)==rownames(Wnew)[k]]),1,2)# gives you names of countries and weights
      if (length(replace)>0){
        wghts<-W[[which(sapply(variable.list,function(x) rownames(Wnew)[k]%in% x))]][nr,] # this selects the weight matrix according to the variable we want to weight (financial, real, etc)
        Wnew[k,substr(colnames(Wnew),4,Max.char)==rownames(Wnew)[k]] <- wghts[replace] # this gives the variable name (e.g., y)
      }else{# in case we have exo variables
        # this includes the exo variables in all country models and the exo countries
        Wnew[k,substr(colnames(Wnew),4,Max.char)==rownames(Wnew)[k]] <- 1
        # in case we look at an exo country, where the variable is endog. defined, we have to set the exo variable to zero
        if(cnames[[nr]]%in%exo.countries&&paste(cnames[nr],rownames(Wnew)[k],sep=".")%in%colnames(Wnew)){
          Wnew[k,substr(colnames(Wnew),4,Max.char)==rownames(Wnew)[k]] <- 0
        }

      }
    }


    Wnew[which(Wnew=="NaN")] <- 1
    #----------------------here we specify the part for the endogenous variabes-----------------------------------------#
    endoW <- matrix(0,length(endnames),ncol(Wnew))
    endonr <- xglobal[,substr(colnames(xglobal),1,2)==cnames[nr]];rownames(endoW) <- colnames(endonr)
    colnames(endoW) <- colnames(Wnew)
    namesW <- colnames(endoW)
    namesNr <- colnames(endonr)

    for (j in 1:nrow(endoW)){
      for (i in 1:length(namesW)){
        if(namesNr[[j]]==namesW[[i]]){
          endoW[j,i]=1
        }
      }
    }
    WfinNR <- rbind(endoW,Wnew)

    WfinNR <- WfinNR[!(rowSums(abs(WfinNR)) == 0),]
    gW[[nr]]<-apply(WfinNR,2,function(x) x/(rowSums(WfinNR)))
  }
  names(gW)<-cnames


  #include weights part for ECB model
  if(EA.flag){
    xglobal<-cbind(xglobal,EB);EB.x<-ncol(EB)
    colnames(xglobal)[(ncol(xglobal)-EB.x+1):ncol(xglobal)]<-paste("EB",colnames(EB),sep=".")
    EBnames<-colnames(EB)
    # this puts the EB variables as exo variable for all countries

    for(i in 1:length(cnames)){
      aux<-gW[[cnames[i]]];ii<-nrow(aux)
      aux<-rbind(cbind(aux,matrix(0,ncol=EB.x,nrow=nrow(aux))),matrix(0,nrow=length(EB.exo),ncol=c(ncol(aux)+EB.x)))
      rownames(aux)[(ii+1):(ii+length(EB.exo))]<-EB.exo
      colnames(aux)[(ncol(aux)-EB.x+1):ncol(aux)]<-paste("EB",EBnames,sep=".")
      if(length(EB.exo)>1){
        #diag(aux[(ii+1):(ii+length(EB.exo)),(ncol(aux)-EB.x+1):ncol(aux)])<-1
        diag(aux[EB.exo,paste0("EB.",EB.exo)])<-1
      }else{
        #aux[(ii+1):(ii+EB.x),(ncol(aux)-EB.x+1):ncol(aux)]<-1
        aux[EB.exo,paste0("EB.",EB.exo)]<-1
      }
      # set the variables we do not want to have specified as exo for other countries to zero
      #aux[,paste0("EB.",EBnames[!EBnames%in%EB.exo])]<-0
      gW[[cnames[i]]]<-aux
    }

    # this creates the W matrix for the ECB model
    Wnew <- matrix(0,(ncol(EB)+length(EB.vars)),ncol(xglobal));colnames(Wnew) <- colnames(xglobal);
    EBnames<-paste0("EB.",colnames(EB))
    rownames(Wnew) <- c(EBnames,EB.vars)
    if(EB.x>1){
      diag(Wnew[EBnames,EBnames])<-1
    }else{
      Wnew[EBnames,EBnames]<-1
    }

    for(i in 1:length(EB.vars)){
      Wnew[EB.vars[i],paste0(names(EA.weights),".",EB.vars[i])]<-EA.weights
    }
    gW[[(length(gW)+1)]]<-Wnew
    names(gW)[length(gW)]<-"EB"

  }


  if(OC.flag){
    xglobal<-cbind(xglobal,OC);OC.x<-ncol(OC)
    colnames(xglobal)[(ncol(xglobal)-OC.x+1):ncol(xglobal)]<-paste("OC",colnames(OC),sep=".")
    OCnames<-colnames(OC)

    # this puts poil as exo variable for all countries but EB

    for(i in 1:length(cnames)){
      aux<-gW[[cnames[i]]];ii<-nrow(aux)
      aux<-rbind(cbind(aux,matrix(0,ncol=OC.x,nrow=nrow(aux))),matrix(0,nrow=length(OC.exo),ncol=c(ncol(aux)+OC.x)))
      rownames(aux)[(ii+1):(ii+length(OC.exo))]<-OC.exo
      colnames(aux)[(ncol(aux)-OC.x+1):ncol(aux)]<-paste("OC",OCnames,sep=".")
      #aux["poil","OC.poil"]<-1  # this excludes qoil from the other models so far, can be modified
      if(length(OC.exo)>1){
        #diag(aux[(ii+1):(ii+OC.x),(ncol(aux)-OC.x+1):ncol(aux)])<-1
        diag(aux[OC.exo,paste0("OC.",OC.exo)])<-1
      }else{
        aux[OC.exo,paste0("OC.",OC.exo)]<-1
      }

      # set the variables we do not want to have specified as exo for other countries to zero
      #aux[,paste0("OC.",OCnames[!OCnames%in%OC.exo])]<-0
      gW[[cnames[i]]]<-aux
    }
    #we do not want to include oil prices in the ECB country model (Taylor rule)
    if(EA.flag){
      aux<-gW$EB;ii<-nrow(aux)
      aux<-cbind(aux,matrix(0,ncol=OC.x,nrow=nrow(aux)))
      colnames(aux)[(ncol(aux)-OC.x+1):ncol(aux)]<-paste("OC",OCnames,sep=".")
      gW$EB<-aux
    }

    Wnew <- matrix(0,(ncol(OC)+length(OC.vars)),ncol(xglobal));colnames(Wnew) <- colnames(xglobal);
    Ocnames<-paste0("OC.",colnames(OC))
    rownames(Wnew) <- c(Ocnames,OC.vars)
    if(ncol(OC)>1){
      diag(Wnew[Ocnames,Ocnames])<-1
    }else{
      Wnew[Ocnames,Ocnames]<-1
    }

    for(i in 1:length(OC.vars)){
      Wnew[OC.vars[i],paste0(names(OC.weights),".",OC.vars[i])]<-OC.weights
    }
    gW[[(length(gW)+1)]]<-Wnew
    names(gW)[length(gW)]<-"OC"

  }

  #re-order to match Data
  gW<-gW[ord]
  return(list(gW=gW,bigx=xglobal,exo=exo,exo.countries=exo.countries,endo=endo))
}

# .get_companion <- function(Beta_,varndxv=c(ncol(xglobal),1,plag)){
#   nn <- varndxv[[1]]
#   nd <- varndxv[[2]]
#   nl <- varndxv[[3]]
#
#   nkk <- nn*nl+nd
#
#   Jm <- matrix(0,nkk,nn)
#   Jm[1:nn,1:nn] <- diag(nn)
#
#   MM <- rbind(Beta_,cbind(diag((nl-1)*nn), matrix(0,(nl-1)*nn,nn+1)),c(matrix(0,1,nn*nl),1))
#
#   return(list(MM=MM,Jm=Jm))
#
# }


.get_companion <- function(Beta_,varndxv=c(ncol(xglobal),1,plag)){
  nn <- varndxv[[1]]
  nd <- varndxv[[2]]
  nl <- varndxv[[3]]

  nkk <- nn*nl+nd

  Jm <- matrix(0,nkk,nn)
  Jm[1:nn,1:nn] <- diag(nn)
  if (nd==1){
    MM <- rbind(Beta_,cbind(diag((nl-1)*nn), matrix(0,(nl-1)*nn,nn+1)),c(matrix(0,1,nn*nl),1))
  }else{
    MM <- rbind(Beta_,cbind(diag((nl-1)*nn),matrix(0,(nl-1)*nn,nn)))
  }


  return(list(MM=MM,Jm=Jm))

}



.gvar.stacking<-function(x,saves,globalpost,plag,xglobal,eigen=FALSE,stats=FALSE,Trim=NULL){
  require(abind)
  if(eigen){
    require(rARPACK)
  }
  # initialize objects here
  bigT <- ncol(x) #number of observations available (i.e., after lags have been excluded)
  pts <- seq(1,bigT)
  #save_thin <- save_thin*saves #number of draws used for MC integration of IRFs
  #draws_thin <- round(seq(1,saves,length.out =save_thin)) #Draws for MC integration to compute IRFs
  F.eigen<-numeric(saves)
  xx<-"No trimming"
  S_large<-Ginv_large<-array(NA, dim=c(nrow(x),nrow(x),saves))
  F_large<-array(NA,dim=c(nrow(x),nrow(x),plag,saves))
  ALPHA_large<-array(NA,dim=c(nrow(x),(nrow(x)*plag+1),saves))
  # assign dimnames right here (take care of trend?)
  dimnames(S_large)[[1]]<-dimnames(S_large)[[2]]<-dimnames(Ginv_large)[[1]]<-dimnames(Ginv_large)[[2]]<-dimnames(ALPHA_large)[[1]]<-colnames(xglobal)
  globalG<-S_post<-list()
  # objects needed if STATS=TRUE
  globalLik_large<-numeric(saves)
  YY <- xglobal[(plag+1):nrow(xglobal),]
  Xlarge <- cbind(BGVAR:::.mlag(YY,plag),1)
  Xlarge <- Xlarge[(plag+1):nrow(Xlarge),]
  Tp <- nrow(Xlarge) #number of observation aftr discarding the initial values used to initialize the model




  pb <- txtProgressBar(min = 0, max = saves, style = 3)

  #start loop here
  for (irep in 1:saves){
    #irep <- draws_thin[irep1]
    irep1<-irep # comment this line if thinning should be implemented in stacking function again
    for (i in 1:length(globalpost)) globalG[[i]] <- globalpost[[i]]$W

    VAR0 <- globalpost[[1]]
    #stacking/solving like in Pesaran et al (2004)
    #parameter stuff
    A0 <- cbind(diag(ncol(VAR0$End)),-t(VAR0$Lambda0post[irep,,]))
    W0 <- globalG[[1]]

    #creates matrices according to PSW 2004 for every lag
    for (kk in 1:length(VAR0$Theta)){
      assign(paste("B0",kk,sep=""),cbind(t(VAR0$Thetapost[[kk]][irep,,]),t(VAR0$Lambdapost[[kk]][irep,,])))
      assign(paste("H0",kk,sep=""),get(paste("B0",kk,sep=""))%*%W0)
    }

    a0 <- matrix(VAR0$a0post[irep,])
    G <- A0%*%W0
    #data stacking
    x <- t(VAR0$End)
    Ylag1 <- t(VAR0$Ylag)
    S_post[[1]] <- VAR0$SIGMApost[irep,,]
    for (i in 2:length(globalpost)){
      #N might create a list containing all country specific W matrices and use that in here (should be much faster)
      VAR1 <- globalpost[[i]]
      W1 <- globalG[[i]]
      #A1 <- cbind(diag(ncol(VAR1$End)),-1*t(VAR1$Lambda0post[irep,,]))
      A1 <- cbind(diag(ncol(VAR1$End)),-1*t(adrop(VAR1$Lambda0post[irep,,,drop=FALSE],drop=1))) # drop only first dimension (MCMC draws)


      for (kk in 1:length(VAR0$Theta)){
        #assign(paste("B1",kk,sep=""),cbind(t(VAR1$Thetapost[[kk]][irep,,]),t(VAR1$Lambdapost[[kk]][irep,,])))
        assign(paste("B1",kk,sep=""),cbind(t(VAR1$Thetapost[[kk]][irep,,]),t(adrop(VAR1$Lambdapost[[kk]][irep, ,,drop=FALSE ],drop=1))))
        assign(paste("H0",kk,sep=""),rbind(get(paste("H0",kk,sep="")),get(paste("B1",kk,sep=""))%*%W1))
      }

      G <- rbind(G,A1%*%W1)
      a0 <- rbind(a0,matrix(VAR1$a0post[irep,]))
      #Variables
      Ylag1 <- rbind(Ylag1,t(VAR1$Ylag))
      x <- rbind(x,t(VAR1$End))
      S_post[[i]] <- VAR1$SIGMApost[irep,,]
    }
    S_large[,,irep1]<-as.matrix(bdiag(S_post))
    #G_large[,,irep1]<-G
    class(x) <- "numeric"

    b2 <- NULL
    G.inv<-solve(G)
    b0 <- G.inv%*%a0 #global constant
    Ginv_large[,,irep1]<-G.inv

    ALPHA <- NULL
    for (kk in 1:plag){
      assign(paste("F",kk,sep=""),G.inv%*%get(paste("H0",kk,sep="")))
      F_large[,,kk,irep1] <- get(paste("F",kk,sep=""))
      ALPHA <- cbind(ALPHA,F_large[,,kk,irep1])

    }

    ALPHA <- cbind(ALPHA,b0)
    ALPHA_large[,,irep1]<-ALPHA


    if(eigen){
      MM<- BGVAR:::.get_companion(ALPHA,c(ncol(xglobal),1,plag))$MM
      aux<-suppressWarnings(eigs(MM,k=1,silent=TRUE))
      if(length(aux$values)>0){
        F.eigen[irep1]<-Mod(aux$values)
      }
    }

    #eigs1 <- Mod(eigen(MM,only.values=TRUE)$values[1:30])
    #eigs1

    if(stats){
      #In order to calculate the likelihood we have to create a global X(t)=(Y(t-1),...,Y(t-p)) matrix
      #res<-unlist(sapply(globalpost,function(x)x$Res));dim(res)<-dim(t(x))
      #global.res<-G.inv%*%t(res)

      globalLik_large[irep1]<-BGVAR:::.globalLik(Y=YY,X=Xlarge,Sig=G.inv%*%S_large[,,irep1]%*%t(G.inv),ALPHA=ALPHA,bigT=Tp)
    }



    #BIC<-dmvnorm(xglobal, xglobal%*%ALPHA[,-ncol(ALPHA)],G.inv%*%S_large[,,irep1]%*%t(G.inv))
    ############ kick out unstable draws

    setTxtProgressBar(pb, irep1)

  }

  #u_post<-unlist(lapply(lapply(globalpost,function(x)x$res.store),function(y) apply(y,c(2,3),median)));dim(u_post)<-c(Tp+1,ncol(xglobal))
  #colnames(u_post)<-colnames(xglobal)


  # kick out in-stable draws
  if(!is.null(Trim)){
    idx<-which(F.eigen<Trim)

    #u_post<-unlist(lapply(lapply(globalpost,function(x)x$res.store[idx,,]),function(y) apply(y,c(2,3),median)));dim(u_post)<-c(Tp+1,ncol(xglobal))
    #colnames(u_post)<-colnames(xglobal)
    F_large<-F_large[,,,idx,drop=FALSE]
    S_large<-S_large[,,idx,drop=FALSE]
    ALPHA_large<-ALPHA_large[,,idx,drop=FALSE]

    globalLik_large<-globalLik_large[idx]
    F.eigen<-F.eigen[idx]
    if(length(idx)<10){
      stop("Less than 10 stable draws have been found. Please re-estimate the model.")
    }
    xx<-round((length(idx)/dim(Ginv_large)[3])*100,2)
    xx<-paste("Trimming leads to ",length(idx) ," (",xx,"%) stable draws out of ",dim(Ginv_large)[3]," total draws",sep="")
    Ginv_large<-Ginv_large[,,idx,drop=FALSE]
    #print(xx)
  }

  # residuals based on posterior median
  #A_post <- apply(ALPHA_large, c(1,2), median)
  #res<-unlist(sapply(globalpost,function(x)x$Res));dim(res)<-dim(t(x))
  #res <- YY[-c(1:plag),]-Xlarge%*%t(A_post)
  #G_post<-solve(apply(Ginv_large, c(1,2),median))
  #res.country<-YY[-c(1:plag),]-Xlarge%*%t(G_post%*%A_post)

  results<-list(S_large=S_large,F_large=F_large,Ginv_large=Ginv_large,ALPHA_large=ALPHA_large,globalLik=globalLik_large,F.eigen=F.eigen,trim.info=xx)
  return(results)
}




.globalLik <- function(Y=YY,X=Xlarge,Sig=G.inv%*%Sigma_large[,,irep1]%*%t(G.inv),ALPHA=ALPHA,bigT=bigT){
  require(mvtnorm)
  PLS  <- 0
  for (i in 1:bigT){
    PLS <- PLS+sum(dmvnorm(Y[i,], X[i,]%*%t(ALPHA),Sig,log=TRUE))
  }
  return(list(Lik=PLS))
}






fcast<-function(gvar.obj,h=0,fhorz=8){
  require(MASS)
  require(Matrix)

  x<-gvar.obj$x
  S_large<-gvar.obj$stacked.results$S_large
  R<-gvar.obj$stacked.results$R
  Ginv_large<-gvar.obj$stacked.results$Ginv_large
  ALPHA_large<-gvar.obj$stacked.results$ALPHA_large
  plag<-gvar.obj$args$plag
  Names<-gvar.obj$ccode
  F.eigen<-gvar.obj$stacked.results$F.eigen
  #this stuff is needed for producing the forecasts

  xglobal<-gvar.obj$xglobal
  nn <- nrow(x)
  nd <- 1  # this is the number of deterministics, 1 is for one constant term
  nl <- plag

  varndxv <- c(nn,nd,nl)
  nkk <- (nl*nn)+nd

  Yn <- t(x)
  Xn <- cbind(BGVAR:::.mlag(Yn,plag),1)
  Xn <- Xn[(plag+1):nrow(Xn),]
  Yn <- Yn[(plag+1):nrow(Yn),]

  #Y0 <- t(Yn[(nrow(Yn)-h+1):nrow(Yn),,drop=FALSE])
  #colnames(Xn) <- rownames(x)
  M <- ncol(Yn)



  saves<-dim(Ginv_large)[3]
  fcst_t<-array(NA,dim=c(saves,ncol(xglobal),fhorz))

  # start loop here
  pb <- txtProgressBar(min = 0, max = saves, style = 3)

  for(irep1 in 1:saves){
    #Step I: Construct a global VC matrix Omega_t
    Sigma_t <- array(0,c(nrow(x),nrow(x),ncol(x)))
    Ginv<-Ginv_large[,,irep1]
    Sig_t <- Ginv%*%(S_large[,,irep1])%*%t(Ginv)
    Sig_t <- as.matrix(Sig_t)
    zt <- Xn[(nrow(Yn)-h),]
    z1 <- zt
    Mean00 <- zt
    Sigma00 <- matrix(0,nkk,nkk)
    Cm <- diag(M)
    y2 <- NULL

    #gets companion form
    companion <- BGVAR:::.get_companion(ALPHA_large[,,irep1],varndxv)
    Jm <- companion$Jm;Mm <- companion$MM
    #Mm <- as(Mm, "sparseMatrix")

    #Jm <- as(Jm,"sparseMatrix")
    Jsigt <- Jm%*%Sig_t%*%t(Jm)

    # this is the forecast loop
    for (ih in 1:fhorz){
      z1 <- Mm%*%z1
      Sigma00 <- Mm%*%Sigma00%*%t(Mm)+Jsigt
      varyt <- as.matrix(Cm%*%Sigma00[1:nn,1:nn]%*%t(Cm))
      chol_varyt <- try(t(chol(varyt)),silent=TRUE)
      if (is(chol_varyt,"try-error")){
        yf  <- mvrnorm(1,mu=z1[1:nn],varyt)
      }else{
        yf <- z1[1:nn]+chol_varyt%*%rnorm(nn,0,1)
      }
      y2 <- cbind(y2,yf)
    }

    fcst_t[irep1,,] <- y2
    setTxtProgressBar(pb, irep1)
  }

  imp_posterior<-array(NA,dim=c(ncol(xglobal),fhorz,5)) # 5 for low25,median,high75,low16,high84
  dimnames(imp_posterior)[[1]]<-colnames(xglobal)
  dimnames(imp_posterior)[[2]]<-1:fhorz
  dimnames(imp_posterior)[[3]]<-c("low25","low16","median","high75","high84")

  imp_posterior[,,"low25"] <- apply(fcst_t,c(2,3),quantile,0.25,na.rm=TRUE)#;rownames(imp_low25)<-colnames(xglobal)
  imp_posterior[,,"low16"] <- apply(fcst_t,c(2,3),quantile,0.16,na.rm=TRUE)#;rownames(imp_low25)<-colnames(xglobal)
  imp_posterior[,,"median"] <- apply(fcst_t,c(2,3),quantile,0.5,na.rm=TRUE)#;rownames(imp_low25)<-colnames(xglobal)
  imp_posterior[,,"high75"] <- apply(fcst_t,c(2,3),quantile,0.75,na.rm=TRUE)#;rownames(imp_low25)<-colnames(xglobal)
  imp_posterior[,,"high84"] <- apply(fcst_t,c(2,3),quantile,0.84,na.rm=TRUE)#;rownames(imp_low25)<-colnames(xglobal)

  lps.stats<-array(0,dim=c(ncol(xglobal),2,fhorz))
  dimnames(lps.stats)[[1]]<-colnames(xglobal);dimnames(lps.stats)[[2]]<-c("mean","sd");dimnames(lps.stats)[[3]]<-1:fhorz
  lps.stats[,"mean",]<-apply(fcst_t,c(2:3),mean);lps.stats[,"sd",]<-apply(fcst_t,c(2:3),sd)
  if(h>0){
    hold.out<-Xn[(nrow(Xn)+1-h):nrow(Xn),-ncol(Xn),drop=FALSE]
  }
  else{
    hold.out<-NULL
  }
  #imp_posterior <- list(low25=imp_low25,median=imp_median,high75=imp_high75,low16=imp_low16,high84=imp_high84,xglobal=xglobal)

  # date function
  tI<-tsp(gvar.obj$args$Data[[1]])
  xlabel<-as.character(format(seq(tI[1],tI[2],by=1/tI[3]),dig=2,nsmall=2))
  if(tI[3]==12){
    xlabel<-gsub(".00","-Jan",xlabel,fixed=TRUE)
    xlabel<-gsub(".08","-Feb",xlabel,fixed=TRUE)
    xlabel<-gsub(".17","-Mar",xlabel,fixed=TRUE)
    xlabel<-gsub(".25","-Apr",xlabel,fixed=TRUE)
    xlabel<-gsub(".33","-May",xlabel,fixed=TRUE)
    xlabel<-gsub(".42","-Jun",xlabel,fixed=TRUE)
    xlabel<-gsub(".50","-Jul",xlabel,fixed=TRUE)
    xlabel<-gsub(".58","-Aug",xlabel,fixed=TRUE)
    xlabel<-gsub(".67","-Sep",xlabel,fixed=TRUE)
    xlabel<-gsub(".75","-Oct",xlabel,fixed=TRUE)
    xlabel<-gsub(".83","-Nov",xlabel,fixed=TRUE)
    xlabel<-gsub(".92","-Dec",xlabel,fixed=TRUE)
  }
  if(tI[3]==4){
    xlabel<-gsub(".00","Q1",xlabel,fixed=TRUE)
    xlabel<-gsub(".25","Q2",xlabel,fixed=TRUE)
    xlabel<-gsub(".50","Q3",xlabel,fixed=TRUE)
    xlabel<-gsub(".75","Q4",xlabel,fixed=TRUE)
  }

  rownames(xglobal)<-xlabel

  return(list(fcast=imp_posterior,xglobal=xglobal,fhorz=fhorz,lps.stats=lps.stats,hold.out=hold.out))
}




lps<-function(fcastM){
  require(mvtnorm)
  hold.out<-fcastM$hold.out
  if(is.null(hold.out)){
    stop("Please submit a forecast object that includes a hold out sampel for evaluation (set h>0 in fcast)!")
  }

  lps.stats<-fcastM$lps.stats
  lps.scores<-numeric(ncol(hold.out))
  for(i in 1:ncol(hold.out)){
    lps.scores[i]<-sum(dnorm(hold.out[,i],mean=lps.stats[i,"mean",],sd=lps.stats[i,"sd",],log=TRUE))
  }
  names(lps.scores)<-dimnames(lps.stats)[[1]]
  return(lps.scores)
}

fcast.plot<-function(fcastM,resp=NULL,Cut=40,Main=NULL,length.Out=10,b=NULL,cex.la=0.6){
  if(is.null(resp) || (!resp%in%dimnames(fcastM$fcast)[[1]])){
    stop("Please select a variable that is part of the GVAR system for which to plot the forecasts.")
  }
  fcast<-fcastM$fcast[resp,,]
  Xdata<-fcastM$xglobal
  hstep<-fcastM$fhorz
  # we should always have around 40 observations including the forecasts
  thin<-nrow(Xdata)-hstep
  if(thin>Cut){
    Xdata<-Xdata[(nrow(Xdata)-Cut+1):nrow(Xdata),]
  }


  x<-rbind(cbind(NA,Xdata[,resp],NA),fcast[,c("low25","median","high75")])
  y<-rbind(cbind(NA,Xdata[,resp],NA),fcast[,c("low16","median","high84")])
  if(is.null(b)){
    b<-range(as.numeric(cbind(x,y)),na.rm=TRUE)
  }


  matplot(x,type="l",col=c("blue","black","blue"),xaxt="n",lwd=4,ylab="",main=Main,
          cex.main=2,cex.axis=cex.la,cex.lab=2.5,lty=c(0,1,0),ylim=c(b[1],b[2]))
  polygon(c(1:nrow(y),rev(1:nrow(y))),c(y[,1],rev(y[,3])),col="lightblue",border=NA)
  polygon(c(1:nrow(x),rev(1:nrow(x))),c(x[,1],rev(x[,3])),col="#0099B4",border=NA)
  lines(x[,2],col="darkblue",lwd=4)

  #axis(2, at=round(seq(b1,b2,length.out=length.Out),3), labels=format(seq(b1,b2,length.out=N),digits=1,nsmall=1),cex.axis=cex.la,las=1)

  axisnames<-c(rownames(Xdata),paste0("t+",1:hstep))
  axisindex<-unique(c(seq(1,length(axisnames),length.out=length.Out), rev(seq(1,length(axisnames)))[1:hstep]))
  axis(side=1, las=1,at=axisindex, labels=axisnames[axisindex], cex.axis=cex.la,tick=FALSE,las=2)
  abline(v=axisindex,col="lightgrey",lty=3)

}


#fcast.plot(fcastM,resp="AT.y",Cut=20,length.Out=10)

.irf.chol <- function(maxlag=1,ccode="EA",shockvar=shockc,invG=gvarobj$G,lF=lF[,,irep1],gcov=sigs,x=gvarobj$x,horizon=40,xglobal=xglobal,...){
  P0G <- diag(nrow(x))
  Names <- substr(rownames(x),1,2)
  Names <- Names[!duplicated(Names)]
  N <- length(Names)

  #calculates cholesky factor for blocks of gcov where shock occurrs
  colnames(gcov) <-rownames(gcov) <- colnames(P0G)<-rownames(P0G)<-rownames(x)
  for(i in 1:N){
    if(Names[i] %in% ccode){
      idx<-which(grepl(Names[[i]],rownames(x)))
      g.cov.temp<-gcov[idx,idx,drop=FALSE]
      P0<-t(chol(g.cov.temp)) # calculate local cholesky factor of gcov
      P0G[idx,idx]<-P0
      gcov[idx,idx]<-diag(ncol(g.cov.temp)) #set vcv matrix to identity for coutnry where shock occurs
    }
  }
  K <- nrow(x)

  PHIx <- array(0,c(K,K,maxlag+horizon+1));rownames(PHIx) <- colnames(PHIx) <- rownames(x);

  for (i in 1:maxlag){
    PHIx[,,i]  <-  matrix(0,K,K) #ok
  }
  PHIx[,,maxlag+1]  <-  diag(K)

  for (t in (maxlag+2):(maxlag+horizon+1)){
    acc = 0
    for (j in 1:maxlag){
      acc  <-  acc + lF[,,j,1]%*%PHIx[,,t-j] # the last dimension of lF is  the mcmc draw dimension, hence fixed to 1
    }
    PHIx[,,t]  <-  acc
  }

  #reindicize
  PHI  <-  PHIx[,,(maxlag+1):(maxlag+horizon+1)]
  # preallocate matrix containing GIRFs
  eslct <- matrix(0,K,1);rownames(eslct) <- rownames(x)

  irfa  <- array(0,c(K,1,horizon+1))

  slct <- diag(nrow(x))
  select_shocks <- intersect(rownames(x),paste(ccode,shockvar$shock,sep="."))
  eslct[select_shocks,] <- 1

  #temp  <-invG%*%Cm%*%t(R)
  invGSigma_u  <-  invG%*%P0G%*%gcov
  cons<-as.numeric(1/sqrt(t(eslct)%*%gcov%*%eslct))
  dimnames(irfa)[[1]]  <- rownames(x)

  # this part is the fixing of stir at the zero lower bound as in Baumeister and Benati ------------------------------------------------------------
  for (ii in 1:(horizon+1)){
    #     if (ii<0){
    #       PHI_hat <- PHI
    #       PHI_hat["DE.stir",,ii] <- 0
    #       Cmhat <- temp
    #       Cmhat["DE.stir",] <- 0
    #     }else{
    PHI_hat <- PHI
    Cmhat <- invGSigma_u
    # }
    irfa[,1,ii]  <-   as.matrix((PHI_hat[,,ii]%*%Cmhat%*%eslct))*cons
  }

  # END -----------------------------------------------------------------------------------------------------------------------------------------
  irfa <- irfa[,1,]
  return(list(impl=irfa,rot=diag(K),gcov=gcov,PHI=PHI))
}




.irf.girf <- function(maxlag=1,shockvar="stir",invG=gvarobj$G,lF=gvarobj$F,gcov=sigs,x=gvarobj$x,horizon=40,ccode=Grps,...){

  Names <- substr(rownames(x),1,2)
  Names <- Names[!duplicated(Names)]
  N <- length(Names)
  Cm <- as.matrix(gcov)
  K <- nrow(x)

  PHIx <- array(0,c(K,K,maxlag+horizon+1));rownames(PHIx) <- colnames(PHIx) <- rownames(x);
  PHIx[,,maxlag+1]  <-  diag(K)

  for (t in (maxlag+2):(maxlag+horizon+1)){
    acc = 0
    for (j in 1:maxlag){
      acc  <-  acc + lF[,,j,1]%*%PHIx[,,t-j] # the last dimension of lF is the mcmc draw dimension, hence fixed to 1
    }
    PHIx[,,t]  <-  acc
  }

  #reindicize
  PHI  <-  PHIx[,,(maxlag+1):(maxlag+horizon+1)]

  irfa  <- array(0,c(K,horizon+1))

  invGSigma_u  <-  invG%*%Cm;rownames(invGSigma_u)<-colnames(invGSigma_u)<-rownames(x)
  eslct <- matrix(0,nrow(x),1,dimnames=list(rownames(x),NULL))
  sl_var <- paste(ccode,shockvar$shock,sep=".")
  sl_var <- intersect(sl_var,rownames(x))
  eslct[sl_var,1] <- 1

  aa<-invGSigma_u%*%eslct
  cons<-as.numeric(1/sqrt(t(eslct)%*%Cm%*%eslct))
  for (i in 1:(horizon+1)){
    temp <- (PHI[,,i]%*%aa)*cons
    irfa[,i]  <-  temp
  }
  dimnames(irfa)[[1]]  <- rownames(x)

  return(list(impl=irfa,rot=diag(K),gcov=Cm))
}





#MOVE THIS FUNCTION SOMEWHERE
.get_V <- function(K=K,N=N,X=X,Wex=Wex,Wexlag=Wexlag,trend=trend,ind=ind,a_bar_1=a_bar_1,a_bar_2=a_bar_2,a_bar_3=a_bar_3,a_bar_4,sigma_sq=sigma_sq,sig_wex=sig_wex,p=p){
  #creating an array of dimension K x N, which will contain the K diagonal elements of the covmatrix, in each eqn
  V_i <- matrix(0,K,N)

  rownames(V_i) <- colnames(X)
  #this double loop fills the prior covariance matrix
  for (i in 1:N){ #for each i equation
    for (j in 1:K){ #for each variable on the rhs
      if (j<= (1+trend+ncol(Wex)+ncol(Wexlag))) {
        V_i[j,i] <- a_bar_3*sigma_sq[i,1] #variance on constant,trend, dummies and on ex variables
      }
      else if (any(j==ind[i,])) {
        V_i[j,i] <- a_bar_1/(p^2) #variance on own lags
      }
      else
      {
        for (kj in 1:N){
          if (any(j==ind[kj,]))
          {
            ll <- kj
          }
        }
        V_i[j,i] <- (a_bar_2*sigma_sq[i,1])/((p^2)*sigma_sq[ll,1])
      }

    }
  }

  #prior variance on const & trend
  for (jj in 1:N){
    V_i[1:(1+trend),jj] <- a_bar_3*sigma_sq[jj,1]
  }
  #starts the Wex part
  for (jj in 1:N){
    for (kk in 0:p){
      if (kk==0) slct <- which(colnames(X)=="Wex") else slct <- which(colnames(X)==paste("Wexlag",kk,sep=""))
      temp <- V_i[slct,jj]
      for (ii in 1:length(slct)){
        temp[ii] <- a_bar_4*sigma_sq[jj,1]/(sig_wex[ii,1]*(kk+1))
      }
      V_i[slct,jj] <- temp
    }
  }
  return(V_i)
}


.irf.sign<-function(maxlag=1,lF=F_large[,,irep1,drop=FALSE],M,horizon,invG=invG_large[,,irep1],gcov=S_large[,,irep1],srots=Srots,xglobal=xglobal,sign.constr=sign.constr,rest=NA,rest.neg=NA,MaxTrys=7500,Global=Global,ss=ss,ccode="EA",...){  # quarter <- 24;lF = F_array;shock=Sig;nhor=nhor;M=ncol(xglobal);srots <- 1; Y <- xglobal;slct_var="ltir";grp_slct=EA_countries
  x <- t(xglobal)
  P0G <- diag(nrow(x))
  Names <- substr(rownames(x),1,2)
  Names <- Names[!duplicated(Names)]
  N <- length(Names)


  #calculates cholesky factor for blocks of gcov where shock occurrs
  colnames(gcov) <-rownames(gcov) <- colnames(P0G)<-rownames(P0G)<-rownames(x)
  for(i in 1:N){
    if(Names[i] %in% ccode){
      idx<-which(grepl(Names[[i]],rownames(x)))
      g.cov.temp<-gcov[idx,idx,drop=FALSE]
      P0<-t(chol(g.cov.temp)) # calculate local cholesky factor of gcov
      P0G[idx,idx]<-P0
      gcov[idx,idx]<-diag(ncol(g.cov.temp)) #set vcv matrix to identity for coutnry where shock occurs
    }
  }
  K <- nrow(x)

  PHIx <- array(0,c(K,K,maxlag+horizon+1));rownames(PHIx) <- colnames(PHIx) <- rownames(x);

  for (i in 1:maxlag){
    PHIx[,,i]  <-  matrix(0,K,K) #ok
  }
  PHIx[,,maxlag+1]  <-  diag(K)

  for (t in (maxlag+2):(maxlag+horizon+1)){
    acc = 0
    for (j in 1:maxlag){
      acc  <-  acc + lF[,,j,1]%*%PHIx[,,t-j] # the last dimension of lF is  the mcmc draw dimension, hence fixed to 1
    }
    PHIx[,,t]  <-  acc
  }

  #reindicize
  PHI  <-  PHIx[,,(maxlag+1):(maxlag+horizon+1)]


  res.time<-lapply(sign.constr,function(x) x$rest.horz)#CHG THIS TO THE MAXIMUM OF ALL RESTRICTIONS SUPPLIED IN A LIST imps <- seq(1:1)
  res.max<-max(unlist(res.time))

  quarter=9 # this is redundant, can fix it later
  horizon<-horizon+1 # this is because of Sims IRF code later on
  shock.nr<-length(rest) # number of shocks
  impulse_rot <- array(0,c(M,horizon,shock.nr,srots))
  store_rot <- array(0,c(M,M,srots))

  #shock_slct <- paste0(ccode,".",shockvar) #shock_slct <- intersect(shock_slct,colnames(xglobal))
  #INCLUDE character vector that tells you where we have to impose restrictions (example: ss <- restr.country <- c("US", "DE")) restr.country <- unique(sapply(sapply(ss,function(x) strsplit(x,".",fixed=TRUE)),function(y) y[[1]]))
  restr.country <- unique(sapply(sapply(ss,function(x) strsplit(x,".",fixed=TRUE)),function(y) y[[1]]))

  #shock_slct<-sapply(sign.constr,function(x) x$shock)
  #shock<-t(chol(invG%*%gcov%*%t(invG)))
  sigma<-invG%*%gcov%*%t(invG)
  if(shock.nr==1 || !Global){
    shock_slct <- sapply(sign.constr, function(x) x$shock)
  }else{
    shock_slct <- lapply(sign.constr, function(x) x$shock) # this guarantees that we get a list object if shock.nr>1
  }
  shock<-try(chol(sigma),silent=FALSE)
  if(class(shock)=="try-error"){
    return(list(impl=NA,rot=NA,icounter=NA,gcov=NA,xglobal=xglobal))
  }
  shock<-t(shock)
  invGSigma_u  <-  invG%*%P0G%*%gcov

  for (kk in 1:srots){
    icounter <- 0
    icheck <- 0
    R_bar <- diag(M)
    colnames(R_bar) <- rownames(R_bar) <- colnames(xglobal)

    while(icheck<1 && icounter < MaxTrys){
      #cat(paste("rotation:",kk,"icounter:",icounter),"\n")
      icounter <- icounter+1
      #Step 1: Draw a rotation matrix
      for (jj in restr.country){
        slct.country <-grepl(jj,colnames(R_bar))
        k_i <-sum(slct.country)
        A <- matrix(rnorm(k_i*k_i,0,1),k_i,k_i)
        qA <- qr(A)
        rotA <- qr.Q(qA)
        rotA <- rotA%*%diag(((diag(rotA)>0)-(diag(rotA)<0)))
        R_bar[slct.country,slct.country] <- rotA
      }

      # irf_o is of dimension K times K times restr.horizon
      if (res.max>1){
        # this part is the fixing of stir at the zero lower bound as in Baumeister and Benati ------------------------------------------------------------
        irf_o <- array(NA,c(K,K,res.max))
        for (ii in 1:(res.max)){
          Cmhat <- invGSigma_u%*%t(R_bar)
          irf_o[,,ii]  <-   as.matrix((PHI[,,ii]%*%Cmhat))
        }
      }else{
        irf_o <- array((invGSigma_u%*%t(R_bar)),c(M,M,1))
      }
      rownames(irf_o)<-colnames(irf_o) <- colnames(xglobal)
      #[,shock_slct]#impulsdtrf(lF,shock%*%t(rotA)%*%R_bar,2)[,M,]

      if(Global){ # GLOBAL SHOCK, sum up effects to single shocks
        if(shock.nr>1){
          irf_fac<-array(0,dim=c(M,shock.nr,res.max))
          dimnames(irf_fac)[[1]]<-colnames(xglobal)
          for(i in 1:shock.nr){
            aux <- array(apply(irf_o[,shock_slct[[i]],,drop=FALSE],c(1,3),sum),dim=c(M,1,res.max))
            irf_fac[,i,]<-aux
          }
        }else{
          irf_fac<-array(apply(irf_o[,shock_slct,,drop=FALSE],c(1,3),sum),dim=c(M,1,res.max));dimnames(irf_fac)[[1]]<-colnames(xglobal)
        }
      }else{ # here we do not have to loop through irf_fac because we don't have to do the summing up part as with the global shock
        irf_fac <- irf_o[,shock_slct,,drop=FALSE] #K*nr. of shocks matrix
      }

      # now start evaluating the restrictions------------------------------------------------------------
      restr<-c()
       for(i in 1:shock.nr){
          restr<-c(restr,eval(parse(text=rest[[i]])))
       }

      # in case restrictions are not fulfilled, try with negative roation matrix (sign switch)
      if(!all(restr==1)){
        irf_fac.temp<-irf_fac;irf_fac<-irf_fac*(-1)
        restr<-c()
        for(i in 1:shock.nr){
          restr<-c(restr,eval(parse(text=rest[[i]])))
        }
        R_bar[slct.country,slct.country]=-rotA # negative rotation matrix in case neg. restr fulfilled
        irf_fac<-irf_fac.temp # set irf_fac object back to normal
      }

      # make final evaluation
      if(!all(restr==1)){
        next
      }
      icheck <- 1
    }
    # end while loop ----------------------------------------------------------------------------


    A_array1 <- lF
    st_impulses <- array(NA,c(K,K,horizon))
    for (ii in 1:(horizon)){
      Cmhat <- invGSigma_u%*%t(R_bar)
      st_impulses[,,ii]  <-   as.matrix((PHI[,,ii]%*%Cmhat))
    }
    dimnames(st_impulses)[[1]] <- dimnames(st_impulses)[[2]] <- colnames(xglobal)

    if(Global){ # GLOBAL SHOCK
      if(shock.nr>1){
        st_impulses2<-NULL
        for(i in 1:shock.nr){
          st_impulses2<-abind(st_impulses2,apply(st_impulses[,shock_slct[[i]],],c(1,3),sum),along=3)
        }
        #st_impulses <- aperm(st_impulses2,c(1,3,2))
        st_impulses<-st_impulses2
      }else{
        st_impulses <- apply(st_impulses[,shock_slct,],c(1,3),sum)
      }
    }else{
      st_impulses<-st_impulses[,shock_slct,,drop=FALSE]
      st_impulses<-aperm(st_impulses,c(1,3,2)) # re-indicize
    }


    if (icounter==MaxTrys){
      st_impulses <- R_bar<-NA
    }

    impulse_rot[,,,kk] <- st_impulses
    store_rot[,,kk] <- R_bar
  }
  # end rotation matrix loop ----------------------------------------------------------------------------

  # FRY PAGAN STUFF

  median_impulses <- apply(impulse_rot,c(1,2,3),median,na.rm=TRUE)
  min_distance <- matrix(0,srots,1)

  for(jj in 1:srots){ # for the number of rotation matrices look at the one that minimizes distance to median
    aux<-c()
    for(z in 1:shock.nr){
      aux<-c(aux,abs(mean(impulse_rot[,,z,jj]-median_impulses[,,z])))
    }
    min_distance[jj,] <-sum(aux)
  }
  sl_rot <- which.min(min_distance)
  rotpick <- store_rot[,,sl_rot]

  if(length(sl_rot>1)){
    st_impulses_1 <- array(NA,c(K,K,horizon))
    for (ii in 1:(horizon)){
      Cmhat <- invGSigma_u%*%t(rotpick)
      st_impulses_1[,,ii]  <-   as.matrix((PHI[,,ii]%*%Cmhat))
    }
    dimnames(st_impulses_1)[[1]] <- dimnames(st_impulses_1)[[2]] <- colnames(xglobal)
    if(Global){ # GLOBAL SHOCK
      if(shock.nr>1){
        st_impulses2<-NULL
        for(i in 1:shock.nr){
          st_impulses2<-abind(st_impulses2,apply(st_impulses_1[,shock_slct[[i]],],c(1,3),sum),along=3)
        }
        #st_impulses_1 <- aperm(st_impulses2,c(1,3,2))
        st_impulses_1<-st_impulses2
      }else{
        st_impulses_1 <- apply(st_impulses_1[,shock_slct,],c(1,3),sum)
      }
    }else{
      st_impulses_1<-st_impulses_1[,shock_slct,,drop=FALSE]
      st_impulses_1<-aperm(st_impulses_1,c(1,3,2)) # re-indicize
    }
    dimnames(st_impulses_1)[[1]] <-colnames(xglobal) #CHG to dimanmes(lF)[[1]]
  }else{ #WHEN WE HAVE ONLY NAs, no rotation matrix found
    rotpick <- R_bar
    st_impulses_1 <- NA
    #dimnames(st_impulses_1)[[1]] <-colnames(xglobal) #CHG to dimanmes(lF)[[1]]
  }

  return(list(impl=st_impulses_1,rot=rotpick,icounter=icounter,gcov=NA))
}




.impulsdtrf<-function(B,smat,nstep,time=FALSE,B1=NULL,quarter=9,lags=maxlag)
  ### By:             As emerges from rfvar, neqn x nvar x lags array of rf VAR coefficients.
  ### smat:           nshock x nvar matrix of initial shock vectors.  To produce "orthogonalized
  ###                 impulse responses" it should have the property that crossprod(t(smat))=sigma,
  ###                 where sigma is the Var(u(t)) matrix and u(t) is the rf residual vector.  One
  ###                 way to get such a smat is to set smat=t(chol(sigma)).  To get the smat
  ###                 corresponding to a different ordering, use
  ###                 smat = t(chol(P %*% Sigma %*% t(P)) %*% P), where P is a permutation matrix.
  ###                 To get impulse responses for a structural VAR in the form A(L)y=eps, with
  ###                 Var(eps)=I, use B(L)=-A_0^(-1)A_+(L) (where A_+ is the coefficients on strictly
  ###                 positive powers of L in A), smat=A_0^(-1).
  ###                 In general, though, it is not required that smat be invertible.
### response:       nvar x nshocks x nstep array of impulse responses.
###
### Code written by Christopher Sims, based on 6/03 matlab code.  This version 3/27/04.
### Added dimension labeling, 8/02/04.
{

  neq <- dim(B)[1]
  nvar <- dim(B)[2]
  #lags <- dim(B)[3]
  dimnB <- dimnames(B)
  if(dim(smat)[2] != dim(B)[2]) stop("B and smat conflict on # of variables")

  response <- array(0,dim=c(neq,nvar,nstep+lags-1))
  response[ , , lags] <- smat
  response <- aperm(response, c(1,3,2))
  irhs <- 1:(lags*nvar)
  ilhs <- lags * nvar + (1:nvar)
  response <- matrix(response, ncol=neq)
  B <- B[, , seq(from=lags, to=1, by=-1),1]  #reverse time index to allow matrix mult instead of loop
  B <- matrix(B,nrow=nvar)
  for (it in 1:(nstep-1)) {
    #     if (time){
    #       if (it==quarter){
    #         B <- B1[, , seq(from=lags, to=1, by=-1)]  #reverse time index to allow matrix mult instead of loop
    #         B <- matrix(B1,nrow=nvar)
    #       }
    #     }

    response[ilhs, ] <- B %*% response[irhs, ]
    irhs <- irhs + nvar
    ilhs <- ilhs + nvar
  }
  ## for (it in 2:nstep)
  ##       {
  ##         for (ilag in 1:min(lags,it-1))
  ##           response[,,it] <- response[,,it]+B[,,ilag] %*% response[,,it-ilag]
  ##       }
  dim(response) <- c(nvar, nstep + lags - 1, nvar)
  if(lags==1){
    response <- aperm(response, c(1, 3, 2)) # array in usual format
  }
  else{
    response <- aperm(response[ , -(1:(lags-1)), ], c(1, 3, 2)) #drop the zero initial conditions; array in usual format
  }
  dimnames(response) <- list(dimnB[[1]], dimnames(smat)[[2]], NULL)
  ## dimnames(response)[2] <- dimnames(smat)[1]
  ## dimnames(response)[1] <- dimnames(B)[2]
  return(response)
}

# Average pairwise cross-correlation
# Calculated as cor(x[,i,1],x[,j!=i,1]) for all variables i=1,...k; and j=1,...,number of countries
# Function provides correlations for the  data submitted and    for the residuals of the individual country models

avg.pair.cc=function(X,dig=3){
  if(class(X)=="bgvar"){
    dat<-t(X$x)
    u<-X$cc.results$res
    u<-u[,colnames(dat)] # make sure they are in the same order
  }
  if(class(X)=="res"){
    dat<-t(X$Dat)
    u<-apply(X$res.array.country,c(1:2),mean)  # calculate mean to be consistent with running mean stuff
    u.g<-apply(X$res.array.global,c(1:2),mean) # global residuals
  }

  bigT<-nrow(u)
  var.set<- colnames(dat)
  # now have to link gdp series to gdp series, price series to price series and so on

  cccode<-unique(sapply(strsplit(var.set,".",fixed=TRUE),function(x) x[1]))
  vars<-unique(sapply(strsplit(var.set,".",fixed=TRUE),function(x) x[2]))

  # before we go into the loop, separate endo and weakly exo from exo variables (for which no corr can be calculated since only in one model contained)
  idx<-lapply(as.list(1:length(vars)),function(x) grep(paste0(".",vars[x]),colnames(dat)));  names(idx)<-vars
  # kick out exo variables
  exo<-which(sapply(idx,length)==1)
  if(length(exo)>0){
    idx<-idx[-exo]
  }

  datL<-resL<-resL.g<-matrix("-",nrow=length(cccode),ncol=length(idx))
  rownames(datL)<-rownames(resL)<-rownames(resL.g)<-cccode
  colnames(datL)<-colnames(resL)<-colnames(resL.g)<-names(idx)

  if(class(X)=="bgvar"){
    for(i in 1:length(idx)){
      aux<-cor(dat[,idx[[i]]])
      aux.u<-cor(u[,idx[[i]]])
      diag(aux)<-diag(aux.u)<-NA
      ii<-sapply(strsplit(rownames(aux),".",fixed=TRUE),function(x) x[1]) #should be the same for u
      aux<-round(rowMeans(aux,na.rm=TRUE),digits=dig)
      aux.u<-round(rowMeans(aux.u,na.rm=TRUE),digits=dig)
      datL[ii,i]<-aux
      resL[ii,i]<-aux.u
    }
  }
  if(class(X)=="res"){  # include analysis based on residuals of the global model as well
    for(i in 1:length(idx)){
      aux<-cor(dat[,idx[[i]]])
      aux.u<-cor(u[,idx[[i]]])
      aux.ug<-cor(u.g[,idx[[i]]])
      diag(aux)<-diag(aux.u)<-diag(aux.ug)<-NA
      ii<-sapply(strsplit(rownames(aux),".",fixed=TRUE),function(x) x[1]) #should be the same for u
      aux<-round(rowMeans(aux,na.rm=TRUE),digits=dig)
      aux.u<-round(rowMeans(aux.u,na.rm=TRUE),digits=dig)
      aux.ug<-round(rowMeans(aux.ug,na.rm=TRUE),digits=dig)
      datL[ii,i]<-aux
      resL[ii,i]<-aux.u
      resL.g[ii,i]<-aux.ug
    }
  }

  # Generate summary-table
  pp<-suppressWarnings(apply(datL,2,as.numeric))
  rr<-suppressWarnings(apply(resL,2,as.numeric))
  rg<-suppressWarnings(apply(resL.g,2,as.numeric))
  dat.res<-res.res<-res.resG<-matrix(0,nrow=4,ncol=ncol(datL))
  for(i in 1:ncol(datL)){
    aux<-pp[,i];aux<-abs(aux[which(!is.na(aux))]);K<-length(aux)
    aux2<-rr[,i];aux2<-abs(aux2[which(!is.na(aux2))]);K2<-length(aux2)
    aux3<-rg[,i];aux3<-abs(aux3[which(!is.na(aux3))]);K3<-length(aux3)

    dat.res[1,i]<-paste0(length(which(aux<=0.1))," (",round((length(which(aux<=0.1))/K)*100,2),"%)")
    res.res[1,i]<-paste0(length(which(aux2<=0.1))," (",round((length(which(aux2<=0.1))/K2)*100,2),"%)")
    res.resG[1,i]<-paste0(length(which(aux3<=0.1))," (",round((length(which(aux3<=0.1))/K3)*100,2),"%)")

    dat.res[2,i]<-paste0(length(which(aux>0.1&aux<=0.2))," (",round((length(which(aux>0.1&aux<=0.2))/K)*100,2),"%)")
    res.res[2,i]<-paste0(length(which(aux2>0.1&aux2<=0.2))," (",round((length(which(aux2>0.1&aux2<=0.2))/K2)*100,2),"%)")
    res.resG[2,i]<-paste0(length(which(aux3>0.1&aux2<=0.2))," (",round((length(which(aux3>0.1&aux3<=0.2))/K3)*100,2),"%)")


    dat.res[3,i]<-paste0(length(which(aux>0.2&aux<=0.5))," (",round((length(which(aux>0.2&aux<=0.5))/K)*100,2),"%)")
    res.res[3,i]<-paste0(length(which(aux2>0.2&aux2<=0.5))," (",round((length(which(aux2>0.2&aux2<=0.5))/K2)*100,2),"%)")
    res.resG[3,i]<-paste0(length(which(aux3>0.2&aux3<=0.5))," (",round((length(which(aux3>0.2&aux3<=0.5))/K3)*100,2),"%)")


    dat.res[4,i]<-paste0(length(which(aux>0.5&aux<=1))," (",round((length(which(aux>0.5&aux<=1))/K)*100,2),"%)")
    res.res[4,i]<-paste0(length(which(aux2>0.5&aux2<=1))," (",round((length(which(aux2>0.5&aux2<=1))/K2)*100,2),"%)")
    res.resG[4,i]<-paste0(length(which(aux3>0.5&aux3<=1))," (",round((length(which(aux3>0.5&aux3<=1))/K3)*100,2),"%)")

  }
  rownames(dat.res)<-rownames(res.res)<-rownames(res.resG)<-c("<0.1","0.1-0.2","0.2-0.5",">0.5")
  colnames(dat.res)<-colnames(res.res)<-colnames(res.resG)<-colnames(datL)


  avg.cc<-list(data.cor=datL,resid.cor=resL,resid.corG=resL.g,dat.res=dat.res,res.res=res.res,res.resG=res.resG)
  return(avg.cc)
}
#avg.pair.cc(model.sims)



# Residual correlation test
# Tests the residuals of the country VECM models for serial autocorrelation
# Input arguments:
#                  x......GVAR object
#                  lag....the degree of serial autocorrelation that should be tested for
#                  alpha..significance level
#                  dig1 / dig2...nr. of digits for the test statistic / p-value


resid.corr.test=function(gvar.obj,lag.cor=1,alpha=0.95,dig1=5,dig2=3){
  # get data and arguments - note each second column of V has sign switched -> does not impact on results of F test so keep it as it is
  Z2<-t(gvar.obj$x)      # get data matrix
  u<-gvar.obj$cc.results$res # get residuals per equation in country vecm
  bigT<-nrow(u)

  pidx<-1:lag.cor
  ccode<-gvar.obj$ccode
  var.set<- colnames(Z2)
  vars<-unique(sapply(strsplit(var.set,".",fixed=TRUE),function(x) x[2]))


  # helper function to construct W projector matrix
  w.t<-function(x,lag=1){
    x.n<-c(rep(0,lag),rev(rev(x)[-c(1:lag)]))
    return(x.n)
  }


  # Calculate F-Statistic in a loop
  Fstat<-critL<-pL<-dofL<-list() # list objects since not for every country some nr. of regressors
  for(i in 1:length(ccode)){
    idx<-grep(paste0(ccode[i],"."),colnames(Z2)) # get data for country model i
    X.dat<-Z2[,idx,drop=FALSE];u.dat<-u[,idx,drop=FALSE];ki<-ncol(X.dat)
    dof<-(bigT-ki-lag.cor)
    M<-diag(bigT)-tcrossprod(X.dat%*%chol2inv(chol(crossprod(X.dat))),X.dat)
    # construct W matrix
    w.array<-array(0,dim=c(bigT,ki,lag.cor))
    faux<-critV<-pV<-NULL
    for(j in 1:ki){ # for all variables
      w<-NULL
      for(p in 1:lag.cor){
        w<-cbind(w,w.t(u.dat[,j],lag=p))
      }
      aux<-bigT*((crossprod(u.dat[,j],w)%*%solve(t(w)%*%M%*%w)%*%(t(w)%*%u.dat[,j]))/crossprod(u.dat[,j]))
      faux<-c(faux,(dof/lag.cor)*(aux/(bigT-aux)))
      pV<-c(pV,c(1-pf(aux,lag.cor,dof)))
    }
    names(faux)<-sapply(strsplit(colnames(X.dat),".",fixed=TRUE),function(x) x[[2]])
    Fstat[[i]]<-faux
    critL[[i]]<-critV<-c(critV,qf(alpha, lag.cor,dof))
    pL[[i]]<-pV
    dofL[[i]]<-dof
  }
  names(Fstat)<-ccode

  # Generate Output

  resTest<-array("-",dim=c(length(ccode)*2,(length(vars)+3)))
  colnames(resTest)<-c("Country","DoF",paste("F-crit."," (",alpha,")",sep=""),vars)
  arrayIdx<-(1:(length(ccode)*2))[c(TRUE, FALSE)]
  for(i in 1:length(arrayIdx)){
    resTest[arrayIdx[i],1:3]<-c(ccode[i],paste("F(",lag.cor," ,",dofL[[i]],")",sep=""),format(round(critL[[i]],dig1)))
    resTest[arrayIdx[i],names(Fstat[[i]])]<-c(format(round(Fstat[[i]],dig1)))
    resTest[arrayIdx[i]+1,names(Fstat[[i]])]<-paste("(",format(round(pL[[i]],dig2)),")",sep="")
  }
  # Generate p-table
  pp<-unlist(pL);K<-length(pp)
  p.res<-matrix(0,nrow=4,ncol=2)
  p.res[1,]<-c(length(which(pp>0.10)),paste0(round((length(which(pp>0.10))/K)*100,2),"%"))
  p.res[2,]<-c(length(which(pp>0.05& pp<=0.1)),paste0(round((length(which(pp>0.05& pp<=0.1))/K)*100,2),"%"))
  p.res[3,]<-c(length(which(pp>0.01& pp<=0.05)),paste0(round((length(which(pp>0.01& pp<=0.05))/K)*100,2),"%"))
  p.res[4,]<-c(length(which(pp<=0.01)),paste0(round((length(which(pp<=0.01))/K)*100,2),"%"))
  rownames(p.res)<-c(">0.1","0.05-0.1","0.01-0.05","<0.01")
  colnames(p.res)<-c("# p-values", "in %")

  res<-list(Fstat=Fstat,resTest=resTest,p.res=p.res,pL=pL)
  return(res)
}
#test=resid.corr.test(x=model.sims,lag.cor=4,alpha=0.95)


.print.vars<-function(gvar.obj){
  endo<-lapply(gvar.obj$cc.results$coeffs,colnames)
  exo<-lapply(gvar.obj$cc.results$coeffs,rownames)
  cN<-names(endo)


  vars<-list()
  for(i in 1:length(endo)){
    vars[[i]]<-c(gsub(paste0(cN[i],"."),"",endo[[i]]), exo[[i]][-grep("_lag",exo[[i]])])
    vars[[i]]<-vars[[i]][-charmatch("constant",vars[[i]])]
  }
  varNames<-lapply(vars,function(y) paste(y,collapse=", "))
  names(varNames)<-cN
  return(varNames)
}


print.bgvar<-function(gvar.obj){
  if (class(gvar.obj)!="bgvar") {
    stop("Submit an object of class bgvar!")
  }
  cat("---------------------------------------------------------------------------------------")
  cat("\n")
  cat("Model Info:")
  cat("\n")
  cat(paste0("Prior: ",gvar.obj$args$prior))
  cat("\n")
  cat(paste0("Nr. of lags: ",gvar.obj$args$plag))
  cat("\n")
  cat(paste0("Nr. of posterior draws: ",gvar.obj$args$saves,"*",gvar.obj$args$save_thin,"=",floor(gvar.obj$args$saves*gvar.obj$args$save_thin)))
  cat("\n")
  cat(paste0("Size of GVAR object: ",format(object.size(gvar.obj),units="MB")))
  cat("\n")
  cat(gvar.obj$stacked.results$trim.info)
  cat("\n")
  cat("---------------------------------------------------------------------------------------")
  cat("\n")
  res<-resid.corr.test(gvar.obj,lag.cor=1,alpha=0.95)
  cat(paste0("F-test, first order serial autocorrelation of cross-country residuals"))
  cat("\n")
  cat("Summary statistics:")
  cat("\n")
  print(res$p.res)
  cat("\n")
  cross.corr<-avg.pair.cc(gvar.obj)
  cat("--------------------------------------------------------------------------------------")
  cat("\n")
  cat(paste0("Average pairwise  cross-country correlation of country model residuals"))
  cat("\n")
  cat("Summary statistics:")
  cat("\n")
  print(cross.corr$res.res)
  cat("--------------------------------------------------------------------------------------")
  cat("\n")
  cat("Model specification:")
  cat("\n")
  Vars<-BGVAR:::.print.vars(gvar.obj)
  print(Vars)
  #return(list(resid=res,cross.corr=cross.corr,vars=Vars))
}



get.res<-function(gvar.obj){
  G.mat<-gvar.obj$stacked.results$Ginv_large
  A.mat<-gvar.obj$stacked.results$ALPHA_large
  plag<-gvar.obj$args$plag;saves<-dim(G.mat)[3]
  X<-gvar.obj$x;xglobal=gvar.obj$xglobal
  YY <- xglobal[(plag+1):nrow(xglobal),]
  XX <- cbind(BGVAR:::.mlag(xglobal,plag),1); XX <- XX[(plag+1):nrow(XX),]


  res.array.country<-res.array.global<-array(0,dim=c(dim(YY),saves))
  for(i in 1:saves){
    res.array.global[,,i]<-(YY-XX%*%t(A.mat[,,i]))
    res.array.country[,,i]<-(res.array.global[,,i]%*%t(solve(G.mat[,,i])))
  }
  results=list(res.array.global=res.array.global,res.array.country=res.array.country,Dat=gvar.obj$x)
  class(results)="res"
  return(results)
}



.avg.pip<-function(Names1=Names,PIP1=PIP,exoVars=exo,exo.countries1=exo.countries,endo1=endo){
  # here collect PIPs
  xx<-lapply(PIP1,rownames);for(i in 1:length(xx)){xx[[i]]<-gsub(paste0(Names1[i],"."),"",xx[[i]])}
  xx<-unique(unlist(xx))
  N<-length(Names1)
  pips<-array(NA,dim=c(length(xx),c(length(endo1)+length(exoVars)),N))
  dimnames(pips)[[1]]<-xx;dimnames(pips)[[2]]<-c(names(endo1),names(exoVars));dimnames(pips)[[3]]<-Names1
  for(i in 1:N){
    aux<-PIP1[[i]];rownames(aux)<-gsub(paste0(Names1[i],"."),"",rownames(aux));colnames(aux)<-gsub(paste0(Names1[i],"."),"",colnames(aux))
    for(z in 1:ncol(aux)){
      pips[rownames(aux),colnames(aux)[z],i]<-aux[,z]
    }
  }
  avg.pips<-apply(pips,c(1,2),function(x) mean(x,na.rm=TRUE))

  #kick out NAs
  idx1<-apply(pips,3,function(x) which(colSums(x,na.rm=TRUE)==0))
  idx2<-apply(pips,3,function(x) which(rowSums(x,na.rm=TRUE)==0))
  pips2<-list()
  for(i in 1:dim(pips)[3]){
     pips2[[i]]<-pips[,,i]
     idx1<-which(colSums(pips[,,i],na.rm=TRUE)==0)
     idx2<-which(rowSums(pips[,,i],na.rm=TRUE)==0)
     if(length(idx1)>0){
       pips2[[i]]<-pips2[[i]][,-idx1,drop=FALSE]
     }
     if(length(idx2)>0){
       pips2[[i]]<-pips2[[i]][-idx2,,drop=FALSE]
     }
  }
  names(pips2)<-names(PIP1)
  results=list(pips=pips2,avg.pips=avg.pips)
  return(results)
}


.relabel<-function(gW1=gW,Names1=Names,country.coeffs=country.coeffs,Plag=plag,exoVars=exo,exo.countries1=exo.countries,endo1=endo){
  xx<-varNames<-sapply(gW1,function(x) dimnames(x)[[1]])
  N<-length(varNames) # nr. of countries
  for(i in 1:length(xx)){xx[[i]]<-gsub(paste0(Names1[i],"."),"",xx[[i]])}
  unique.names<-unique(unlist(xx))
  if(length(exoVars)>0){
    Exo<-TRUE
  }
  else{
    Exo<-FALSE
  }

  # this does the re-naming
  for(i in 1:N){
    varx<-varNames[[i]]
    endo=grep(paste0(Names1[i],"."),varx)
    exo<-which(varx%in%names(exoVars))
    wex=which(!c(1:length(varx))%in%c(exo,endo))

    wexL<-endoL<-exoL<-c()

    for(j in 1:Plag){
        wexL<-c(wexL, paste0(varx[wex],"*_lag",j))
        if(length(exo)>0){
          exoL<-c(exoL,paste0(varx[exo],"**_lag",j))
        }
        endoL<-c(endoL,paste0(varx[endo],"_lag",j))
    }

    if(length(exo)>0){
      rownames(country.coeffs[[i]])<-c("constant",paste0(varx[wex],"*"), wexL,paste0(varx[exo],"**"),exoL,endoL)
    }else{
      rownames(country.coeffs[[i]])<-c("constant",paste0(varx[wex],"*"), wexL,endoL)
    }
  }

  names(country.coeffs)<-Names1
  return(country.coeffs)
}


.quart.mean<-function(x){
  x<-as.numeric(x)
  bigT<-length(x)
  means<-c()
  for(i in 1:(bigT/4)){
    means<-c(means,x[(i*4-3):(i*4)])
  }
  return(means)
}


fevd.decomp <- function(irf.obj,R=NULL,var.slct=NULL){

  ident<-irf.obj$ident

  # get posterior median objects from irf.obj
  Sigma_u=irf.obj$struc.obj$Sigma_u
  xglobal<-irf.obj$struc.obj$xglobal;x<-irf.obj$struc.obj$x
  K<-ncol(xglobal)
  horizon<-dim(irf.obj$posterior)[2]
  if(!ident%in%c("sign","chol")){
    stop("FEVD implemented for shocks identified via cholesky ordering or sign restrictions only.")
  }
  if(ident=="sign" && is.null(R)){ #we use the median rotation matrix at the posterior median
    R<-irf.obj$struc.obj$Rmed
  }
  if(ident=="chol"){
    R<-diag(K)
  }

  rownames(R) <- colnames(R) <- rownames(x)


  plag<-irf.obj$struc.obj$plag;ALPHA=irf.obj$struc.obj$ALPHA
  bigT<-ncol(x)
  Ginv<-irf.obj$struc.obj$Ginv
  Fmat<-irf.obj$struc.obj$Fmat
  S<-irf.obj$struc.obj$S

  # we use the irf.chol function to extract the gcov object and the dynamic multipliers, which are the same regardless whether we use
  # sign restrictions or recursive ordering
  irf.o<-BGVAR:::.irf.chol(maxlag=plag,ccode=irf.obj$struc.obj$ccode,shockvar=irf.obj$struc.obj$shockc,invG=Ginv,lF=Fmat,gcov=S,x=x,horizon=horizon,xglobal=xglobal,M=K,srots=1,rest=NA,rest.neg=NA,MaxTrys=NA,sign.constr=NA,Global=irf.obj$struc.obj$Global,ss=irf.obj$struc.obj$ss)
  PHI<-irf.o$PHI
  gcov<-irf.o$gcov



  vslct  <-  diag(K)
  invGSigmau  <-  Ginv%*%gcov # use gcov not SIGMA_u from irf.obj
  invGSinvG<-invGSigmau%*%t(Ginv)
  scale  <-  1/diag(gcov)



  if(!is.null(var.slct)){
    if(!all(var.slct%in%colnames(xglobal))){
      stop("One of the variables you want to decompose is not contained in the system. Please re-specify!")
    }
  }
  if(is.null(var.slct)){
    var.slct<-colnames(xglobal)
  }

  GFEVDres  <-  array(0,dim=c(K,horizon+1,length(var.slct)))
  dimnames(GFEVDres)[[1]]<-colnames(xglobal)
  dimnames(GFEVDres)[[2]]<-0:horizon
  dimnames(GFEVDres)[[3]]<-paste("Decomp. of",var.slct)


  for(zz in 1:length(var.slct)){
    eslct <-matrix(0,K,1);rownames(eslct) <- rownames(x) #input x
    eslct[var.slct[zz],1] <- 1


    num  <-  matrix(0,K,horizon+1)
    den  <-  matrix(0,K,horizon+1)


    N <- 1
    while (N<=horizon+1){
      for (l in 1:N){
        acc1  <-  t((t(eslct)%*%R%*%PHI[,,l]%*%invGSigmau%*%vslct)^2)
        num[,N]  <-  num[,N] + acc1
        acc2  <-  (t(eslct)%*%R%*%PHI[,,l]%*%invGSinvG%*%t(R%*%PHI[,,l])%*%eslct)
        den[,N]  <-  den[,N] + matrix(1,K,1)*as.numeric(acc2)
      }
      GFEVDres[,N,paste("Decomp. of",var.slct[zz])]  <-  (scale*num[,N])/den[,N]
      N <- N+1
    }
  }
  #------------------------------------------------------------------------------------------------------

  return(FEVD=GFEVDres)
}


bgvar.predict<-function(GVAR.obj,plot=FALSE,File="predict.pdf"){
  xglobal<-GVAR.obj$xglobal
  plag<-GVAR.obj$args$plag
  YY <- xglobal[(plag+1):nrow(xglobal),]
  Res<-get.res(GVAR.obj)$res.array.global
  Y.hat<-YY-apply(Res,c(1,2),median)
  if(plot){
    pdf(File)
    for(i in 1:ncol(YY)){
      matplot(cbind(YY[,i],Y.hat[,i]),type="l",ylab="",main=colnames(YY)[i])
      legend("topleft",col=1:2,lty=1:2,c("series","fitted"),bty="n")
      grid()
    }
    dev.off()
  }
  return(list(Y=YY,Y.hat=Y.hat))
}


.irf.girf.sims <- function(invG=gvarobj$G,lF=gvarobj$F,gcov=sigs,x=gvarobj$x,horizon=40,...){

  Names <- substr(rownames(x),1,2)
  Names <- Names[!duplicated(Names)]
  N <- length(Names)
  Cm <- as.matrix(gcov)
  K <- nrow(x)
  p<-dim(lF)[[3]] # number of lags

  invGSigma_u  <-  invG%*%Cm;rownames(invGSigma_u)<-colnames(invGSigma_u)<-rownames(x)

  A_array<-lF[,,,1] # lF contains the reduced form coefficients
  if(p==1){dim(A_array)<-c(K,K,1)}
  impls.girf<- BGVAR:::.impulsdtrf(A_array,invGSigma_u,(horizon))

  cons.girf<-(1/diag(invGSigma_u))*sqrt(diag(invGSigma_u))
  # irfa is a K responses times K shocks times horizon array
  irfa<-impls.girf*array(matrix(cons.girf,K,K,byrow=TRUE),dim=c(K,K,(horizon)))

  return(list(impl=irfa,gcov=Cm))
}


.impulsdtrf <- function(B,smat,nstep)
  ### By:             As emerges from rfvar, neqn x nvar x lags array of rf VAR coefficients.
  ### smat:           nshock x nvar matrix of initial shock vectors.  To produce "orthogonalized
  ###                 impulse responses" it should have the property that crossprod(t(smat))=sigma,
  ###                 where sigma is the Var(u(t)) matrix and u(t) is the rf residual vector.  One
  ###                 way to get such a smat is to set smat=t(chol(sigma)).  To get the smat
  ###                 corresponding to a different ordering, use
  ###                 smat = t(chol(P %*% Sigma %*% t(P)) %*% P), where P is a permutation matrix.
  ###                 To get impulse responses for a structural VAR in the form A(L)y=eps, with
  ###                 Var(eps)=I, use B(L)=-A_0^(-1)A_+(L) (where A_+ is the coefficients on strictly
  ###                 positive powers of L in A), smat=A_0^(-1).
  ###                 In general, though, it is not required that smat be invertible.
### response:       nvar x nshocks x nstep array of impulse responses.
###
### Code written by Christopher Sims, based on 6/03 matlab code.  This version 3/27/04.
### Added dimension labeling, 8/02/04.
{

  neq <- dim(B)[1]
  nvar <- dim(B)[2]
  lags <- dim(B)[3]
  dimnB <- dimnames(B)
  if(dim(smat)[2] != dim(B)[2]) stop("B and smat conflict on # of variables")
  response <- array(0,dim=c(neq,nvar,nstep+lags-1));
  response[ , , lags] <- smat
  response <- aperm(response, c(1,3,2))
  irhs <- 1:(lags*nvar)
  ilhs <- lags * nvar + (1:nvar)
  response <- matrix(response, ncol=neq)
  B <- B[, , seq(from=lags, to=1, by=-1)]  #reverse time index to allow matrix mult instead of loop
  B <- matrix(B,nrow=nvar)
  for (it in 1:(nstep-1)) {
    response[ilhs, ] <- B %*% response[irhs, ]
    irhs <- irhs + nvar
    ilhs <- ilhs + nvar
  }
   dim(response) <- c(nvar, nstep + lags - 1, nvar)
  #drop the zero initial conditions; array in usual format
  if(lags>1){
    response<-response[,-(1:(lags-1)),]
  }
  response <- aperm(response, c(1, 3, 2))
  dimnames(response) <- list(dimnB[[1]], dimnames(smat)[[2]], NULL)
  ## dimnames(response)[2] <- dimnames(smat)[1]
  ## dimnames(response)[1] <- dimnames(B)[2]
  return(response)
}


.mk_fevd.sims <- function(irfa){
  ny <- dim(irfa)[[1]];nH <- dim(irfa)[[3]]

  fevda <- apply(irfa*irfa,c(1,2),cumsum);
  fevda <- aperm(fevda,c(2,3,1))
  accm <- matrix(0,ny,ny)
  for (ih in 1:nH){
    accm <- accm+irfa[,,ih]%*%t(irfa[,,ih])
    denm <- matrix((diag(accm)),ny,ny)
    fevda[,,ih]=fevda[,,ih]/denm
  }
  return(fevda)
}

GFEVD.LN<-function(gvar.obj,nhor=24,cpu=1,running=TRUE){
  require(doParallel)
  x<-gvar.obj$x
  bigT<-ncol(x)
  F_large<-gvar.obj$stacked.results$F_large
  S_large<-gvar.obj$stacked.results$S_large
  F.eigen<-gvar.obj$stacked.results$F.eigen
  Ginv_large<-gvar.obj$stacked.results$Ginv_large
  plag<-gvar.obj$args$plag
  xglobal<-gvar.obj$xglobal
  kkk<-ncol(xglobal)
  Kbig<-plag*kkk


  saves<-dim(Ginv_large)[3]
  xglobal<-gvar.obj$xglobal
  shock.nr<-1



  # END initializing --------------------------------------------------------------------------------------------------------------

 if(running){ # calculate only running mean
   GFEVD_post<-array(0,dim=c(kkk,kkk,nhor)) # mean(sic!)
   dimnames(GFEVD_post)[[1]]<-dimnames(GFEVD_post)[[2]]<-colnames(xglobal)
   dimnames(GFEVD_post)[[3]]<-0:(nhor-1)

     # start loop here########
  if(cpu>1){# use parallel computing for IRF analysis
    registerDoParallel(cores=cpu)
    cat("\n","Start impulse response analysis on", cpu, "cores.","\n")
    sfilename<-"log.txt";file.create(sfilename)
    cat(as.character(Sys.time()),": starting loop ... \n",append=TRUE, file=sfilename)  #write one line
    imp.obj <-foreach(irep1=1:saves) %dopar%{
      irfa<-BGVAR:::.irf.girf.sims(Ginv_large[,,irep1],lF=F_large[,,,irep1,drop=FALSE],gcov=S_large[,,irep1],x=x,horizon=nhor)$impl
      GFEVD<-BGVAR:::.mk_fevd.sims(irfa)
      cat(as.character(Sys.time()),":",irep1,"current draw \n",append=TRUE, file=sfilename)
      rm(irfa);gc()
      return(list(GFEVD=GFEVD))
    }

    for(irep1 in 1:saves){
      GFEVD_post<-GFEVD_post+imp.obj[[irep1]]$GFEVD
    }
     GFEVD_post<-GFEVD_post/saves
  }else{ #carry out impulse response function analysis in a loop
    cat("\n","Start impulse response analysis","\n")
    pb <- txtProgressBar(min = 0, max = saves, style = 3)
    for(irep1 in 1:saves){
      #t1<-Sys.time()
      #F_array <- array(F_large[,,,irep1],c(kkk,kkk,plag))
      irfa<-BGVAR:::.irf.girf.sims(Ginv_large[,,irep1],lF=F_large[,,,irep1,drop=FALSE],gcov=S_large[,,irep1],x=x,horizon=nhor)$impl
      GFEVD_post<-GFEVD_post+BGVAR:::.mk_fevd.sims(irfa)
      setTxtProgressBar(pb, irep1)
    }
    GFEVD_post<-GFEVD_post/saves
  }
 }else{ #-------------------------HERE DO FULL CALCULATION INCLUDING BOUNDS- VERY MEMORY INTENSIVE!!!-----
   GFEVD_draws<-array(NA,dim=c(kkk,kkk,nhor,saves)) # GFEVD Post object
   GFEVD_post<-array(NA,dim=c(kkk,kkk,nhor,3)) # low mean(sic!) high
   dimnames(GFEVD_post)[[1]]<-dimnames(GFEVD_post)[[2]]<-colnames(xglobal)
   dimnames(GFEVD_post)[[3]]<-0:(nhor-1)
   dimnames(GFEVD_post)[[4]]<-c("low16","mean","high84")

   # start loop here########
   if(cpu>1){# use parallel computing for IRF analysis
     registerDoParallel(cores=cpu)
     cat("\n","Start impulse response analysis on", cpu, "cores.","\n")
     sfilename<-"log.txt";file.create(sfilename)
     cat(as.character(Sys.time()),": starting loop ... \n",append=TRUE, file=sfilename)  #write one line
     imp.obj <-foreach(irep1=1:saves) %dopar%{
       irfa<-BGVAR:::.irf.girf.sims(Ginv_large[,,irep1],lF=F_large[,,,irep1,drop=FALSE],gcov=S_large[,,irep1],x=x,horizon=nhor)$impl
       GFEVD<-BGVAR:::.mk_fevd.sims(irfa)
       cat(as.character(Sys.time()),":",irep1,"current draw \n",append=TRUE, file=sfilename)
       rm(irfa);gc()
       return(list(GFEVD=GFEVD))
     }

     for(irep1 in 1:saves){
       GFEVD_draws[,,,irep1] <- imp.obj[[irep1]]$GFEVD
     }
   }else{ #carry out impulse response function analysis in a loop
     cat("\n","Start impulse response analysis","\n")
     pb <- txtProgressBar(min = 0, max = saves, style = 3)
     for(irep1 in 1:saves){
       #t1<-Sys.time()
       #F_array <- array(F_large[,,,irep1],c(kkk,kkk,plag))
       irfa<-BGVAR:::.irf.girf.sims(Ginv_large[,,irep1],lF=F_large[,,,irep1,drop=FALSE],gcov=S_large[,,irep1],x=x,horizon=nhor)$impl
       GFEVD_draws[,,,irep1]<-BGVAR:::.mk_fevd.sims(irfa)

       setTxtProgressBar(pb, irep1)
     }
   }
   GFEVD_post[,,,"low16"]<-apply(GFEVD_draws,c(1,2,3),quantile,0.16,na.rm=TRUE)
   GFEVD_post[,,,"mean"]<-apply(GFEVD_draws,c(1,2,3),mean,na.rm=TRUE)
   GFEVD_post[,,,"high84"]<-apply(GFEVD_draws,c(1,2,3),quantile,0.84,na.rm=TRUE)
 }


  #----------------------------------------------------------------------------------------------------------------------------------------------------------------
  # END LOOP HERE ----------------------------------------------------------------------------------------------------------------
  #-----------------------------------------------------------------------------------------------------------------------------------------------------------------



  results<-list(GFEVD_post=GFEVD_post)
  cat(paste("Size of IRF object:", format(object.size(results),unit="MB")))
  return(results)
}


rot.eval<-function(File){
  draws<-read.table(File,sep="\t",header=TRUE)
  total<-as.numeric(gsub(".","",strsplit(unlist(strsplit(colnames(draws),"cores",fix=TRUE))[2],"stable",fix=TRUE)[[1]][1],fix=TRUE))
  idx=which(grepl("after",draws[,1]))
  cat(paste0("So far ", length(idx), " rotations out of ", nrow(draws), " found (", round(length(idx)/nrow(draws)*100,2),"%)."), "\n")


  rot.idx<-draws[idx,1]

  tries<-NULL
  for(i in 1:length(rot.idx)){
    tries<-c(tries,as.numeric(unlist(strsplit(strsplit(as.character(rot.idx[i]),"after")[[1]][2],"tries")))[[1]])
  }

  print(summary(tries))

  # gather start time
  year<-substr(colnames(draws),2,11)
  time<-substr(colnames(draws),13,20)
  start.time<-strptime(paste(year,time),"%Y.%m.%d %H.%M.%S")
  elapsed<-difftime(Sys.time(),start.time, units = c( "hours"))
  print(elapsed)

  #time.per.rot<-round(as.numeric(elapsed)/length(idx)*60,2)
  #cat(paste0(time.per.rot, " minutes per successful rotation."))

  cat("\n",paste0("****Still ", total-nrow(draws)," out of ",total, " MCMC draws left to check!****"))
  #cat("\n",paste0("****",round((total-nrow(draws))*as.numeric(elapsed),2))," minutes to go!***")


  hist(tries)
  return(tries)
}
