\name{avg.pair.cc}
\alias{avg.pair.cc}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Average pairwise cross-sectional correlations
}
\description{
Computes average pairwise cross-sectional correlations of the data and the country models' residuals. To be more specific,
for each country and a given variable, the average cross-sectional correlation is computed (either for the data or for the residuals). In theory, including foreign variables should soak up cross-sectional residual dependence and  correlation of the residuals should be small. Otherwise dynamic analysis, especially using GIRFs, might lead to invalid results. See Dees et al. (2007) for more details.
}
\usage{
avg.pair.cc(X, dig = 3)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{X}{ Either an object fitted by \code{bgvar} (class \code{bgvar}) or residuals extracted via \code{get.res} (class \code{res}).
}
  \item{dig}{ Number of digits that should be used to print output to the console.
}
}
\details{
%%  ~~ If necessary, more details than the description above ~~
}
\value{

\item{data.cor }{ is a matrix containing in the rows the cross-sections and in the columns the cross-sectional pairwise correlations of the data per variable. }
\item{resid.cor }{is a matrix containing in the rows the cross-sections and in the columns the cross-sectional pairwise correlations of the country models' residuals per variable.}
\item{resid.corG}{is a matrix containing in the rows the cross-sections and in the columns the cross-sectional pairwise correlations of the global models' residuals per variable. Only availabe when \code{avg.pair.cc} has been applied to a \code{res} object from \code{get.res}.}
\item{dat.res }{ is a summary object showing the number and percentage of correlations <0.1, between 0.1-0.2, 0.2-0.5 and <0.5 per variable of the data. }
\item{res.res }{ is a summary object showing the number and percentage of correlations <0.1, between 0.1-0.2, 0.2-0.5 and <0.5 per variable of the country models' residuals. This is also what is used by \code{print.bgvar}.}
\item{res.resG}{is a summary object showing the number and percentage of correlations <0.1, between 0.1-0.2, 0.2-0.5 and <0.5 per variable of the global models' residuals. Only availabe when \code{avg.pair.cc} has been applied to a \code{res} object from \code{get.res}.}
}
\references{
Dees, S., Di Mauro, F., Pesaran, M.H. and L. V. Smith (2007) Exploring the international linkages of the euro
area: A global VAR analysis. In: Journal of Applied Econometrics, Vol. 22, pp. 1-38.
}
\author{
Martin Feldkircher
}
\note{
%%  ~~further notes~~
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
See also \code{\link{print.bgvar}}, \code{\link{get.res}}.
}
\examples{
library(BGVAR)
data(eerData)
model.sims<-bgvar(Data=eer.data,W=W.trade0012,saves=100,burns=100,plag=1,Cpu=4,prior="SIMS",save_thin=1,eigen=TRUE,logfile=FALSE,stats=TRUE)
avg.pair.cc(model.sims)

get.Res<-get.res(model.sims)
avg.pair.cc(get.Res)

}

