\name{conv.diag}
\alias{conv.diag}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{MCMC Convergence Diagnostics
}
\description{
This function invokes Geweke's CD using the \code{coda} package. Geweke (1992) proposed a convergence diagnostic for Markov chains based on a test for equality of the means of the first and last part of a Markov chain (by default we use the first 10\% and the last 50\%). If the samples are drawn from the stationary distribution of the chain, the two means are equal and Geweke's statistic has an asymptotically standard normal distribution.

The test statistic is a standard Z-score: the difference between the two sample means divided by its estimated standard error. The standard error is estimated from the spectral density at zero and so takes into account any autocorrelation.
}
\usage{
conv.diag(gvar.obj)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{gvar.obj}{ An objected fitted by function \code{bgvar}.}
}
\details{
%%  ~~ If necessary, more details than the description above ~~
}
\value{
\itemize{
  \item{\code{geweke.z}}{Z-scores for a test of equality of means between the first and last parts of the chain. A separate statistic is calculated for each variable in each chain.}
  \item{\code{xx}}{The percentage of Z-scores exceeding 1.96 (in absolute terms).}
}
}
\references{
Geweke, J. (1992) Evaluating the accuracy of sampling-based approaches to calculating posterior moments. In Bayesian Statistics 4 (ed JM Bernado, JO Berger, AP Dawid and AFM Smith). Clarendon Press, Oxford, UK.
}
\author{
Martin Feldkircher
}
\note{
Depends on the \code{coda} package.
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{geweke.diag} in the \code{coda} package.
}
\examples{
library(BGVAR)
data(monthlyData)
model.sims<-bgvar(Data=new.data,W=W,saves=100,burns=100,plag=1,hyperparm=NULL,Cpu=1,prior="SIMS",save_thin=1,eigen=TRUE,logfile=FALSE,stats=TRUE,trim=1.05)
geweke<-conv.diag(model.sims)
}

