\name{eerData}
\alias{eerData}
\docType{data}
\title{Example data set to replicate Feldkircher and Huber (2016)
}
\description{
This data set contains 76 quarterly observations by country, spanning the period from 1995Q1 to 2013Q4. The country coverage is 43 countries + the euro area (EA) as a regional aggregate.
}

\details{The data loads two objects \code{eer.data}, which is a list object of length \code{N} (i.e, the number of countries) and \code{W.trade0012}, which is an \code{N} times \code{N} weight matrix with rowsums summing up to unity and zero elements on its diagonal. The global variable, oil prices, is included in the US country model as e.g., in Dees et al. (2007). The countries are abbreviated using ISO-2 codes. The weight matrix corresponds to average annual bilateral trade flows (including services) over the period from 2000 to 2012.\code{eer.data} contains the country data, for more details, see below:

\itemize{
\item{W.trade0012} {\code{N} times \code{N} weight matrix based on trade flows, rowsums equal unity.}
\item{\code{W.list}}{ A list of 10 weight matrices, described in Feldkircher and Huber (2016).}
\item{eer.data} { is a list object of length \code{N} containing
\itemize{
\item{\code{y}}{ Real GDP, average of 2005=100. Seasonally adjusted, in logarithms.}
\item{\code{Dp}}{ Consumer price inflation (period-on-period). CPI seasonally adjusted, in logarithms.}
\item{\code{stir}}{ Short-term interest rate, typically 3 months money market rate.}
\item{\code{ltir}}{ Long term interest rates, typically 10-year government bond yields.}
\item{\code{reer}}{ Real effective exchange rate index, deflated by consumer prices.}
\item{\code{tb}}{ Trade balance (ratio of real exports to real imports).}
\item{\code{poil}}{ Price of oil, seasonally adjusted, in logarithms.}
}
}
}
}
\usage{data(eerData)}


\source{
%%  ~~ reference to a publication or URL from which the data were obtained ~~
}
\references{
Feldkircher, M. and F. Huber (2016) The International Transmission of US Shocks - Evidence from Bayesian Global Vector Autoregressions. In:  European Economic Review, Vol. 81, pages 167-188.

Dees, S., di Mauro, F., Pesaran, H. M. and L. V. Smith (2007) Exploring the international
linkages of the euro area: a global VAR analysis. In: Journal of Applied Econometrics 22(1).

}
\examples{
# DO NOT RUN (THIS REPLICATES FELDKIRCHER AND HUBER, 2016)
library(BGVAR)
data(eerData)
#model.ssvs.eer<-bgvar(Data=eer.data,W=W.trade0012,saves=15000,burns=5000,plag=1,Cpu=28,prior="SSVS",save_thin=0.3,eigen=TRUE,trim=1.05)
#print.bgvar(model.ssvs.eer)


# sign.constr.eer<-list()
# sign.constr.eer$shock1$shock<-"US.y" # Positive AD Shock, gdp goes up,
# sign.constr.eer$shock1$restrictions$res1<-"US.Dp" #  inflation up and interest rates as well
# sign.constr.eer$shock1$restrictions$res2<-"US.stir"
# sign.constr.eer$shock1$sign<-c(">",">",">")
# sign.constr.eer$shock1$rest.horz<-c(1,1,1)
# sign.constr.eer$shock1$constr<-c(1,1,1)# no cross-country restrictions, set constr. to 1
# sign.constr.eer$shock1$scal<-1 #+1% increase
# sign.constr.eer$shock2$shock<-"US.Dp" # Negative AS shock, inflation up
# sign.constr.eer$shock2$restrictions$res1<-"US.y"
# sign.constr.eer$shock2$restrictions$res2<-"US.stir" # interest rates up (since M/P P up, LM to the left)
# sign.constr.eer$shock2$restrictions$res3<-"US.rer"#  and real exchange rate up (since stir up and infl up)
# sign.constr.eer$shock2$sign<-c(">","<",">",">")
# sign.constr.eer$shock2$rest.horz<-c(1,1,1,1)
# sign.constr.eer$shock2$constr<-c(1,1,1,1) # no cross-country restrictions, set constr. to 1
# sign.constr.eer$shock2$scal<-1 #+1% increase
# sign.constr.eer$shock3$shock<-"US.stir" # Contractionary MP shock
# sign.constr.eer$shock3$restrictions$res1<-"US.y"
# sign.constr.eer$shock3$restrictions$res2<-"US.Dp"
# sign.constr.eer$shock3$restrictions$res3<-"US.ltir"
# sign.constr.eer$shock3$sign<-c(">","<","<","<")
# sign.constr.eer$shock3$rest.horz<-c(1,1,1,1)
# sign.constr.eer$shock3$constr<-c(1,1,1,1)# no cross-country restrictions, set constr. to 1
# sign.constr.eer$shock3$scal<-1 #-100 bp decrease
# names(sign.constr.eer)<-c("AD","AS","MP")
#
# rotSpec<-list();rotSpec$Srots<-4;rotSpec$MaxTries<-10000
# irf.sign<-IRF(gvar.obj=model.ssvs.eer,shockc=NULL,nhor=48,sign.constr=sign.constr.eer,rotSpec=rotSpec)
#




  # AD shock -----------------------------------------------------------------
  #irf.plot(irf.sign,resp="US.y",shock.nr=1)
  #irf.plot(irf.sign,resp="US.Dp",shock.nr=1)
  #irf.plot(irf.sign,resp="US.stir",shock.nr=1)
  #irf.plot(irf.sign,resp="US.ltir",shock.nr=1)
  #irf.plot(irf.sign,resp="US.tb",shock.nr=1)
  #irf.plot(irf.sign,resp="US.poil",shock.nr=1)
  #irf.plot(irf.sign,resp="US.rer",shock.nr=1)

  # AS shock -----------------------------------------------------------------
  #irf.plot(irf.sign,resp="US.y",shock.nr=2)
  #irf.plot(irf.sign,resp="US.Dp",shock.nr=2)
  #irf.plot(irf.sign,resp="US.stir",shock.nr=2)
  #irf.plot(irf.sign,resp="US.ltir",shock.nr=2)
  #irf.plot(irf.sign,resp="US.tb",shock.nr=2)
  #irf.plot(irf.sign,resp="US.poil",shock.nr=2)
  #irf.plot(irf.sign,resp="US.rer",shock.nr=2)
  # MP shock -----------------------------------------------------------------
  #irf.plot(irf.sign,resp="US.y",shock.nr=3)
  #irf.plot(irf.sign,resp="US.Dp",shock.nr=3)
  #irf.plot(irf.sign,resp="US.stir",shock.nr=3)
  #irf.plot(irf.sign,resp="US.ltir",shock.nr=3)
  #irf.plot(irf.sign,resp="US.tb",shock.nr=3)
  #irf.plot(irf.sign,resp="US.poil",shock.nr=3)
  #irf.plot(irf.sign,resp="US.rer",shock.nr=3)





}
\keyword{datasets}
