\name{get.res}
\alias{get.res}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Calculate residuals of the global and the country models
}
\description{
This function calculates residuals of the global and the country models based on a \code{gvar object}. Country models' residuals are equivalent to output generated by the \code{print.bgvar} function in case no trimming has been used. If trimming was invoked to discard unstable draws output of both functions might differ since \code{print.bgvar} calculates residuals as a running mean to save storage which is based on the \emph{whole} set of posterior draws (including discarded draws). In this case it is recommended to recalculate the residuals with \code{get.res} and re-do the serial autocorrelation or average pairwise cross-correlation analysis using functions \code{resid.corr.test} and \code{avg.pair.cc}
}
\usage{
get.res(gvar.obj)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{gvar.obj}{
A fitted gvar object.
}
}
\details{
%%  ~~ If necessary, more details than the description above ~~
}
\value{
\item{res.array.global}{ A (T-p) times K times saves*save_thin array containing the residuals of the global model.}
\item{res.array.country}{ A (T-p) times K times saves*save_thin array containing the residuals of the country models.}
}
\references{
%% ~put references to the literature/web site here ~
}
\author{
Martin Feldkircher
}
\note{
%%  ~~further notes~~
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link{print.bgvar}}, \code{\link{resid.corr.test}}, \code{\link{avg.pair.cc}}
}
\examples{
# This example demonstrates the equivalence of the cross-country residuals that are returned from the gvar object
# and the get.res residulas if no trimming is invoked for the SIMS, SSVS and NG priors.
library(BGVAR)

data(monthlyData)
model.sims<-bgvar(Data=new.data,W=W,saves=100,burns=100,plag=1,hyperparm=NULL,Cpu=1,prior="SIMS",save_thin=1,eigen=TRUE,logfile=FALSE,stats=TRUE)
aa=get.res(model.sims)

#calculate "running mean" as in bgvar function
aa=apply(aa$res.array.country,c(1:2),sum)/100
matplot(cbind(aa[,1],model.sims$country.res[,1]),type="l")

model.ssvs<-bgvar(Data=new.data,W=W,saves=100,burns=100,plag=1,hyperparm=NULL,Cpu=1,prior="SSVS",save_thin=1,eigen=TRUE,logfile=FALSE,stats=TRUE)
aa=get.res(model.ssvs)

#calculate "running mean" as in bgvar function
aa=apply(aa$res.array.country,c(1:2),sum)/100
matplot(cbind(aa[,1],model.ssvs$country.res[,1]),type="l")

model.NG<-bgvar(Data=new.data,W=W,saves=100,burns=100,plag=1,hyperparm=NULL,Cpu=1,prior="NG",save_thin=1,eigen=TRUE,logfile=FALSE,stats=TRUE,ea.weights=EA.weights)
aa=get.res(model.NG)

#calculate "running mean" as in bgvar function
aa=apply(aa$res.array.country,c(1:2),sum)/100
matplot(cbind(aa[,1],model.NG$country.res[,1]),type="l")
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
% __ONLY ONE__ keyword per line
