\name{monthlyData}
\alias{monthlyData}
\docType{data}
\title{Example data set
}
\description{
This data set contains monthly observations on industrial production, consumer price indices, short- and long-term interest rates, real effective exchange rates and equity prices. The time period covered is from January 2000 to December 2015 and the country coverage amounts to  28 countries -- roughly corresponding to EU member states + G-8 countries and a country model to model common monetary policy in the euro area.
}

\details{The data loads three objects \code{monthly.data}, which is a list object of length \code{N+1} (i.e, the number of countries + the ECB country model), \code{W}, which is an \code{N} times \code{N} weight matrix with rowsums summing up to unity and zero elements on its diagonal. The countries are abbreviated using ISO-2 codes. The weight matrix corresponds to average annual input output flows for the \code{N} countries over the period from 2000 to 2014. The data are from the world input outpu table database (\url{www.wiod.org}) and are fully described in Timmerman et al. (2015). \code{monthly.data} contains the country data. Per default, variables that should affect all countries (global variables) are treated as endogenoues variables in the US country model (\code{poil}, \code{pcom}, \code{vix}).
Akin to Georgiadis (2015), interest setting in the euro area is modeled by a Taylor rule that includes ppp-weighted output and prices of euro area countries. The euro area interest rate enters other country models as an additional exogeneous variable.
For more details, see below:

\itemize{
\item{W.} {\code{N} times \code{N} weight matrix, rowsums equal unity.}
\item{monthly.data} { is a list object of length \code{N} containing
\itemize{
\item{\code{y}}{ Industrial production index, in real terms, logarithmic transform and seasonally adjusted.}
\item{\code{p}}{ Harmonized Consumer Price Index (HCPI) for EU member states, for other countries Consumer Price Index. Data in logarithmic transform and seasonally adjusted.}
\item{\code{stir}}{ Short-term interest rate, typically 3 months money market rate.}
\item{\code{EAstir}}{ Short-term interest rate, typically 3 months money market rate (3 months euribor).}
\item{\code{ltir}}{ Long term interest rates, typically 10-year government bond yields.}
\item{\code{er}}{ Real effective exchange rate index, deflated by consumer prices.}
\item{\code{eq}}{ Equity price index, in logarithmic transform.}
\item{\code{poil}}{ Price of oil, seasonally adjusted, in logarithms.}
\item{\code{pcom}}{ Commodity price index, seasonally adjusted, in logarithms.}
\item{\code{vix}}{ Volatility index, in logarithms.}
}
}
}
}

\usage{data(monthlyData2)}


\source{
%%  ~~ reference to a publication or URL from which the data were obtained ~~
}
\references{
Georgiadis, G. (2015) Examining asymmetries in the transmission of monetary policy in the euro area: Evidence from a mixed cross-section global VAR model. In: European Economic Review, Vol. 75, pp. 195-215.

Timmer, M. P., Dietzenbacher, E., Los, B., Stehrer, R. and de Vries, G. J. (2015)
An Illustrated User Guide to the World Input–Output Database: the Case of Global Automotive Production. In:
Review of International Economics, Vol. 23, pp. 575–605.
}
\examples{
library(BGVAR)
data(monthlyData2)
names(new.data2) # shows the countries contained in the data set
ts.plot(new.data2$AT[,"y"]) # plots industrial production for Austria
}
\keyword{datasets}
