  # Note this script is not (sic!) a full replication file of the study 
  # Global Inflation Dynamics and Inflation Expectations, Martin Feldkircher and Pierre Siklos. 
  # International Review of Economics & Finance, 2019, Vol. 64, pp. 217-241. 
  # The data included in this replicatin script does not include inflation expectations
  # that are based on Consensus economics. This script serves only for illustrationary purposes


  load("replication_data.RData")
  library(BGVAR)
  
  hyperparm<-list(c_tau = 0.01, d_tau = 0.01,e_lambda=1.5,d_lambda=1, prmean=1,a_start=.6,sample_A=FALSE)#standard setting
  hyper<-hyperparm;hyper$a_start=0.6;hyper$e_lambda<-3;hyper$d_lambda<-0.03;hyper$prmean<-0;hyper$a_i=0.01;hyper$b_i=0.01
  # this estimates the model without inflation expectations data from Consensus Economics
  # also we use a standard weight matrix based on bilateral trade flows for the purpose of illustration
  # note, we use typically parallel computing, setting Cpu to the nr. of countries (42+2)
  model.ng.p2.mixed.ur.dpyoy<-bgvar(new.data,W=W.0012,saves=20000,burns=30000,plag=2,Cpu=length(new.data),prior="NG",hyperparm=hyper,save_thin=0.1,eigen=TRUE,stats=TRUE,logfile=TRUE,trim=1.05,ea.weights=EA.weights,OC.weights=OC.weights)

  
  # after estimation we can calculate IRFs based on sign restrictions
  
  cn<-names(model.ng.p2.mixed.ur.dpyoy.poilyoy$args$Data);cn<-rev(rev(cn)[-c(1:2)])
  Cores<-length(cn)
  
  EA<-c("AT", "BE" ,"DE" , "GR" ,"ES", "FI", "FR", "IE", "IT","NL", "PT","SI","SK")
  
  
  
  
  library(doParallel)
  
foreach(i=1:length(cn))%dopar%{ 
    sfilename="sign.log"
    file.create(sfilename)
    cat(as.character(Sys.time()),": country", cn[i]," ... \n",append=TRUE, file=sfilename)  #write one line
    if(cn[i]%in%EA){
      sign.constr.eer<-list()
      sign.constr.eer$shock1$shock<-paste0(cn[i],".gdp.m") # Positive AD Shock, gdp goes up,
      sign.constr.eer$shock1$restrictions$res1<-paste0(cn[i],".Dp.yoy") #  inflation up and interest rates as well
      sign.constr.eer$shock1$restrictions$res2<-"EB.EAstir"
      sign.constr.eer$shock1$restrictions$res3<-paste0(cn[i],".reer")#  and real exchange rate up (since stir up and infl up)
      sign.constr.eer$shock1$restrictions$res4<-paste0(cn[i],".ur") # Blanchard and Galí (2010):  negative relationship of output and unemployment
      sign.constr.eer$shock1$sign<-c(">",">",">",">","<")
      sign.constr.eer$shock1$rest.horz<-c(1,1,1,1,1)
      sign.constr.eer$shock1$constr<-c("all","all","all","all","all")
      sign.constr.eer$shock1$scal<-0.01 #+1% increasee
      sign.constr.eer$shock2$shock<-paste0(cn[i],".Dp.yoy") # Negative AS shock, inflation up
      sign.constr.eer$shock2$restrictions$res1<-paste0(cn[i],".gdp.m")
      sign.constr.eer$shock2$restrictions$res2<-"EB.EAstir" # interest rates up (since M/P P up, LM to the left)
      sign.constr.eer$shock2$restrictions$res3<-paste0(cn[i],".reer")#  and real exchange rate up (since stir up and infl up)
      sign.constr.eer$shock2$restrictions$res4<-paste0(cn[i],".ur") # unemployment and price level in same direction
      sign.constr.eer$shock2$sign<-c(">","<",">",">",">")
      sign.constr.eer$shock2$rest.horz<-c(1,1,1,1,1)
      sign.constr.eer$shock2$constr<-c("all","all","all","all","all")
      sign.constr.eer$shock2$scal<-1 #+1% increase
    }
    if(cn[i] %in%c("CN","TR","ID","IN","MY","PE")){
      sign.constr.eer<-list()
      sign.constr.eer$shock1$shock<-paste0(cn[i],".gdp.m") # Positive AD Shock, gdp goes up,
      sign.constr.eer$shock1$restrictions$res1<-paste0(cn[i],".Dp.yoy") #  inflation up and interest rates as well
      sign.constr.eer$shock1$restrictions$res2<-paste0(cn[i],".stir")
      sign.constr.eer$shock1$restrictions$res3<-paste0(cn[i],".reer")#  and real exchange rate up (since stir up and infl up)
      #sign.constr.eer$shock1$restrictions$res3<-paste0(cn[i],".ur") # Blanchard and Galí (2010):  negative relationship of output and unemployment
      sign.constr.eer$shock1$sign<-c(">",">",">",">")
      sign.constr.eer$shock1$rest.horz<-c(1,1,1,1)
      sign.constr.eer$shock1$constr<-c("all","all","all","all")
      sign.constr.eer$shock1$scal<-0.01 #+1% increase
      sign.constr.eer$shock2$shock<-paste0(cn[i],".Dp.yoy") # Negative AS shock, inflation up
      sign.constr.eer$shock2$restrictions$res1<-paste0(cn[i],".gdp.m")
      sign.constr.eer$shock2$restrictions$res2<-paste0(cn[i],".stir") # interest rates up (since M/P P up, LM to the left)
      sign.constr.eer$shock2$restrictions$res3<-paste0(cn[i],".reer")#  and real exchange rate up (since stir up and infl up)
      #sign.constr.eer$shock2$restrictions$res4<-paste0(cn[i],".ur") # unemployment and price level in same direction
      sign.constr.eer$shock2$sign<-c(">","<",">",">")
      sign.constr.eer$shock2$rest.horz<-c(1,1,1,1)
      sign.constr.eer$shock2$constr<-c("all","all","all","all")
      sign.constr.eer$shock2$scal<-1 #+1% increase
    }
    if(!cn[i]%in%c(EA, "CN","TR","ID","IN","MY","PE")){
      sign.constr.eer<-list()
      sign.constr.eer$shock1$shock<-paste0(cn[i],".gdp.m") # Positive AD Shock, gdp goes up,
      sign.constr.eer$shock1$restrictions$res1<-paste0(cn[i],".Dp.yoy") #  inflation up and interest rates as well
      sign.constr.eer$shock1$restrictions$res2<-paste0(cn[i],".stir")
      sign.constr.eer$shock1$restrictions$res3<-paste0(cn[i],".reer")#  and real exchange rate up (since stir up and infl up)
      sign.constr.eer$shock1$restrictions$res4<-paste0(cn[i],".ur") # Blanchard and Galí (2010):  negative relationship of output and unemployment
      sign.constr.eer$shock1$sign<-c(">",">",">",">","<")
      sign.constr.eer$shock1$rest.horz<-c(1,1,1,1,1)
      sign.constr.eer$shock1$constr<-c("all","all","all","all","all")
      sign.constr.eer$shock1$scal<-0.01 #+1% increase
      sign.constr.eer$shock2$shock<-paste0(cn[i],".Dp.yoy") # Negative AS shock, inflation up
      sign.constr.eer$shock2$restrictions$res1<-paste0(cn[i],".gdp.m")
      sign.constr.eer$shock2$restrictions$res2<-paste0(cn[i],".stir") # interest rates up (since M/P P up, LM to the left)
      sign.constr.eer$shock2$restrictions$res3<-paste0(cn[i],".reer")#  and real exchange rate up (since stir up and infl up)
      sign.constr.eer$shock2$restrictions$res4<-paste0(cn[i],".ur") # unemployment and price level in same direction
      sign.constr.eer$shock2$sign<-c(">","<",">",">",">")
      sign.constr.eer$shock2$rest.horz<-c(1,1,1,1,1)
      sign.constr.eer$shock2$constr<-c("all","all","all","all","all")
      sign.constr.eer$shock2$scal<-1 #+1% increase
    }  
    names(sign.constr.eer)<-c("AD","AS")
    
    rotSpec<-list();rotSpec$Srots<-1;rotSpec$MaxTries<-10000
    irf.sign<-IRF(gvar.obj=model.ng.p2.mixed.ur.dpyoy.poilyoy,shockc=NULL,nhor=24,cpu=100,sign.constr=sign.constr.eer,rotSpec=rotSpec,full.post = TRUE)
    aux<-irf.sign$full.post[grepl(cn[i],dimnames(irf.sign$full.post)[[1]]),,,,drop=FALSE]
    save(aux,file=paste0("sign_ng_mixedW12_ur_dpyoy_poilyoy_",cn[i],".rda"))
  }
  


  # oil supply shock
oil.imp<-c("CN",EA,"JP","US","BR","CL","PE","KR","MY","PH","SG","IN","ZA","SE","TR",
           "BG","CZ","HR","HU","PL","RO","EE","LT","LV")
oil.exp<-c("ID","CA","MX","NO","RU","GB")


  sign.constr.eer<-list()
  sign.constr.eer$shock1$shock<-"OC.poil" # OIL supply shock,  oil price up
  sign.constr.eer$shock1$restrictions$res1<-"OC.qoil"#  quantity = output down
  sign.constr.eer$shock1$restrictions$res2<-paste0(oil.imp,".gdp.m") # oil importers' gdp down
  sign.constr.eer$shock1$sign<-c(">","<","<")
  sign.constr.eer$shock1$rest.horz<-c(1,1,1)
  sign.constr.eer$shock1$constr<-c("all","all","median")
  sign.constr.eer$shock1$scal<-1 #+1% increase
  sign.constr.eer$shock2$shock<-"OC.poil" # OIL demand shock, oil price up
  sign.constr.eer$shock2$restrictions$res1<-"OC.qoil" # quantity = ouput up
  #sign.constr.eer$shock2$restrictions$res2<-paste0(oil.imp,".gdp.m") # oil importers' gdp up
  #sign.constr.eer$shock2$restrictions$res3<-paste0(oil.exp,".gdp.m") #  oil exporters' gdp up
  sign.constr.eer$shock2$sign<-c(">",">")
  sign.constr.eer$shock2$rest.horz<-c(1,1)
  sign.constr.eer$shock2$constr<-c("all","all")
  sign.constr.eer$shock2$scal<-1 #+1% increase
  # 
  names(sign.constr.eer)<-c("oil.supply","oil.demand")
  
  rotSpec<-list();rotSpec$Srots<-1;rotSpec$MaxTries<-10000
  irf.sign.oil<-IRF(gvar.obj=model.ng.p2.mixed.ur.dpyoy,shockc=NULL,nhor=24,sign.constr=sign.constr.eer,rotSpec=rotSpec,cpu=1)
  save(irf.sign,file="irf_oil_supply.rda")



  # processing the IRFS and produce graphs----------------------
  source("aux_plot_script.R")
  
  cesee<-c("BG", "CZ", "HR", "HU", "PL", "RO", "RU", "TR")
  ea1<-c("AT", "BE", "DE",  "IT", "ES", "FI", "FR", "NL")
  ea2<-c("PT", "GR", "IE", "EE", "LT", "LV", "SK", "SI")
  asia<-c("CN", "JP", "KR", "MY", "PH", "SG", "ID", "IN")
  latam.za<-c("BR", "CL", "MX", "PE","ZA")
  advanced<-c("US", "CA", "NO", "SE", "GB")
  
  
  # read in data and stack to make plots
  irf.array<-list()
  for(i in 1:length(cc)){
    String<-paste0("sign_ng_mixedW12_ur_dpyoy_poilyoy_",cc[i],".rda")
    eval(parse(text= paste0("load(\"",(String),"\")")))
    irf.array[[i]]<-aux
    
  }
  names(irf.array)<-cc
  
  
  # normalization to 1% Dp.mom increase (for each estimation window separately due to different number of stable MCMC draws)
  for(i in 1:length(cc)){
    dimnames(irf.array[[i]])[[3]]<-c("AD","AS")
    for(z in 1:dim(irf.array[[i]])[4]){
      irf.array[[i]][,,"AD",z]<-irf.array[[i]][,,"AD",z]/irf.array[[i]][paste0(cc[i],".Dp.yoy"),1,"AD",z] # AD shock
      irf.array[[i]][,,"AS",z]<-irf.array[[i]][,,"AS",z]/irf.array[[i]][paste0(cc[i],".Dp.yoy"),1,"AS",z] # AS shock
    }
  }
  
  
  load("irf_poilyoy_supply.rda");irf.sign.poil.full<-irf.sign.poil
  
  library(gplots)
  
  imp.ad<-imp.ad[order(imp.ad[,2],decreasing=TRUE),]
  Col<-rep("steelblue3",nrow(imp.ad))
  sig1<-intersect(which(imp.ad[,1]<0),which(imp.ad[,3]<0))
  sig2<-intersect(which(imp.ad[,1]>0),which(imp.ad[,3]>0))
  Col[c(sig1,sig2)]<-"tomato"
  barplot2(imp.ad[,2],plot.ci=T,ci.l=imp.ad[,1],ci.u=imp.ad[,3],plot.grid=TRUE,las=2,col=Col)
  
  
  barplot(sort(imp.ad[,2],decreasing=FALSE),ylim=c(-0.2,1.5))
  barplot(sort(imp.as[,2],decreasing=FALSE),ylim=c(-6,1.0))
  
  
  
  
  
  bitmap("infl_ad.png","png16m",width=32,height=20,res=220)
  par(mfrow = c(7,6), oma = c(5,4,1,1) + 0.1, mar = c(2,2,1,1) + 0.1) 
  cc2<-c(advanced,ea1,ea2,cesee,asia,latam.za)
  
  for(i in 1:length(cc2)){
    idx<-which(names(irf.array)==cc2[i])
    irf.plot2(irf.array[[idx]],resp=paste0(cc2[i],".Dp.yoy"),shock.nr=1,Main=paste0(cc2[i]),cex.la=0.6)
  }
  
  dev.off()
  
  
  bitmap("infl_ad.pdf",width=32,height=20)
  par(mfrow = c(7,6), oma = c(5,4,1,1) + 0.1, mar = c(2,2,1,1) + 0.1) 
  for(i in 1:length(cc2)){
    idx<-which(names(irf.array)==cc2[i])
    irf.plot2(irf.array[[idx]],resp=paste0(cc2[i],".Dp.yoy"),shock.nr=1,Main=paste0(cc2[i]),cex.la=0.6)
  }
  dev.off()
  
  
  
  
  bitmap("infl_as.png","png16m",width=32,height=20,res=220)
  par(mfrow = c(7,6), oma = c(5,4,1,1) + 0.1, mar = c(2,2,1,1) + 0.1) 
  cc2<-c(advanced,ea1,ea2,cesee,asia,latam.za)
  
  for(i in 1:length(cc2)){
    idx<-which(names(irf.array)==cc2[i])
    irf.plot2(irf.array[[idx]],resp=paste0(cc2[i],".Dp.yoy"),shock.nr=2,Main=paste0(cc2[i]),cex.la=0.6)
  }
  
  dev.off()
  
  
  
  pdf("infl_as.pdf",width=32,height=20)
  par(mfrow = c(7,6), oma = c(5,4,1,1) + 0.1, mar = c(2,2,1,1) + 0.1) 
  cc2<-c(advanced,ea1,ea2,cesee,asia,latam.za)
  
  for(i in 1:length(cc2)){
    idx<-which(names(irf.array)==cc2[i])
    irf.plot2(irf.array[[idx]],resp=paste0(cc2[i],".Dp.yoy"),shock.nr=2,Main=paste0(cc2[i]),cex.la=0.6)
  }
  
  dev.off()
  
  
  
  bitmap("infl_oil.png","png16m",width=32,height=20,res=220)
  par(mfrow = c(7,6), oma = c(5,4,1,1) + 0.1, mar = c(2,2,1,1) + 0.1) 
  cc2<-c(advanced,ea1,ea2,cesee,asia,latam.za)
  for(i in 1:length(cc2)){
    irf.plot(irf.sign.poil.full,resp=paste0(cc2[i],".Dp.yoy"),Main=paste0(cc2[i]),cex.la=0.6)
  }
  dev.off()
  
  
  
  pdf("infl_oil.pdf",width=32,height=20)
  par(mfrow = c(7,6), oma = c(5,4,1,1) + 0.1, mar = c(2,2,1,1) + 0.1) 
  cc2<-c(advanced,ea1,ea2,cesee,asia,latam.za)
  for(i in 1:length(cc2)){
    irf.plot(irf.sign.poil.full,resp=paste0(cc2[i],".Dp.yoy"),Main=paste0(cc2[i]),cex.la=0.6)
  }
  dev.off()
  
  
  
  bitmap("gdp_ad.png","png16m",width=32,height=20,res=220)
  par(mfrow = c(7,6), oma = c(5,4,1,1) + 0.1, mar = c(2,2,1,1) + 0.1) 
  cc2<-c(advanced,ea1,ea2,cesee,asia,latam.za)
  
  for(i in 1:length(cc2)){
    idx<-which(names(irf.array)==cc2[i])
    irf.plot2(irf.array[[idx]],resp=paste0(cc2[i],".gdp.m"),shock.nr=1,Main=paste0(cc2[i]),cex.la=0.6)
  }
  
  dev.off()
  
  
  bitmap("gdp_as.png","png16m",width=32,height=20,res=220)
  par(mfrow = c(7,6), oma = c(5,4,1,1) + 0.1, mar = c(2,2,1,1) + 0.1) 
  cc2<-c(advanced,ea1,ea2,cesee,asia,latam.za)
  
  for(i in 1:length(cc2)){
    idx<-which(names(irf.array)==cc2[i])
    irf.plot2(irf.array[[idx]],resp=paste0(cc2[i],".gdp.m"),shock.nr=2,Main=paste0(cc2[i]),cex.la=0.6)
  }
  
  dev.off()
  
  
  bitmap("gdp_oil.png","png16m",width=32,height=20,res=220)
  par(mfrow = c(7,6), oma = c(5,4,1,1) + 0.1, mar = c(2,2,1,1) + 0.1) 
  cc2<-c(advanced,ea1,ea2,cesee,asia,latam.za)
  
  for(i in 1:length(cc2)){
    irf.plot(irf.sign.poil.full,resp=paste0(cc2[i],".gdp.m"),Main=paste0(cc2[i]),cex.la=0.6)
  }
  
  dev.off()
  
  
  